/* BOREXINO Reconstruction program
 *
 * Author: Serguei Soukhotin <sukhotin@cdf.in2p3.fr>
 *
 * $Id: bx_fadc_event.hh,v 1.4 2004/08/06 10:30:28 razeto Exp $
 *
 * The fadc event object
 * 09/07/2004 - 2 classes: bx_fadc_decoded_event & bx_fadc_decoded_window 
 * are implemented.
 */
 
#ifndef _BX_RAW_EVENT_HH
#define _BX_RAW_EVENT_HH

#include <vector>

#include "bx_base_event.hh"
#include "bx_rec_general.hh"
#include "constants.hh"


class bx_fadc_raw_channel {
  public:
    bx_fadc_raw_channel (const char *disk_channel, unsigned short nsamples);
    unsigned char  get_sample    (int i) const { return samples_v[i];   }
    unsigned short get_nsamples ()  const { return samples_v.size();   }
  private:
    std::vector<unsigned char> samples_v;
};

class bx_fadc_raw_window {
  public:
    bx_fadc_raw_window (const char *disk_window);
    unsigned long  get_win_length () const { return u4_entries;    }
    unsigned char  get_win_number () const { return u1_win_number; }
    unsigned char  get_win_flag   () const { return u1_win_flag;   }
    unsigned char  get_nchannels  () const { return u1_nchannels;  }
    signed short   get_win_begin  () const { return u2_win_begin;  }
    unsigned short get_win_size   () const { return u2_win_size;   }
    const bx_fadc_raw_channel& get_channel (int i) const { return channels_v[i]; }
    bx_fadc_raw_channel& get_channel (int i) { return channels_v[i]; }

  private:
    unsigned long  u4_entries;
    unsigned char  u1_win_number;
    unsigned char  u1_win_flag;
    unsigned char  u1_nchannels;
    unsigned short u2_reserved;
    signed   short u2_win_begin;
    unsigned short u2_win_size;
    std::vector<bx_fadc_raw_channel> channels_v;
};

class bx_fadc_raw_event {
  public:
    bx_fadc_raw_event (const char *disk_event);
    unsigned long  get_ev_number   () const { return u4_ev_number;   }
    unsigned long  get_ppc_time    () const { return u4_ppc_time;    }
    unsigned char  get_num_windows () const { return u1_num_windows; }
    unsigned char  get_flag_header () const { return u1_flag_header; }
    unsigned long  get_fadc_time   () const { return u4_fadc_time;   }
    const bx_fadc_raw_window& get_window (int i) const { return windows_v[i]; }
    bx_fadc_raw_window& get_window (int i) { return windows_v[i]; }

  private:
    unsigned long  u4_ev_number;
    unsigned long  u4_ppc_time;
    unsigned char  u1_num_windows;
    unsigned char  u1_flag_header;
    unsigned long  u4_fadc_time;
    std::vector<bx_fadc_raw_window> windows_v;
};
//==============================================================================
class bx_fadc_decoded_window {
  public:
    unsigned char  get_dwin_number () const { return u1_win_number; }     
    unsigned char  get_dwin_flag   () const { return u1_win_flag;   }
    signed short   get_dwin_begin  () const { return u2_win_begin;  }
    unsigned short get_dwin_size   () const { return u2_win_size;   }
    float          get_sum_ampl    () const { return f4_sum_ampl;   }
    float          get_sum_charge  () const { return f4_sum_charge; }
    float          get_sum_peak    () const { return f4_sum_peak;   }    
    float get_channel_ampl    (int i) const { return channels_ampl[i];   }
    float get_channel_charge  (int i) const { return channels_charge[i]; } 
    float get_channel_peak    (int i) const { return channels_peak[i];   }
    float get_sum_sample      (int i) const { return sum_samples[i];     }    

  private:
    unsigned char  u1_win_number;
    unsigned char  u1_win_flag;
    signed   short u2_win_begin;
    unsigned short u2_win_size;
    float          f4_sum_ampl, f4_sum_charge, f4_sum_peak;
    std::vector<float> channels_ampl;
    std::vector<float> channels_charge;
    std::vector<float> channels_peak;
    std::vector<float> sum_samples;

    friend class bx_fadc_decoder;
};

class bx_fadc_decoded_event {
  public:
    bx_fadc_decoded_event ();
    virtual ~bx_fadc_decoded_event () {}
    
    unsigned char  get_dnum_windows     () const { return u1_num_windows;       }
    unsigned char  get_event_flag       () const { return u1_event_flag;        }
    float          get_time_to_previous () const { return f4_time_to_previous;  }
    float          get_trigger_time     () const { return f4_trigger_time;      }
    const bx_fadc_decoded_window& get_decoded_window (int i) const { return decoded_win[i]; }
    bx_fadc_decoded_window& get_decoded_window       (int i)  { return decoded_win[i]; }
  private:
    unsigned char  u1_num_windows;
    unsigned char  u1_event_flag;
    float          f4_time_to_previous;
    float          f4_trigger_time;
    std::vector<bx_fadc_decoded_window> decoded_win;

    friend class bx_fadc_decoder;
};

class bx_fadc_event:
  public bx_base_event,
  public bx_fadc_raw_event,
  public bx_fadc_decoded_event {
  public:
    bx_fadc_event (const char *disk_event): bx_fadc_raw_event(disk_event) { mark_stage (bx_base_event::raw); }
    virtual ~bx_fadc_event () {}
  private:
    void mark_stage(event_stage es, bool v = true) { stages[es] = v; }

  friend class bx_fadc_decoder;
};

#endif

/* 
 * $Log: bx_fadc_event.hh,v $
 * Revision 1.4  2004/08/06 10:30:28  razeto
 * cycle_1 branch merged in the main trunk.
 *
 * Revision 1.3.2.1  2004/07/26 17:58:16  ddangelo
 * added raw event mark stage
 *
 * Revision 1.3  2004/07/09 08:53:30  sukhotin
 * Classes bx_fadc_decoded_event and bx_fadc_decoded window are implemented.
 * BX_FADC_DECODER module is a new module to decode raw fadc event
 * FADC_TRIGGER_TIME and FADC_CHANNEL are the classes to decode a trigger ramp
 * and raw channel data.
 *
 * Revision 1.2  2004/05/27 16:16:50  ddangelo
 * added support for event stage marking
 *
 * Revision 1.1  2004/05/27 14:46:59  ddangelo
 * added fadc event class (raw event for now), by Sergio
 *
 */
