/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 *
 * $Id: bx_global_event.hh,v 1.1 2004/07/12 10:28:14 ddangelo Exp $
 *
 * The global event object
 * Info stored in this objects are not related to a specific detector segment 
 * but are meant to be high level compound info
 * 
 */
#ifndef _BX_GLOBAL_EVENT_HH
#define _BX_GLOBAL_EVENT_HH

#include <vector>

#include "bx_rec_general.hh"
#include "bx_base_event.hh"
#include "constants.hh"


// ------- SPLIT --------- //
class bx_laben_clustered_hit;
class bx_laben_cluster;
class bx_global_split_hit {
  public:
    bx_global_split_hit (const bx_laben_clustered_hit &hit): p_clustered_hit(&hit) {} // ctor
    virtual ~bx_global_split_hit () {} // dtor
    const bx_laben_clustered_hit& get_clustered_hit () const { return *p_clustered_hit; } // leftward getter

    float get_my_variable () const { return f4_my_variable; }

  private:
    float f4_my_variable;
    const bx_laben_clustered_hit* p_clustered_hit;

  public:
    typedef std::vector<bx_global_split_hit> bx_global_split_hit_vector;
    friend class bx_global_splitter;
};


class bx_global_split_fragment {
  public:
    bx_global_split_fragment (const bx_laben_cluster &cluster) : p_cluster(&cluster) { split_hits.clear(); } // ctor
    virtual ~bx_global_split_fragment () {} // dtor
    const bx_laben_cluster& get_cluster () const { return *p_cluster; } // leftward getter
    const bx_global_split_hit& get_split_hit (int i) const { return split_hits[i]; } // upward getter
    int get_split_nhits () const { return split_hits.size (); }

    float get_my_variable () const { return f4_my_variable; }

  private:
    float f4_my_variable;
    const bx_laben_cluster* p_cluster;
    bx_global_split_hit::bx_global_split_hit_vector split_hits;

  public:
    typedef std::vector<bx_global_split_fragment> bx_global_split_fragment_vector;
    friend class bx_global_splitter;
};

class bx_global_split_event {
  public:
    bx_global_split_event () { split_fragments.clear(); } // ctor
    virtual ~bx_global_split_event () {} // dtor

    const bx_global_split_fragment& get_split_fragment (int i) const { return split_fragments[i]; } // upward getter
    int get_split_nfragments () const { return split_fragments.size (); }

  private:
    bx_global_split_fragment::bx_global_split_fragment_vector split_fragments;

    friend class bx_global_splitter;
};


// ------- REC --------- //
class bx_global_rec_hit {
  public:
    bx_global_rec_hit (const bx_global_split_hit &hit): p_split_hit(&hit) {}
    const bx_global_split_hit& get_split_hit () const { return *p_split_hit; }

    float get_my_variable () const { return f4_my_variable; }

  private:
    float f4_my_variable;
    const bx_global_split_hit* p_split_hit;

  public:
    typedef std::vector<bx_global_rec_hit> bx_global_rec_hit_vector;
    friend class bx_milan_reconstucter;
    friend class bx_moscow_reconstucter;
};


class bx_global_rec_fragment {
  public:
    bx_global_rec_fragment (const bx_global_split_fragment &fragment): p_split_fragment(&fragment) { rec_hits.clear(); } // ctor
    virtual ~bx_global_rec_fragment () {} // dtor
    const bx_global_split_fragment& get_split_fragment () const { return *p_split_fragment; } // leftward getter
    const bx_global_rec_hit& get_rec_hit (int i) const { return rec_hits[i]; } // upward getter
    int get_rec_nhits () const { return rec_hits.size (); }

    float get_my_variable () const { return f4_my_variable; }

  private:
    float f4_my_variable;
    const bx_global_split_fragment* p_split_fragment;
    bx_global_rec_hit::bx_global_rec_hit_vector rec_hits;

  public:
    typedef std::vector<bx_global_rec_fragment> bx_global_rec_fragment_vector;
    friend class bx_milan_reconstucter;
    friend class bx_moscow_reconstucter;
};

class bx_global_rec_event {
  public:
    bx_global_rec_event () { rec_fragments.clear(); } // ctor
    virtual ~bx_global_rec_event () {} // dtor

    const bx_global_rec_fragment& get_rec_fragment (int i) const { return rec_fragments[i]; } // upward getter
    int get_rec_nfragments () const { return rec_fragments.size (); }

  private:
    bx_global_rec_fragment::bx_global_rec_fragment_vector rec_fragments;

    friend class bx_milan_reconstucter;
    friend class bx_moscow_reconstucter;
};


// ------- PSD --------- //
// no more hits in this level
class bx_global_psd_fragment {
  public:
    bx_global_psd_fragment (const bx_global_rec_fragment &fragment): p_rec_fragment(&fragment) {} // ctor
    virtual ~bx_global_psd_fragment () {} // dtor
    const bx_global_rec_fragment& get_rec_fragment () const { return *p_rec_fragment; } // leftward getter

    float get_my_variable () const { return f4_my_variable; }

  private:
    float f4_my_variable;
    const bx_global_rec_fragment* p_rec_fragment;

  public:
    typedef std::vector<bx_global_psd_fragment> bx_global_psd_fragment_vector;
    friend class bx_discriminator;
};

class bx_global_psd_event {
  public:
    bx_global_psd_event () { psd_fragments.clear(); } // ctor
    virtual ~bx_global_psd_event () {} // dtor

    const bx_global_psd_fragment& get_psd_fragment (int i) const { return psd_fragments[i]; } // upward getter
    int get_psd_nfragments () const { return psd_fragments.size (); }

  private:
    bx_global_psd_fragment::bx_global_psd_fragment_vector psd_fragments;

    friend class bx_discriminator;
};



// ------- derived general global class --------- //
class bx_global_event : 
  public bx_base_event,
  public bx_global_split_event,
  public bx_global_rec_event,
  public bx_global_psd_event {
public:
  bx_global_event () {}
  virtual ~bx_global_event () {}
  private:
    void mark_stage(event_stage es, bool v = true) { stages[es] = v; }

  friend class bx_global_splitter;
};

#endif
/*
 * $Log: bx_global_event.hh,v $
 * Revision 1.1  2004/07/12 10:28:14  ddangelo
 * added bx_global_event
 *
 *
 */
