/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 * based on work by Razeto&Pallas 
 *
 * $Id: bx_laben_event.hh,v 1.19 2004/09/13 12:13:06 pallas Exp $
 *
 * The laben event object
 * 
 */
#ifndef _BX_LABEN_EVENT_HH
#define _BX_LABEN_EVENT_HH

#include <vector>

#include "bx_rec_general.hh"
#include "bx_base_event.hh"
#include "constants.hh"


// ------- RAW --------- //
class bx_laben_raw_hit {
  public:
    bx_laben_raw_hit (const char *disk_hit);
    unsigned short get_logical_channel ()  const { return u2_channel; }
    unsigned char  get_time_1          ()  const { return u1_time_1; }
    unsigned char  get_time_2          ()  const { return u1_time_2; }
    unsigned short get_gray_counter    ()  const { return u2_gray_counter; }
    unsigned char  get_base            ()  const { return u1_base; }
    unsigned char  get_peak            ()  const { return u1_peak; }
    unsigned short get_flags           ()  const { return u2_flags; }
    unsigned short get_errors          ()  const { return u2_errors; }
  private:
    unsigned short u2_channel;
    unsigned char  u1_time_1, u1_time_2;
    unsigned short u2_gray_counter;
    unsigned char  u1_base, u1_peak;
    unsigned short u2_flags;
    unsigned short u2_errors;
  public:
    typedef std::vector<bx_laben_raw_hit> bx_laben_raw_hit_vector;
};

class bx_laben_raw_event {
  public:
    bx_laben_raw_event (const char *disk_event);
    virtual ~bx_laben_raw_event () {}
    const bx_laben_raw_hit& get_raw_hit (int i) const { return raw_hits[i]; }
    int get_raw_nhits () const { return raw_hits.size (); }
  private:
    unsigned long u4_errors;
    bx_laben_raw_hit::bx_laben_raw_hit_vector raw_hits;
};


// ------- DECODED --------- //
class db_channel;
class laben_time_hit;
class laben_charge_hit;
class bx_laben_decoded_hit {
  public:
    bx_laben_decoded_hit (const bx_laben_raw_hit &hit): b_out_of_gate(false), raw_hit(&hit) {}
    const bx_laben_raw_hit& get_raw_hit () const { return *raw_hit; }

      // TDC data from the laben_hit class
    double get_raw_time  ()                  const { return f8_raw_time; } // Relative to a gray counter crossing window
    float get_d80        ()                  const { return f4_d80; }
    float get_time_error ()                  const { return f4_time_error; }
    bool is_good         (float error = .45) const { return bool(f4_time_error <= error); }
    bool is_out_of_gate  ()                  const { return b_out_of_gate; } 
    bool operator< (const bx_laben_decoded_hit& hit) const { return get_raw_time () < hit.get_raw_time (); }
	
      // ADC data
    float get_charge_bin ()             const { return f4_charge_bin; }
    float get_uncorrected_charge_bin () const { return f4_uncorrected_charge_bin; }
    float get_charge_pe ()              const { return f4_charge_pe; }
    float get_uncorrected_charge_pe ()  const { return f4_uncorrected_charge_pe; }

    const db_channel* get_db_channel () const { return p_db_channel; }

  private:
      // From laben_time_hit
    double f8_raw_time;
    float f4_d80, f4_time_error;
      // From laben_charge_hit
    float f4_charge_bin, f4_uncorrected_charge_bin;
    float f4_charge_pe, f4_uncorrected_charge_pe;
      // From laben_decoder
    bool b_out_of_gate;
    const bx_laben_raw_hit* raw_hit;
    const db_channel *p_db_channel;

    const bx_laben_decoded_hit& operator= (laben_time_hit& t_hit);
    const bx_laben_decoded_hit& operator= (laben_charge_hit& c_hit);

  public:
    typedef std::vector<bx_laben_decoded_hit> bx_laben_decoded_hit_vector;
    friend class bx_laben_decoder;
};

class bx_laben_decoded_event {
  public:
    bx_laben_decoded_event ();
    virtual ~bx_laben_decoded_event () {}

    const bx_laben_decoded_hit& get_decoded_hit (int i) const { return decoded_hits[i]; }
    int get_decoded_nhits () const { return decoded_hits.size (); }
    
    double get_trigger_rawt () const { return f8_trigger_rawt; }
    int get_trigger_reference_decoded_nhits () const { return trigger_reference_decoded_hits.size (); }
    const bx_laben_decoded_hit& get_trigger_reference_decoded_hit (int i) const { return trigger_reference_decoded_hits[i]; }

    double get_laser_rawt () const { return f8_laser_rawt; }
    int get_laser_reference_decoded_nhits () const { return laser_reference_decoded_hits.size (); }
    const bx_laben_decoded_hit& get_laser_reference_decoded_hit (int i) const { return laser_reference_decoded_hits[i]; }
  private:
    bx_laben_decoded_hit::bx_laben_decoded_hit_vector decoded_hits, trigger_reference_decoded_hits, laser_reference_decoded_hits;
    double f8_trigger_rawt, f8_laser_rawt;
    friend class bx_laben_decoder;
};


// ------- CLUSTERED --------- //
class bx_laben_clustered_hit {
  public:
    bx_laben_clustered_hit (const bx_laben_decoded_hit &hit) : f8_time(0.), decoded_hit(&hit) {}
    const bx_laben_decoded_hit& get_decoded_hit () const { return *decoded_hit; }

    double get_time () const { return f8_time;}

  private:
    double f8_time;
    const bx_laben_decoded_hit* decoded_hit;
    
  public:
    typedef std::vector<bx_laben_clustered_hit> bx_laben_clustered_hit_vector;
    friend class bx_laben_findcluster;
    friend class bx_laben_findcluster_time;
};


class bx_laben_cluster {
  public:
    bx_laben_cluster ();
    virtual ~bx_laben_cluster () {}
    const bx_laben_clustered_hit& get_clustered_hit (int i) const { return clustered_hits[i]; }
    int get_clustered_nhits () const { return clustered_hits.size (); }

    int get_npe () const { return i4_npe; }
    int get_npmt () const { return i4_npmt; }
    int get_npmt_conc () const { return i4_npmt_conc; }
    float get_start_time () const { return f4_start_time; }
    float get_mean_time () const { return f4_mean_time; }
    
  private:
    int i4_npe;
    int i4_npmt;
    int i4_npmt_conc;
    float f4_start_time;
    float f4_mean_time;
    bx_laben_clustered_hit::bx_laben_clustered_hit_vector clustered_hits;

  public:
    typedef std::vector<bx_laben_cluster> bx_laben_cluster_vector;
    friend class bx_laben_findcluster;
    friend class bx_laben_findcluster_time;
};

class bx_laben_clustered_event {
  public:
    bx_laben_clustered_event ();
    virtual ~bx_laben_clustered_event () {}

    const bx_laben_cluster& get_cluster (int i) const { return clusters[i]; }
    int get_nclusters () const { return clusters.size (); }

  private:
    bx_laben_cluster::bx_laben_cluster_vector clusters;

    friend class bx_laben_findcluster;
    friend class bx_laben_findcluster_time;
};



// ------- derived general laben class --------- //
class bx_laben_event : 
  public bx_base_event,
  public bx_laben_raw_event,
  public bx_laben_decoded_event,
  public bx_laben_clustered_event {
public:
  bx_laben_event (const char *disk_event): bx_laben_raw_event(disk_event) { if (get_raw_nhits ()) mark_stage (bx_base_event::raw); }
  virtual ~bx_laben_event () {}
  private:
    void mark_stage(event_stage es, bool v = true) { stages[es] = v; }

  friend class bx_laben_decoder;
  friend class bx_laben_findcluster;
  friend class bx_laben_findcluster_time;
};

#endif
/*
 * $Log: bx_laben_event.hh,v $
 * Revision 1.19  2004/09/13 12:13:06  pallas
 * Changed the time references hits handling (by Alessandro)
 *
 * Revision 1.18  2004/08/31 13:29:03  ddangelo
 * added charge to laben hit
 *
 * Revision 1.17  2004/07/24 16:24:21  ddangelo
 * debugging (patch by Alessandro)
 *
 * Revision 1.16  2004/07/09 13:53:53  ddangelo
 * added friend declaration in bx_laben_event to allow mark stage by clustering modules
 *
 * Revision 1.15  2004/07/07 12:56:18  ddangelo
 * laben cluster and relative hit classes filled with meaningful variables (by Daniela)
 *
 * Revision 1.14  2004/06/07 17:17:09  ddangelo
 * b_is_valid removed from decoded hit.
 * time renamed to raw_time. getter renamed accordingly.
 *
 * Revision 1.13  2004/06/07 11:06:11  ddangelo
 * added operator< for decoded hit (by Alessandro)
 *
 * Revision 1.12  2004/06/01 18:23:40  ddangelo
 * applied patch by Alessandro
 * in bx_laben_decoded_event:
 * vectors of new private sub class time_reference_elements
 * + other upgrades
 *
 * Revision 1.11  2004/05/31 16:48:01  ddangelo
 * non-english name 'clusterized' replaced by 'clustered'
 *
 * Revision 1.10  2004/05/31 14:16:04  ddangelo
 * added clusterized level classes (still empty)
 *
 * Revision 1.9  2004/05/31 13:39:16  ddangelo
 * applied patch with 2 bx_laben_decoded_hit::operator=() (by Alessandro)
 *
 * Revision 1.8  2004/05/27 14:54:14  ddangelo
 *  removed useless friend declarations
 *
 * Revision 1.7  2004/05/27 14:52:30  ddangelo
 * p_db_channel changed to const
 *
 * Revision 1.6  2004/05/25 16:37:05  ddangelo
 * added support for event stage flagging.
 * Inheritance from bx_base_event introduced
 *
 * Revision 1.5  2004/05/21 11:11:51  ddangelo
 * some names updated for consistency
 *
 * Revision 1.4  2004/05/20 18:32:26  ddangelo
 * added support for db_channel
 *
 * Revision 1.3  2004/05/20 12:55:46  ddangelo
 * applied patch by Alessandro.
 * classes bx_laben_decoded_hit and bx_laben_decoded_event introduced.
 *
 * Revision 1.2  2004/04/27 16:58:57  ddangelo
 * minor updates
 *
 * Revision 1.1  2004/04/27 09:41:09  ddangelo
 * Event structure redisegned.
 * bx_reco_event now handles event header and has object for different detector portions.
 * Inheritance from different reconstruction status is kept within each detector portion obj.
 * Hit lists are linked by const ptr (maybe a temporary solution).
 *
 *
 */
