/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 *
 * $Id: bx_muon_event.cc,v 1.5 2004/06/07 12:47:34 ddangelo Exp $
 *
 * Implementation of bx_muon_event.hh
 *
 */

#include "bx_muon_event.hh"
#include "bx_event_disk_format.h"
#include "messenger.hh"

bx_muon_raw_hit::bx_muon_raw_hit (const bx_muon_edge& first_edge, const bx_muon_edge& second_edge, unsigned char board) : lead_edge(first_edge), 
															  trail_edge(second_edge), 
															  u1_tdc_board(board) {
}

// Performs preliminary checks on the edge couple. Returns 0 on error.
bx_muon_raw_event::check_t bx_muon_raw_event::m_check_validity (const bx_muon_edge& el, const bx_muon_edge& et) const {
  if (el.slope  == et.slope || el.tdc_chip != et.tdc_chip || el.tdc_channel != et.tdc_channel )
    return missing_edge;
  if (el.time  < et.time )
    return inverted;
  return fine;
}
  
bx_muon_raw_event::bx_muon_raw_event (const char *disk_event) {
  muon_header_disk_format *head = (muon_header_disk_format *)disk_event;
  if (head->nwords == 0) return;

  u4_nedges = head->nwords - sizeof (muon_header_disk_format) / sizeof (unsigned long);
  u4_trgid = head->trgid;
  b_error_flag = bool (head->error_flag);

  raw_hits.clear ();
  unsigned char chip = 0;
  for (unsigned long i = 0; i < u4_nedges; i++) {
    unsigned char board = chip / constants::muon::tdc::chips_per_board;
    bx_muon_edge *el = (bx_muon_edge*)(disk_event + sizeof (muon_header_disk_format) + i * sizeof (muon_edge_disk_format));
    if ( el->is_last ) chip++;
    if ( el->is_event_number || el->is_invalid ) continue;

    bx_muon_edge *et = (bx_muon_edge*)(disk_event + sizeof (muon_header_disk_format) + (++i) * sizeof (muon_edge_disk_format));
    if ( et->is_last ) chip++;
    if ( et->is_event_number || et->is_invalid ) continue;

    switch (m_check_validity(*el, *et)) {
    case fine:
      raw_hits.push_back (bx_muon_raw_hit(*el, *et, board));
      if ( ( (board * constants::muon::tdc::chips_per_board + el->tdc_chip) * constants::muon::tdc::channels_per_chip + el->tdc_channel ) > 255 ) {
	bx_message msg(bx_message::error, "bx_muon_raw_event::bx_muon_raw_event() ");
	msg << "Invalid muon channel. brd " << board << "; chip " << int(el->tdc_chip) << " ; chann " << int(el->tdc_channel) << dispatch;
      }
      break;
    case missing_edge: {
      bx_message msg(bx_message::debug, "bx_muon_raw_event: ");
      msg << "event " << u4_trgid << "; mch " << (board * constants::muon::tdc::chips_per_board + el->tdc_chip) * constants::muon::tdc::channels_per_chip + el->tdc_channel 
      	  << "; missing edge" << dispatch;
      i--;
      if ( et->is_last ) chip--; // to correct double counting    
      break;
    }
    case inverted: {
      bx_message msg(bx_message::debug, "bx_muon_raw_event: ");
      msg << "event " << u4_trgid << "; mch " << (board * constants::muon::tdc::chips_per_board + el->tdc_chip) * constants::muon::tdc::channels_per_chip + el->tdc_channel 
      	  << "; inverted time ordering; lead " << el->time << "; trail " << et->time << ";\n" << dispatch;
      break;
    }
    }    
  } // end of loop on edge pairs
    
  if (chip != (constants::muon::tdc::max_boards * constants::muon::tdc::chips_per_board)) {
    bx_message msg(bx_message::critic, "bx_muon_raw_event: ");
    msg << "Inconsitent muon data. Found tdc chips #" << int(chip) << dispatch;
  }
}

/*
 * $Log: bx_muon_event.cc,v $
 * Revision 1.5  2004/06/07 12:47:34  ddangelo
 * fixed some messages
 *
 * Revision 1.4  2004/06/01 11:38:23  ddangelo
 * updated to new constants syntax
 *
 * Revision 1.3  2004/05/20 18:17:26  ddangelo
 * low level data containers simplified.
 * class bx_muon_edge changed to a bitfield.
 * removed static masks and shifts for old data parsing.
 * raw hit construction directly into hit vector.
 * check_validity moved from hit to event class.
 *
 * Revision 1.1  2004/04/27 09:41:09  ddangelo
 * Event structure redisegned.
 * bx_reco_event now handles event header and has object for different detector portions.
 * Inheritance from different reconstruction status is kept within each detector portion obj.
 * Hit lists are linked by const ptr (maybe a temporary solution).
 *
 *
 */
