/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 *
 * $Id: bx_muon_event.hh,v 1.11 2004/07/25 13:31:14 ddangelo Exp $
 *
 * The muon event object
 * 
 */
#ifndef _BX_MUON_EVENT_HH
#define _BX_MUON_EVENT_HH

#include <vector>

#include "bx_rec_general.hh"
#include "bx_base_event.hh"
#include "constants.hh"

// ****** raw *************/
// bitfield to parse tdc word (32 bits)
struct bx_muon_edge {
  unsigned short time : 16;
  unsigned char slope : 1;
  bool is_overflow : 1;
  unsigned char tdc_channel : 5; 
  bool is_event_number : 1;
  unsigned char tdc_chip : 2;
  int : 4; // this bits are reserved by CAEN in the tdc word
  bool is_last : 1;
  bool is_invalid : 1;
};


class bx_muon_raw_hit {
  public:
    enum polarity_t {negative=-1, positive=+1}; 

    bx_muon_raw_hit (const bx_muon_edge& lead, const bx_muon_edge& trail, unsigned char board);
 
    unsigned short get_muon_channel    ()  const { return (u1_tdc_board * constants::muon::tdc::chips_per_board + lead_edge.tdc_chip) * constants::muon::tdc::channels_per_chip + lead_edge.tdc_channel; } // 0 based
    unsigned short get_logical_channel ()  const { return get_muon_channel () + constants::muon::channel_offset + 1; } // 1 based
    unsigned long  get_lead_time        () const {return lead_edge.time; }
    unsigned long  get_trail_time       () const {return trail_edge.time; }
    unsigned long  get_time_diff        () const {return lead_edge.time - trail_edge.time; }
    polarity_t     get_polarity         () const {return lead_edge.slope ? negative : positive; }

  private:
    bx_muon_edge lead_edge;
    bx_muon_edge trail_edge;
    unsigned char u1_tdc_board;
};
   

class bx_muon_raw_event {
  public:
    typedef std::vector<bx_muon_raw_hit> bx_muon_raw_hit_vector;

    bx_muon_raw_event (const char *disk_event);
    virtual ~bx_muon_raw_event () {}
    const bx_muon_raw_hit& get_raw_hit (int i) const { return raw_hits[i]; }
    unsigned long get_raw_nhits    ()      const {return raw_hits.size();}
    const bx_muon_raw_hit_vector&  get_raw_hits    ()      const {return raw_hits;}
    unsigned long get_nedges       ()      const {return u4_nedges;}
    unsigned long get_trgid        ()      const {return u4_trgid;}
    bool get_error_flag            ()      const {return b_error_flag;}

  private:
    enum check_t {fine, missing_edge, inverted};
    check_t m_check_validity  (const bx_muon_edge&, const bx_muon_edge&) const;

    unsigned long u4_nedges;
    unsigned long u4_trgid;
    bool b_error_flag;
    bx_muon_raw_hit_vector raw_hits;
};

/********** decoded *************/
class db_channel;
class bx_muon_decoded_hit {
  public:
    bx_muon_decoded_hit ( float time, float charge, const bx_muon_raw_hit* raw_hit_ptr) : f4_time(time), f4_charge(charge), raw_hit(raw_hit_ptr) {}
    const bx_muon_raw_hit* get_raw_hit () const {return raw_hit;}

    float get_time          () const { return f4_time; }
    float get_charge        () const { return f4_charge; }
    const db_channel* get_db_channel () const { return p_db_channel; }
  
  private:
    float f4_time, f4_charge;
    const bx_muon_raw_hit* raw_hit;
    const db_channel* p_db_channel;

}; 

class bx_muon_decoded_event {
  public:
    bx_muon_decoded_event () {}
    virtual ~bx_muon_decoded_event () {}

    const bx_muon_decoded_hit& get_decoded_hit (int i) const { return decoded_hits[i]; }
    unsigned long get_decoded_nhits ()            const { return decoded_hits.size(); }
    unsigned long get_decoded_nhits (int channel) const { return nhits_per_channel[channel]; }
    
  private:
    unsigned long nhits_per_channel[256]; // to be replaced as soon as constants is improved
    typedef std::vector<bx_muon_decoded_hit> bx_muon_decoded_hit_vector;
    bx_muon_decoded_hit_vector decoded_hits;

  friend class bx_muon_decoder;
};

class bx_muon_event: 
  public bx_base_event,
  public bx_muon_raw_event, 
  public bx_muon_decoded_event {
  public:
    bx_muon_event (const char *disk_event): bx_muon_raw_event(disk_event) { if (get_raw_nhits ()) mark_stage (bx_base_event::raw); }
    virtual ~bx_muon_event () {}

  private:
    void mark_stage(event_stage es, bool v = true) { stages[es] = v; }

  friend class bx_muon_decoder;
};

#endif
/*
 * $Log: bx_muon_event.hh,v $
 * Revision 1.11  2004/07/25 13:31:14  ddangelo
 * added mark stage for raw event
 *
 * Revision 1.10  2004/06/01 11:38:23  ddangelo
 * updated to new constants syntax
 *
 * Revision 1.9  2004/05/27 14:54:14  ddangelo
 *  removed useless friend declarations
 *
 * Revision 1.8  2004/05/27 14:52:30  ddangelo
 * p_db_channel changed to const
 *
 * Revision 1.7  2004/05/25 16:37:05  ddangelo
 * added support for event stage flagging.
 * Inheritance from bx_base_event introduced
 *
 * Revision 1.6  2004/05/21 11:11:51  ddangelo
 * some names updated for consistency
 *
 * Revision 1.5  2004/05/20 18:32:26  ddangelo
 * added support for db_channel
 *
 * Revision 1.4  2004/05/20 18:17:26  ddangelo
 * low level data containers simplified.
 * class bx_muon_edge changed to a bitfield.
 * removed static masks and shifts for old data parsing.
 * raw hit construction directly into hit vector.
 * check_validity moved from hit to event class.
 *
 * Revision 1.2  2004/05/20 11:12:15  ddangelo
 * added a getter for the hit vector (required by algorythm use)
 *
 * Revision 1.1  2004/04/27 09:41:09  ddangelo
 * Event structure redisegned.
 * bx_reco_event now handles event header and has object for different detector portions.
 * Inheritance from different reconstruction status is kept within each detector portion obj.
 * Hit lists are linked by const ptr (maybe a temporary solution).
 *
 *
 */
