/* BOREXINO Reconstruction program
 *
 * Author: Marco Pallavicini <pallas@ge.infn.it>
 *
 * $Id: bx_trigger_event.hh,v 1.7 2004/05/25 21:46:10 ddangelo Exp $
 *
 * The trigger event object
 * 
 */
#ifndef _BX_TRIGGER_EVENT_HH
#define _BX_TRIGGER_EVENT_HH

#include "bx_rec_general.hh"
#include "bx_base_event.hh"
#include "constants.hh"

// bitfield to parse btb MSB of trigger word (8 bits)
struct bx_btb_inputs {
  char           : 2; // BTB input #1 & #2: currently unused
  bool mtb_flag  : 1; // BTB input #3 muon MTB
  bool tct_flag  : 1; // BTB input #4 muon analog 
  bool l355_flag : 1; // BTB input #5 laser355 
  bool l266_flag : 1; // BTB input #6 laser266 
  bool lcr_flag  : 1; // BTB input #7 common to laser394, calibration and random
  char           : 1; // BTB input #8: currently unused
};


class bx_trigger_raw_event {
  public:
    bx_trigger_raw_event (const char *disk_trigger_event);

    unsigned short get_length          () const { return u2_length; }
    unsigned short get_error           () const { return u2_error; }    
    unsigned short get_blocks          () const { return u2_blocks; }
    unsigned short get_version         () const { return u2_version; } 
    unsigned long  get_dsp_registers   () const { return u4_dsp_registers; }
    unsigned short get_btb_threshold   () const { return u2_btb_threshold; }
    unsigned short get_btb_firmware    () const { return u2_btb_firmware; }
    unsigned long  get_trg_time        () const { return u4_trg_time; }
    unsigned long  get_evid            () const { return u4_evid; }
    unsigned short get_tab_sum         () const { return u2_tab_sum; }
    bool           has_mtb_flag        () const { return u1_btb_inputs.mtb_flag; }
    bool           has_tct_flag        () const { return u1_btb_inputs.tct_flag; }
    bool           has_l355_flag       () const { return u1_btb_inputs.l355_flag; }
    bool           has_l266_flag       () const { return u1_btb_inputs.l266_flag; }
    bool           has_lcr_flag        () const { return u1_btb_inputs.lcr_flag; }
    bool           has_comparator_only () const { return !(has_mtb_flag() || 
		    				 	   has_tct_flag() || 
						 	   has_l266_flag() || 
						           has_l355_flag() || 
						           has_lcr_flag()); }
    unsigned char  get_trgtype         () const { return u1_trgtype; }
    unsigned long  get_gps1            () const { return u4_gps1; }
    unsigned long  get_gps2            () const { return u4_gps2; }
    unsigned long  get_gps3            () const { return u4_gps3; }

  private:
    unsigned short u2_length;
    unsigned short u2_error;
    unsigned short u2_blocks;
    unsigned short u2_version;
    unsigned long  u4_dsp_registers;
    unsigned short u2_btb_firmware;
    unsigned short u2_btb_threshold;
    unsigned long  u4_trg_time;
    unsigned long  u4_evid;
    unsigned char  u1_trgtype;
    bx_btb_inputs  u1_btb_inputs;
    unsigned short u2_tab_sum;
    unsigned long  u4_trgw;
    unsigned long  u4_gps1;
    unsigned long  u4_gps2;
    unsigned long  u4_gps3;
};

class bx_trigger_decoded_event {
  public:
    bx_trigger_decoded_event () {}
   
    bool is_neutrino () const { return b_is_neutrino; }
    bool is_muon_mtb () const { return b_is_muon_mtb; }
    bool is_muon_tct () const { return b_is_muon_tct; }
    bool is_laser266 () const { return b_is_laser266; }
    bool is_laser355 () const { return b_is_laser355; }
    bool is_laser394 () const { return b_is_laser394; }
    bool is_pulser   () const { return b_is_pulser;   }
    bool is_random   () const { return b_is_random;   }

    // getters
    int get_day      () const { return i4_day; }
    int get_mon      () const { return i4_mon; }
    int get_year     () const { return i4_year; }
    int get_hour     () const { return i4_hour; }
    int get_min      () const { return i4_min; }
    int get_sec      () const { return i4_sec; }
    int get_millisec () const { return i4_millisec; }
    int get_microsec () const { return i4_microsec; }

    // return event time in seconds, and microseconds since Jan 01, 2000 0:0:0;
    void get_gps_time    (unsigned long &secs, unsigned long &micro) const {secs=i4_time1;  micro=i4_time2; }

  private:
    int i4_day, i4_mon, i4_year;
    int i4_hour, i4_min, i4_sec, i4_millisec, i4_microsec;
    unsigned long i4_time1, i4_time2;  // absolute secs and microsecs since 01/01/200 

    bool b_is_neutrino;
    bool b_is_muon_mtb;
    bool b_is_muon_tct;
    bool b_is_laser266;
    bool b_is_laser355;
    bool b_is_laser394;
    bool b_is_pulser;
    bool b_is_random;

  friend class bx_trigger_decoder;
};

class bx_trigger_event: 
  public bx_base_event,
  public bx_trigger_raw_event, 
  public bx_trigger_decoded_event {
  public:
    bx_trigger_event (const char *disk_event): bx_trigger_raw_event(disk_event) {}
    virtual ~bx_trigger_event () {}

  private:
    void mark_stage(event_stage es, bool v = true) { stages[es] = v; }

  friend class bx_trigger_decoder;
};

#endif
/*
 * $Log: bx_trigger_event.hh,v $
 * Revision 1.7  2004/05/25 21:46:10  ddangelo
 * fixed a name duplication bug
 *
 * Revision 1.6  2004/05/25 16:37:05  ddangelo
 * added support for event stage flagging.
 * Inheritance from bx_base_event introduced
 *
 * Revision 1.5  2004/05/20 15:26:38  ddangelo
 * fixed a byte ordering problem
 *
 * Revision 1.4  2004/05/20 10:25:00  ddangelo
 * bool locals for trigger types introduced in decoded event, along with relative getters.
 *
 * Revision 1.3  2004/05/18 17:29:33  ddangelo
 * trigger disk event format expanded. class bx_trigger_raw_event expanded accordingly.
 * Getters expanded also. A bitfield added for btb inputs (8 bit only, should be portable).
 *
 * Revision 1.2  2004/04/27 10:45:23  ddangelo
 * fixed module name in friend declaration
 *
 * Revision 1.1  2004/04/27 09:41:09  ddangelo
 * Event structure redisegned.
 * bx_reco_event now handles event header and has object for different detector portions.
 * Inheritance from different reconstruction status is kept within each detector portion obj.
 * Hit lists are linked by const ptr (maybe a temporary solution).
 *
 *
 */
