/* BOREXINO Reconstruction program
 *
 * Author: Sergey Sukhotin <sukhotin@cdf.in2p3.fr>
 * 
 * $Id: fadc_sum.cc,v 1.1 2004/08/26 14:30:27 sukhotin Exp $
 *
 * This class prepares the digital sum, deconvolute it and
 * calculates the amplitude, charge and peak position
 * for deconvoluted digital sum.
 *
 */

#include "constants.hh"
#include "fadc_sum.hh"
#include "fadc_calc.hh"
#include "fadc_channel.hh"
#include "messenger.hh"

#include <math.h>
#include <algorithm>
#include <numeric>
#include <vector>

const float fadc_sum::tau  = 195.0;       // to deconvolute raw data
const float fadc_sum::E0   = 1000.0;      // energy threshold for second pulse BEFORE the main one 
const float fadc_sum::ECUT = 3000.0;      // energy threshold for second pulse AFTER  the main one   
const unsigned int fadc_sum::shift1 = 10; // main peak shift to search a second pulse BEFORE the main
const unsigned int fadc_sum::shift2 = 30; // main peak shift to search a second pulse AFTER the main

fadc_sum::fadc_sum (const bx_fadc_raw_window& win, unsigned char trgtype) {
  int size = (int)win.get_win_size ();
  std::vector<float> v(size, 0.0);
  for (unsigned char j = 3; j < win.get_nchannels (); j++) {
    const bx_fadc_raw_channel &ch = win.get_channel (j);
    fadc_channel r_ch(ch, 0, size);
    for(unsigned short k = 0; k < size; k++) v[k] += r_ch.get_sample(k);
  }     
  std::vector<float> s(size);
  std::partial_sum(v.begin (), v.end (), s.begin ());
	
  for(unsigned short k = 0; k < size; ++k)  dsum_v.push_back (v[k] + s[k] / tau);
  
  fadc_calc result(dsum_v);
  float main_peak   = result.get_peak();
  float main_ampl   = result.get_ampl();
  float main_charge = result.get_charge();
  u1_npulses     = 1;
  
  unsigned char flag1=0, flag2=0;

// Look for a possible second pulse BEFORE the main peak
  int zone1 = (int)main_peak - shift1;
  int zone2 = (int)main_peak + shift2;
  if ( (trgtype != 32) && (zone1 > 20) && (zone2 < (size - 10)) ) {
    std::vector<float> v1(zone1);
    std::copy(dsum_v.begin(), dsum_v.begin() + zone1, v1.begin());
    fadc_calc res1(v1);
    if (res1.get_charge() > E0) {
      // second pulse was found BEFORE the main peak   
      flag1 = 1;   
      u1_npulses++;
    }

// Look for a possible second pulse AFTER the main peak  
    std::vector<float> v2(size - zone2 + 1);
    std::copy(dsum_v.begin() + zone2, dsum_v.end(), v2.begin());
    fadc_calc res2(v2);
    if (res2.get_charge() > ECUT) {
      // second pulse was found AFTER the main peak
      flag2   = 1;   
      u1_npulses++;
    }

    if ( flag1==0 && flag2==0 ) {
      first_bin.push_back (0);      last_bin.push_back (size);
      
      peak_v.push_back (main_peak);
      ampl_v.push_back (main_ampl);
      charge_v.push_back (main_charge);
      u1_flag = 1;      
    }
  
    if ( flag1==1 && flag2==0) {
      first_bin.push_back (0);       last_bin.push_back (zone1);
      first_bin.push_back (zone1);   last_bin.push_back (size);
    
      peak_v.push_back (res1.get_peak());
      peak_v.push_back (main_peak);
    
      ampl_v.push_back (res1.get_ampl());
      ampl_v.push_back (main_ampl);
    
      charge_v.push_back (res1.get_charge());
      charge_v.push_back (main_charge);
      u1_flag = 2;      
    }
  
    if ( flag1==0 && flag2==1) {
      first_bin.push_back (0);       last_bin.push_back (zone2);
      first_bin.push_back (zone2);   last_bin.push_back (size);
          
      peak_v.push_back (main_peak);
      peak_v.push_back (res2.get_peak());
    
      ampl_v.push_back (main_ampl);
      ampl_v.push_back (res2.get_ampl());
    
      charge_v.push_back (main_charge);
      charge_v.push_back (res2.get_charge());
      u1_flag = 3;       
    }
  
    if ( flag1==1 && flag2==1) {
      first_bin.push_back (0);       last_bin.push_back (zone1);
      first_bin.push_back (zone1);   last_bin.push_back (zone2);
      first_bin.push_back (zone2);   last_bin.push_back (size);
    
      peak_v.push_back (res1.get_peak());    
      peak_v.push_back (main_peak);
      peak_v.push_back (res2.get_peak());
    
      ampl_v.push_back (res1.get_ampl());
      ampl_v.push_back (main_ampl);
      ampl_v.push_back (res2.get_ampl());
    
      charge_v.push_back (res1.get_charge());
      charge_v.push_back (main_charge);
      charge_v.push_back (res2.get_charge());
      u1_flag = 4;       
    }
 } else {
    first_bin.push_back (0);      last_bin.push_back (size);
     
    peak_v.push_back (main_peak);
    ampl_v.push_back (main_ampl);
    charge_v.push_back (main_charge);
    u1_flag = 1; 
   }      
}

/*
 * $Log: fadc_sum.cc,v $
 * Revision 1.1  2004/08/26 14:30:27  sukhotin
 * This is an additional class for fadc decoder.
 * It does all needed operations with digital sum.
 *
 *
 */
