/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: laben_charge_hit.cc,v 1.5 2004/09/12 12:57:15 razeto Exp $
 *
 * laben_charge_hit implementation
 *
 */
#include "laben_charge_hit.hh"
#include "messenger.hh"
#include <math.h>

const float laben_charge_hit::default_bin_to_charge = 1.;
const float laben_charge_hit::integrator_decay_time = 500.;
const float laben_charge_hit::restoration_time = 2700;  // e^-(2800/500) < 1/255
const float laben_charge_hit::d80 = 80;


laben_charge_hit::laben_charge_hit (const bx_laben_decoded_hit& hit): bx_named ("laben_charge_hit") {
  u1_peak = hit.get_raw_hit ().get_peak ();
  u1_base = hit.get_raw_hit ().get_base ();
  f4_prev_dt = restoration_time;

  f4_baseline_fast_drift = get_parameter ("baseline_fast_drift").get_float (); 
  if (get_parameter ("use_charge_calib").get_int ()) {
    get_message (bx_message::error) << "carge calibration not yet implemented" << dispatch;
   // Get some parameters from the db
  //const db_profile& profile_info = bx_dbi::get()->get_profile ();
  //f4_channel_bin_to_charge = ...
  } else f4_channel_bin_to_charge = default_bin_to_charge;
}	

laben_charge_hit::laben_charge_hit (const bx_laben_decoded_hit& hit, const bx_laben_decoded_hit& previus_hit): bx_named ("laben_charge_hit") {
  int lg = hit.get_raw_hit ().get_logical_channel ();
  int lg_prev = previus_hit.get_raw_hit ().get_logical_channel ();
  if (lg != lg_prev) 
    get_message (bx_message::critic) << "laben_charge_hit constructed with hit of different channels " << lg << "!=" << lg_prev << dispatch;

  f4_prev_dt = hit.get_raw_time () - previus_hit.get_raw_time ();
  if (f4_prev_dt < 0) 
    get_message (bx_message::critic) << "laben_charge_hit construced with wrong time sorted hit " << hit.get_raw_time () << "<" << previus_hit.get_raw_time () << dispatch;
  
  u1_peak = hit.get_raw_hit ().get_peak ();
  u1_base = hit.get_raw_hit ().get_base ();
  u1_prev_peak = previus_hit.get_raw_hit ().get_peak ();
  u1_prev_base = previus_hit.get_raw_hit ().get_base ();
  
  f4_baseline_fast_drift = get_parameter ("baseline_fast_drift").get_float (); 
  if (get_parameter ("use_charge_calib").get_int ()) {
    get_message (bx_message::error) << "carge calibration not yet implemented" << dispatch;
   // Get some parameters from the db
  //const db_profile& profile_info = bx_dbi::get()->get_profile ();
  //f4_channel_bin_to_charge = ...
  } else f4_channel_bin_to_charge = default_bin_to_charge;
}

float laben_charge_hit::get_charge_bin () {
  float simple_charge = u1_peak - u1_base;

    // Some simple check to see if a overlapping routine is to be used
  if (f4_prev_dt >= restoration_time) return simple_charge; 	// keep >= since one hit ctor sets f4_prev_dt = restoration_time
  if (f4_prev_dt < d80) return simple_charge; 			// It should not happen since dt for good channels should be > 140ns
  if (u1_prev_peak <= u1_prev_base) return simple_charge; 	// No info can be found on prev hit
  
    // A more detailed check to see if the actual base sample is too distant from where it should
    // be (simply considering integrator discarge. If not the baseline is too noisy.
  float prev_simple_charge = u1_prev_peak - u1_prev_base;
  float base_check = prev_simple_charge * ::expf ((d80 - f4_prev_dt) / integrator_decay_time) + u1_prev_base;
  if (::fabsf(base_check - u1_base) > f4_baseline_fast_drift) return simple_charge; 

    // If here all check were sucesfull so calculate pile-up
  float prev_base_delayed = prev_simple_charge * ::expf (-f4_prev_dt / integrator_decay_time) + u1_prev_base;
  return u1_peak - prev_base_delayed;
}
/*
 * $Log: laben_charge_hit.cc,v $
 * Revision 1.5  2004/09/12 12:57:15  razeto
 * Added math.h include
 *
 * Revision 1.4  2004/09/09 11:44:28  razeto
 * Upgraded to be a bx_named; added pileup calculation
 *
 * Revision 1.3  2004/06/08 10:25:19  razeto
 * Updated to new hit name (rawt -> raw_time)
 *
 * Revision 1.2  2004/05/25 17:18:20  razeto
 * Upgraded
 *
 * Revision 1.1  2004/05/20 10:51:34  razeto
 * Added sketch version
 *
 */
