/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 * 	   Marco Pallavicini <pallas@ge.infn.it>
 *
 * $Id: bx_module_factory.cc,v 1.31 2004/08/10 11:14:03 razeto Exp $
 *
 * Implementation of bx_module_factory
 *
 */
#include <stdexcept>
#include <fstream>
#include <iostream>
#include <iterator>
#include <algorithm>

#include "bx_module_factory.hh"
#include "bx_base_module.hh"
#include "bx_test_module.hh"
#include "bx_trigger_decoder.hh"
#include "bx_options.hh"
#include "bx_reader.hh"
#include "bx_writer.hh"
#include "bx_muon_decoder.hh"
#include "bx_reader.hh"
#include "bx_writer.hh"
#include "bx_precalib_laben_adc.hh"
#include "bx_precalib_laben_gray.hh"
#include "bx_precalib_laben_phases.hh"
#include "bx_precalib_laben_d80.hh"
#include "bx_precalib_laben_check_tdc.hh"
#include "bx_precalib_muon_findpulse.hh"
#include "bx_precalib_muon_pedestals.hh"
#include "bx_laben_findcluster_time.hh"
#include "bx_laben_findcluster.hh"
#include "bx_laben_decoder.hh"
#include "bx_fadc_decoder.hh"
#include "bx_laben_electronics_status.hh"
#include "bx_laben_allign_crates.hh"
#include "messenger.hh"

bx_module_factory *bx_module_factory::me = 0;

// ctor
bx_module_factory::bx_module_factory () {
  modules.clear ();
}

bx_module_factory::~bx_module_factory () {
  std::for_each (modules.begin (), modules.end (), bx_base_module::delete_operation ());
  modules.clear ();
}

// singleton
bx_module_factory* bx_module_factory::get () {
  if (!me) 
    me = new bx_module_factory;

  return me;
}

bx_base_module *bx_module_factory::get_module (const std::string &module_name) {

  bx_base_module::bx_base_module_vector::iterator item = std::find_if (modules.begin (), modules.end (), bx_base_module::compare_name_operation (module_name));
  if (item != modules.end ()) return *item;
  
  bx_message msg(bx_message::critic, "bx_module_factory: ");
  msg << "asked for module " << module_name << " which is not present. Present modules:";
  std::copy (modules.begin (), modules.end (), std::ostream_iterator<bx_base_module *>(msg, "\n"));
  msg << dispatch;

  return *item;
}


bx_base_module *bx_module_factory::get_module (bx_base_module::module_role role) {
  bx_base_module::bx_base_module_vector::iterator item = std::find_if (modules.begin (), modules.end (), bx_base_module::compare_role_operation (role));
  if (item != modules.end ()) return *item;
  
  bx_message msg(bx_message::critic, "bx_module_factory: ");
  msg << "asked for module with role " << role << " which is not present. Present modules:";
  std::copy (modules.begin (), modules.end (), std::ostream_iterator<bx_base_module *>(msg, "\n"));
  msg << dispatch;

  return *item;
}


void bx_module_factory::create_modules (const std::vector<std::string>& module_name_list) {
  bx_message msg(bx_message::critic, "bx_module_factory: ");

  if (modules.size ()) msg << "already inited" << dispatch;

  for (unsigned long i = 0; i < module_name_list.size (); i++) {
    std::string m_name = module_name_list[i];
    if (m_name == "bx_event_reader") {
      modules.push_back (new bx_event_reader);
    } else if (m_name == "bx_event_writer") {
      modules.push_back (new bx_event_writer);
    } else if (m_name == "bx_precalib_laben_adc") {
      modules.push_back (new bx_precalib_laben_adc);
    } else if (m_name == "bx_precalib_laben_gray") {
      modules.push_back (new bx_precalib_laben_gray);
    } else if (m_name == "bx_precalib_laben_phases") {
      modules.push_back (new bx_precalib_laben_phases);
    } else if (m_name == "bx_precalib_laben_d80") {
      modules.push_back (new bx_precalib_laben_d80);
    } else if (m_name == "bx_precalib_laben_check_tdc") {
      modules.push_back (new bx_precalib_laben_check_tdc);
    } else if (m_name == "bx_precalib_muon_findpulse") {
      modules.push_back (new bx_precalib_muon_findpulse);
    } else if (m_name == "bx_precalib_muon_pedestals") {
      modules.push_back (new bx_precalib_muon_pedestals);
    } else if (m_name == "bx_test_module") {
      modules.push_back (new bx_test_module);
    } else if (m_name == "bx_trigger_decoder") {
      modules.push_back (new bx_trigger_decoder);
    } else if (m_name == "bx_muon_decoder") {
      modules.push_back (new bx_muon_decoder);
    } else if (m_name == "bx_laben_decoder") {
      modules.push_back (new bx_laben_decoder);
    } else if (m_name == "bx_fadc_decoder") {
      modules.push_back (new bx_fadc_decoder);
    } else if (m_name == "bx_laben_findcluster") {
      modules.push_back (new bx_laben_findcluster);
    } else if (m_name == "bx_laben_findcluster_time") {
      modules.push_back (new bx_laben_findcluster_time);
    } else if (m_name == "bx_laben_electronics_status") {
      modules.push_back (new bx_laben_electronics_status);
    } else if (m_name == "bx_laben_allign_crates") {
      modules.push_back (new bx_laben_allign_crates);
    } else {
      msg << "unknown module name: " << m_name << dispatch;
    }
  }

  // Sort the array
  std::sort (modules.begin (), modules.end (), bx_base_module::sort_operation ());

  bx_message info(bx_message::info, "bx_module_factory: module list:");
  for (unsigned long i = 0; i < modules.size (); i++) {
    info << std::endl << "\t" <<  modules[i]->get_name () << " role " << modules[i]->get_role ();
    info << " priority " << modules[i]->get_priority ();
    if (modules[i]->is_enabled ()) info << " enabled";
    else info << " disabled";
  }
  info << dispatch;
  
  // A consinstency test for the modules array: reader have to be present in
  // only 1 enabled module.
  m_check_single_presence (bx_base_module::reader);
}


void bx_module_factory::m_check_single_presence (bx_base_module::module_role role) {
  bx_message msg(bx_message::critic, "bx_module_factory: ");

  // compare_role_operation only takes in account enabled modules.
  bx_base_module::bx_base_module_vector::iterator item1 = std::find_if (modules.begin (), modules.end (), bx_base_module::compare_role_operation (role));
  if (item1 == modules.end ()) msg << "no module for role " << role << ", aborting" << dispatch;

  bx_base_module::bx_base_module_vector::iterator item2 = std::find_if (item1 + 1, modules.end (), bx_base_module::compare_role_operation (role));
  if (item2 != modules.end ()) msg << "several modules for role " << role << " (" << (*item1)->get_name () << ", " << (*item2)->get_name () << "), aborting" << dispatch;
}
  
/*
 * $Log: bx_module_factory.cc,v $
 * Revision 1.31  2004/08/10 11:14:03  razeto
 * Added a module for calculating the time differences from crate to crate
 *
 * Revision 1.30  2004/07/22 10:16:29  razeto
 * Added bx_laben_electronics_status
 *
 * Revision 1.29  2004/07/22 10:13:01  razeto
 * Jumbo patch for the configuration manager applied, read the documentation for usage
 *
 * Revision 1.28  2004/07/16 07:34:38  razeto
 * Add fadc decoder
 *
 * Revision 1.27  2004/07/12 10:05:30  razeto
 * Added some clustering modules
 *
 * Revision 1.26  2004/06/10 12:40:19  razeto
 * Added some needed includes
 *
 * Revision 1.25  2004/06/07 10:59:20  razeto
 * Upgraded to handle multiple writer at the same time
 *
 * Revision 1.24  2004/06/01 15:36:14  razeto
 * Upgrade to full support vdt vectors
 *
 * Revision 1.23  2004/05/31 14:59:03  razeto
 * Updated to new method is_enabled in base module
 *
 * Revision 1.22  2004/05/21 08:41:59  razeto
 * Added laben decoder
 *
 * Revision 1.21  2004/05/18 14:54:27  razeto
 * Added few modules
 *
 * Revision 1.20  2004/04/27 16:58:00  ddangelo
 * removed old includes
 *
 * Revision 1.19  2004/04/27 09:50:43  ddangelo
 * test module precalibrator removed.
 * module bx_trigger_decode_module renamed to bx_trigger_decoder.
 *
 * Revision 1.18  2004/04/24 17:36:45  razeto
 * Added a check if the parameter already exist in setting (added an init method too)
 *
 * Revision 1.17  2004/04/20 16:08:02  ddangelo
 * added handling for a few muon models
 *
 * Revision 1.16  2004/04/18 10:30:14  razeto
 * Fixed to compile wit g++ 2.95 present on the cluster
 *
 * Revision 1.15  2004/04/12 16:01:15  razeto
 * Added the precalib modules and a message
 *
 * Revision 1.14  2004/04/09 07:50:19  razeto
 * Added some more messages
 *
 * Revision 1.13  2004/04/06 12:43:55  razeto
 * Added module role to base_module, and the code to handle everywhere. Some other minor fixes
 *
 * Revision 1.12  2004/04/03 09:23:24  razeto
 * Fixed a bug. Corrected the test module name
 *
 * Revision 1.11  2004/04/01 12:08:53  razeto
 * Moved the base_module_operation operators in the bx_base_module class
 * scope; moved the bx_base_module_vector typedef in the bx_base_module
 * class.
 * Added a event delete to the frame.
 *
 * Revision 1.10  2004/03/26 16:36:11  razeto
 * Introduced bx_options; a lot of code modified to read options and parameters.
 * Bx_event_reader interface changed: now there is a standard constructor and
 * the file opening is done at begin using the parameters for the file name.
 *
 * Revision 1.9  2004/03/26 16:31:14  razeto
 * Fixed a bug: reader, writer and precalibrator were pointers, but the could
 * be even not initialized since the initialization were left to a procedure
 * which would allocate them if they where present on the modules.cfs file.
 * This could not be true.
 * Now these special modules are in a dedicated vector which however has
 * the same paradigm of the standard module vector.
 * This solves the problem; maybe a role for the module could be introduced
 * in future, allowing to have just one vector.
 *
 * Revision 1.8  2004/03/24 16:22:08  razeto
 * Moved reader,writer,precalib creation from framework to factory
 *
 * Revision 1.7  2004/03/24 14:23:35  razeto
 * Moved module list ownership from framewrok to module_factory, this allows:
 *   - a better syntax for bx_options which can store parameters even if the
 * framework is not created
 *   - inter module comunications
 * Since the module factory was already a singleton this does not affect
 * the modules array lifecycle.
 *
 * Revision 1.6  2004/03/23 14:26:47  pallas
 * Debugging trigger module
 * GPS clock OK
 *
 * Revision 1.5  2004/03/22 14:29:25  razeto
 * Some cosmetic changes
 *
 * Revision 1.4  2004/03/21 20:19:17  razeto
 * Added std:: to string
 *
 * Revision 1.3  2004/03/20 18:46:45  pallas
 * debugging
 *
 * Revision 1.2  2004/03/20 17:50:54  pallas
 * Debugging
 *
 * Revision 1.1  2004/03/20 17:39:45  pallas
 * First release
 *
 * Revision 1.1  2004/03/20 14:16:47  razeto
 * Added sources
 *
 *
 */
