/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 * 	   Marco Pallavicini <pallas@ge.infn.it>
 *
 * $Id: bx_reco_framework.cc,v 1.20 2004/06/07 10:59:20 razeto Exp $
 *
 * Implementation of bx_reco_framework
 *
 */
#include "bx_reco_framework.hh"
#include "bx_module_factory.hh"
#include "messenger.hh"
#include "bx_options.hh"
#include "bx_dbi.hh"
#include "db_run.hh"

#include <stdexcept>
#include <iostream>
#include <algorithm>

// ctor
bx_reco_framework::bx_reco_framework(): modules(bx_module_factory::get ()->get_modules ()) {
  p_reader = dynamic_cast<bx_event_reader *>(bx_module_factory::get ()->get_module (bx_base_module::reader));
}

// dtor
bx_reco_framework::~bx_reco_framework () {
  // modules has not to be deleted since are owned by bx_module_factory
}

// execute all program; basically, this is the Borexino Reco Main
void bx_reco_framework::run () {
  
  // init reader module 
  p_reader->begin ();

    // Do precalibration (4 cycles), only if not yet present on db
  if (!bx_dbi::get ()->get_run ().is_precalib_present ()) {
    m_do_precalib (bx_base_module::precalib_cycle1);
    m_do_precalib (bx_base_module::precalib_cycle2);
    m_do_precalib (bx_base_module::precalib_cycle3);
    m_do_precalib (bx_base_module::precalib_cycle4);
  }
    
    // Init main loop modules and writer
  std::for_each (modules.begin (), modules.end (), bx_base_module::begin_operation (bx_base_module::main_loop));
  std::for_each (modules.begin (), modules.end (), bx_base_module::begin_operation (bx_base_module::writer));

  long int ev_count = 0;
    // Do main loop
  while (bx_reco_event *ev = p_reader->doit (0)) {
    // check here for maximum number of events; to be done
    if (bx_options::get()->get_max_events() >= 0 && ev_count++ >= bx_options::get()->get_max_events()) break;

    // loop on modules and filter event
    std::for_each (modules.begin (), modules.end (), bx_base_module::doit_operation (ev, bx_base_module::main_loop));
  
    // write event (root file or other)
    std::for_each (modules.begin (), modules.end (), bx_base_module::doit_operation (ev, bx_base_module::writer));

    delete ev;
  }

  // end of job
  p_reader->end ();
  std::for_each (modules.begin (), modules.end (), bx_base_module::end_operation (bx_base_module::main_loop));
  std::for_each (modules.begin (), modules.end (), bx_base_module::end_operation (bx_base_module::writer));
  
  bx_message msg;
  msg.set_level (bx_message::info);
  msg.set_log_level (bx_message::info);
  msg.set_print_level (bx_message::info);
  msg << "Echidna NORMAL END" << dispatch;
}

void bx_reco_framework::m_do_precalib (bx_base_module::module_role cycle) {
  std::for_each (modules.begin (), modules.end (), bx_base_module::begin_operation (cycle));
  for (int i = 0; i < 1000; i++) {
    bx_reco_event *ev = p_reader->doit (0);
    if (!ev) break;
    if (ev->get_trigger ().get_trgtype () != 32) continue;
    std::for_each (modules.begin (), modules.end (), bx_base_module::doit_operation (ev, cycle));
    delete ev;
  }
  p_reader->rewind ();
  std::for_each (modules.begin (), modules.end (), bx_base_module::end_operation (cycle));
}

/*
 * $Log: bx_reco_framework.cc,v $
 * Revision 1.20  2004/06/07 10:59:20  razeto
 * Upgraded to handle multiple writer at the same time
 *
 * Revision 1.19  2004/05/26 09:24:53  razeto
 * Upgraded to check if precalib are needed
 *
 * Revision 1.18  2004/05/26 08:27:26  razeto
 * Added max number of events (negative = unlimited)
 *
 * Revision 1.17  2004/05/21 08:41:15  razeto
 * Fixed the trgtype code; still to be removed
 *
 * Revision 1.16  2004/05/19 08:43:37  razeto
 * Updated to follow new trigger raw event syntax
 *
 * Revision 1.15  2004/05/18 14:54:52  razeto
 * Added a simple test, to be removed
 *
 * Revision 1.14  2004/04/12 16:09:42  razeto
 * Added precalib loop handler. Added the loops for the precalibs
 *
 * Revision 1.13  2004/04/06 12:43:55  razeto
 * Added module role to base_module, and the code to handle everywhere. Some other minor fixes
 *
 * Revision 1.12  2004/04/03 09:24:42  razeto
 * Added messenger
 *
 * Revision 1.11  2004/04/01 12:08:53  razeto
 * Moved the base_module_operation operators in the bx_base_module class
 * scope; moved the bx_base_module_vector typedef in the bx_base_module
 * class.
 * Added a event delete to the frame.
 *
 * Revision 1.10  2004/03/24 16:22:08  razeto
 * Moved reader,writer,precalib creation from framework to factory
 *
 * Revision 1.9  2004/03/24 14:23:35  razeto
 * Moved module list ownership from framewrok to module_factory, this allows:
 *   - a better syntax for bx_options which can store parameters even if the
 * framework is not created
 *   - inter module comunications
 * Since the module factory was already a singleton this does not affect
 * the modules array lifecycle.
 *
 * Revision 1.8  2004/03/23 13:01:05  razeto
 * Moved base_module_operatos to bx_base_module.hh.
 * Added compare_operation.
 * Added get_module by name to framework.
 *
 * Revision 1.7  2004/03/22 14:50:08  razeto
 * Introduced algorithms for looping on modules with predefined operations
 *
 * Revision 1.6  2004/03/22 14:29:25  razeto
 * Some cosmetic changes
 *
 * Revision 1.5  2004/03/20 19:05:17  pallas
 * Debugging
 *
 * Revision 1.4  2004/03/20 18:55:02  pallas
 * Debugging framework
 *
 * Revision 1.3  2004/03/20 18:46:45  pallas
 * debugging
 *
 * Revision 1.2  2004/03/20 17:50:54  pallas
 * Debugging
 *
 * Revision 1.1  2004/03/20 17:39:45  pallas
 * First release
 *
 * Revision 1.1  2004/03/20 14:16:47  razeto
 * Added sources
 *
 *
 */
