/* BOREXINO Reconstruction program
 * 
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: bx_dbi.hh,v 1.10 2004/05/26 11:02:44 razeto Exp $
 *
 * The database interface
 *
 */
#ifndef _BX_DBI_H
#define _BX_DBI_H

#include "vdt.hh"
#include "bx_rec_general.hh"
#include "cmap.hh"
#include "messenger.hh"

#include <string>
#include <map>
#include <vector>
#include "libpq-fe.h"


class db_run;
class db_profile;
class db_channel;

class bx_dbi {
  public:
    enum database_id {
      daq_config = 0,
      bx_geometry = 1,
      bx_calib = 2,
      bx_physics = 3,
      bx_slow = 4,
      channelhistory = 5,
    };
   typedef std::vector<vdt> column;
   typedef std::cmap<std::string, column> table;

    static bx_dbi* get ();

    // Do a query: the first method allow not to type a valid SQL query, while the second is present
    // if the user has to make some complicated query, not foreseen by the first.
    // max_lines is the maximum number of line to read and has the following syntax:
    //   0 means unlimited
    //   > 0 do not read more the indicated lines (soft limit)
    //   < 0 if the query returns more than the indicated lines (abs value) there is an exception (hard limit)
    // Every field has to be quoted with the SQL syntax, ie table_fields has to be "\"ProfileID\""
    // For best use a const reference has to be assigned to the return value of query, as follow:
    // const bx_dbi::table &table = dbi->query (...);
    const table query (database_id db_id, const std::string& from_table, const std::string& where,
		  const std::string &table_fields = "*", const std::string& order = "", int max_lines = 0);
    const table query (database_id db_id, const std::string& sql_query, int max_lines); 
    
    
    int get_current_run_number () const { return i4_current_run_number; }
    void set_current_run_number (int run_number) { i4_current_run_number = run_number; }

    // Get db_run/db_profile table for the requested run/profile.
    db_run& get_run (int run_number = -1);
    db_profile& get_profile (int profile_id = -1);
    // Get the db_channel for the specified channel
    const db_channel& get_channel (int lg, int profile_id = -1);

      // intendend to be used locally or in the db_profile/db_run
    bx_message& get_message (bx_message::message_level level);
  private:
    bx_dbi ();
    bx_dbi (const bx_dbi& dbi) { } // I do not want a copy constructor
    static bx_dbi *me;

    long int i4_current_run_number;

    void m_initialize_from_file (const std::string &filename);
    void m_open_connections (database_id id);
    
      // The database are accessed by index of database_id type; since, unfortunatelly,
      // the database name is mutable a double reference process has to be done using
      // db_name_hash_index which indexes formal DB names using standards names.
      // Then the params map contain a lookup from these standard names and the 
      // db name. It is important that the paramenter names in the config file
      // (the name before the colon) are excactly the formal names.
      // BTW, no one should change the format of the configuration file.
    typedef std::map<database_id, std::string> db_name_hash_index;
    typedef std::map<database_id, PGconn *> db_connection;
    typedef std::cmap<std::string, vdt> db_params;
    db_name_hash_index hash_name_indexes;
    db_connection connections;
    db_params params;
    
      // since db_run has private constructors and is friend to bx_dbi a map
      // of db_run objects can not be done (since the map allocator fails in
      // initializing the stored object). An array of pointers is instead possible.
      // Idem for profile map.
    typedef std::cmap<int, db_run*> db_run_map;
    typedef std::cmap<int, db_profile*> db_profile_map;
    typedef std::cmap<int, std::cmap<int, db_channel *> > db_channel_map;
    db_run_map run_map;
    db_profile_map profile_map;
    db_channel_map channel_map;

    bx_message message;
};

#endif
/*
 * $Log#
 */
