/* BOREXINO Reconstruction program
 * 
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: bx_options.cc,v 1.11 2004/08/06 10:30:28 razeto Exp $
 *
 * Implementation of bx_options
 * Here bx_message messages are not used and direct throw are called on error,
 * this is done because every error is a line option/config file parsing error 
 * and have to be reported directly.
 *
 */
#include "bx_options.hh"
#include "bx_module_factory.hh"
#include "messenger.hh"
#include "configuration_manager.hh"

#include <stdexcept>
#include <iostream>
#include <fstream>
#include <sstream>
#include <unistd.h>
#include <time.h>


bx_options *bx_options::me = 0;


bx_options* bx_options::get () {
  if (!me) me = new bx_options;

  return me;
}

bx_options::bx_options (): i4_max_events(-1) { }


void bx_options::initialize_echidna (int argc, char * const argv[]) {
  extern char *optarg;
  extern int optind, opterr, optopt;

  configuration_manager *cm = new configuration_manager;
  int verbosity = 0;
  std::string user_config_file = "user.cfg";
  std::string main_config_file = "echidna.cfg";
  std::string config = "default";
  std::string log_filename;

  do {		// do {} while to avoid too many local variables
    time_t t = time (0);
    char time_string[31];
    ::strftime(time_string, 31, "echidna_%d:%m:%Y_%H%M%S.log", ::localtime(&t));
    log_filename = time_string;
  } while (0); 

  opterr = 0;
  while (1) {
    const char *valid_options = ":vc:C:f:e:n:p:lo:";
    int c = ::getopt (argc, argv, valid_options);
    if (c == -1) break;
    
    switch (c) {
      case 'c':
	user_config_file = optarg;
	break;
      case 'C':
	main_config_file = optarg;
	break;
      case 'f':
	cm->add_comman_line_parameter ("bx_event_reader.file_url", optarg);
	break;
      case 'e':
	i4_max_events = vdt(optarg, "-e").get_int ();
	break;
      case 'p':
	  // optind is the index to the next option (so argv[optind] is the second opt parameter)
	if (!argv[optind] || argv[optind][0] == '-') throw std::runtime_error ("option -p requires 2 parameters with the \"module.param_name value\" syntax"); 
	cm->add_comman_line_parameter (optarg, argv[optind++]);
	break;
      case 'l':
        if (optind < argc && argv[optind] && argv[optind][0] != '-') log_filename = argv[optind++];
	else log_filename = "";
	break;
      case 'v':
	verbosity ++;
	break;
      case 'o':
	cm->add_comman_line_parameter ("bx_event_writer.file_name", optarg);	
	break;
      case ':':
        throw std::runtime_error ("missing value on command line option -" + char (optopt));
	break;
      default:
        throw std::runtime_error ("unknown command line option: -" + char (optopt));
    }
  }

    // Check for extra option comman line fields
  if (optind < argc) {
      // First argument = config
    if ((optind + 1) == argc) config = argv[optind++];
    if (optind < argc) {
      std::ostringstream msg;
      msg << "unknown fields on the command line:";
      while (optind < argc) msg << " \"" << argv[optind++] << "\"";
      throw std::runtime_error (msg.str ());
    }
  }

    // Set verbosity level and log file
  if (verbosity == 1) bx_message::set_default_print_level (bx_message::info);
  else if (verbosity > 1) bx_message::set_default_print_level (bx_message::debug);
  else bx_message::set_default_print_level (bx_message::warn);
  messenger::get ()->open_log_file (log_filename);

    // Send config files to the configuration manager
  cm->add_main_config_file (main_config_file);
  if (user_config_file.size ()) cm->add_user_config_file (user_config_file);
  
    // Upload the parameter broker with parameters from the current config
  cm->upload_configuration (config);

    // Set the parameter broker to bx_named (indeed the configuration manager is a parameter broker)
  bx_named::set_parameter_broker (cm);

    // Create modules
  bx_module_factory::get ()->create_modules (cm->get_module_name_list());
}

/*
 * $Log: bx_options.cc,v $
 * Revision 1.11  2004/08/06 10:30:28  razeto
 * cycle_1 branch merged in the main trunk.
 *
 * Revision 1.10.2.1  2004/07/26 14:16:29  razeto
 * Fixed a output file name parameter name.
 *
 * Revision 1.10  2004/07/22 10:13:01  razeto
 * Jumbo patch for the configuration manager applied, read the documentation for usage
 *
 * Revision 1.9  2004/05/26 09:22:37  razeto
 * Upgraded to last version of vdt
 *
 * Revision 1.8  2004/05/26 08:27:15  razeto
 * Added max number of events (negative = unlimited)
 *
 * Revision 1.7  2004/04/24 17:40:22  razeto
 * Updated to the new bx_reader file access
 *
 * Revision 1.6  2004/04/18 09:16:52  razeto
 * Added -o option, to select the output root file
 *
 * Revision 1.5  2004/04/12 16:07:39  razeto
 * Added configurable default print level for messages
 *
 * Revision 1.4  2004/04/06 12:39:28  razeto
 * Added logfile handling
 *
 * Revision 1.3  2004/04/05 13:12:27  razeto
 * Updated a comment, changed the behaviour if no user option file is present
 *
 * Revision 1.2  2004/03/29 13:17:30  razeto
 * Updated bx_options to the vdt usage
 *
 * Revision 1.1  2004/03/26 16:36:11  razeto
 * Introduced bx_options; a lot of code modified to read options and parameters.
 * Bx_event_reader interface changed: now there is a standard constructor and
 * the file opening is done at begin using the parameters for the file name.
 *
 *
 */
