/* BOREXINO Reconstruction program
 * 
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: bx_root_barn.cc,v 1.3 2004/05/21 11:34:48 razeto Exp $
 *
 * Implementation of bx_root_barn
 *
 */

#include "bx_root_barn.hh"
#include "messenger.hh"

#include <TKey.h>
#include <TSystem.h>
#include <algorithm>

struct check_name {
  check_name (const std::string& name): s(name) {}
  bool operator() (const root_item& item) { return item.get_name () == s; }
  const std::string& s;
};

struct check_type {
  check_type (bx_root_barn::item_type type): t(type) {}
  bool operator() (const root_item& item) { return item.get_type () == t; }
  bx_root_barn::item_type t;
};


bx_root_barn* bx_root_barn::me = 0;

bx_root_barn::bx_root_barn () {
  items.clear ();
}

bx_root_barn *bx_root_barn::get () {
 if (!me) me = new bx_root_barn;

 return me;
}

void bx_root_barn::disable_root_signals () {
  gSystem->ResetSignal(kSigSegmentationViolation);
  gSystem->ResetSignal(kSigIllegalInstruction);
  gSystem->ResetSignal(kSigSystem);
  gSystem->ResetSignal(kSigPipe);
  gSystem->ResetSignal(kSigFloatingException);
}

void bx_root_barn::store (item_type type, TNamed *obj, const bx_base_module *module) {

    // Check if a object is already present with that name
    std::list<root_item>::const_iterator item = std::find_if (items.begin (), items.end (), check_name (obj->GetName ()));
    
  if (item != items.end ()) {
    bx_message msg(bx_message::error);
    msg << "bx_root_barn: " << module->get_name () << " module registering a TNamed with a name already registered by " << item->get_owner_module_name () << " module" << dispatch;
    return;
  }

  items.push_back (root_item (type, obj, module));
}

TDirectory* bx_root_barn::m_get_dir (TDirectory* cwd, const std::string& dir_name) {
  TDirectory *dir;

  TKey *key = cwd->FindKey (dir_name.c_str ());
  if (key) { 
    dir = dynamic_cast<TDirectory*>(key->ReadObj ());
  } else {
    dir = cwd->mkdir (dir_name.c_str ());
  }

  return dir;
}

TObject* bx_root_barn::m_get_obj (const root_item& item) {
  TObject *obj = gROOT->FindObjectAny (item.get_name ().c_str ());

  if (!obj) {
    bx_message msg(bx_message::warn);
    msg << "bx_root_barn: object " << item.get_name () << " owned by " << item.get_owner_module_name () << " module disappeared" << dispatch; 
  }
  
  return obj;
}

void bx_root_barn::m_obj_write (TDirectory *parent_dir, const root_item& item) {
  TObject *obj = m_get_obj (item);
  if (!obj) return;

  TDirectory* dir = m_get_dir (parent_dir, item.get_owner_module_name ());
  
  dir->cd ();
  obj->Write ();
  dir->Save ();

  bx_message msg(bx_message::debug);
  msg << "bx_root_barn: object " << obj->GetName () << " written" << dispatch;
}

 
void bx_root_barn::write (TFile* file) {

  
  TDirectory *test_dir = 0; 
  if (std::count_if (items.begin (), items.end (), check_type  (bx_root_barn::test))) 
    test_dir = m_get_dir (file, "Test_Results");
  

  std::list<root_item>::const_iterator item;
  for (item = items.begin (); item != items.end (); item++) {
    file->cd ();
    if (item->get_type () == bx_root_barn::test) {
      m_obj_write (test_dir, *item);
    } else if (item->get_type () == bx_root_barn::file) {
      m_obj_write (file, *item);
    }
    file->Flush ();
  }
}

root_item::root_item (bx_root_barn::item_type type, TNamed *obj, const bx_base_module *module): i_type(type), 
  name(obj->GetName ()), owner_module_name(module->get_name ()) {}

/*
 * $Log: bx_root_barn.cc,v $
 * Revision 1.3  2004/05/21 11:34:48  razeto
 * Disabled root signal handlers
 *
 * Revision 1.2  2004/05/21 08:34:52  razeto
 * Updated to working
 *
 * Revision 1.1  2004/05/18 15:07:01  razeto
 * Added
 *
 */
