/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: vdt.cc,v 1.11 2004/06/14 11:51:59 razeto Exp $
 *
 * Implementation of vdt
 *
 */
#include "vdt.hh"
#include "messenger.hh"

#include <sstream>
#include <stdexcept>
#include <iterator>
#include <ctype.h>
#include <stdlib.h>

vdt::vdt_type vdt::m_search_type (const std::string& str) {
  bool is_float = false;
  bool is_hex = false;
  std::string::size_type i = 0;

  for (; i < str.size (); i++) {
    register char c = str[i];
    if (isdigit (c)) continue;                  // continue on digit
    else if (c == '-' && i == 0) continue;	// negative numbers
    else if (is_hex && isxdigit (c)) continue;  // hex digit are good is is an hex number
    else if (c == 'x') {                        // Allow 0x hex syntax
      if (i != 1 || is_hex) break;
      else is_hex = true;
    } else if (c == '.') {                      // On float only a "." is allowed
      if (is_float) break;
      else is_float = true;
    } else break;
  }

  if (str[0] == '{' && str[str.size () - 1] == '}') return vector_vdt;
  if (i != str.size () || !str.size ()) return string_vdt;  // Use string type on null string 
  else if (is_float) return float_vdt;
  else return int_vdt;
}

void vdt::m_assign_value (const std::string& str) {
  bx_message msg(bx_message::critic, "vdt: ");
  if (name_s.size ()) msg << "\"" << name_s << "\" ";

  char *error_ptr;
  switch (type_internal) {
    case vdt::string_vdt:
      value_s = str;
      break;
    case vdt::float_vdt:
      value_f = ::strtod (str.c_str (), &error_ptr);
      if (value_f == 0 && error_ptr == str) msg << "float conversion failed for " << str << dispatch;
      break;
    case vdt::int_vdt:
      value_i = ::strtol (str.c_str (), &error_ptr, 0);  // 0 means autodetect the base, which is fine for 0x starting hex
      if (value_i == 0 && error_ptr == str) msg << "integer conversion failed for \"" << str << "\"" << dispatch;
      break;
    case vdt::vector_vdt:
      do {
	  // Fill the vector
        std::string sub_str = str.substr (1, str.size () - 2);  	// copy to local (modificable) discarding '{' and '}'
        while (sub_str.size ()) {
          std::string::size_type pos = sub_str.find (',', pos);  	// Find next ','

          if (pos == std::string::npos) pos = sub_str.size () - 1; 	// If no more ',' are present sub_str is the last field

          std::string::size_type start_field = 0;			// look for spaces around the element like " 12 ,"
	  for (; start_field < sub_str.size (); start_field++) if (!isspace (sub_str[start_field])) break;
          std::string::size_type end_field = pos - 1;
	  for (; end_field > 0; end_field--) if (!isspace (sub_str[end_field])) break;

	  std::string value = sub_str.substr (start_field, end_field);	// Copy the value

	  if (!value.size ()) break;					// Stop on last void element

	  value_v.push_back (vdt (value, name_s));			// Fill the vector

	  sub_str.erase (0, pos + 1);  					// Move forward
        }	
      } while (0); // This do while is present to allow the declaration of variables
      break;
    default:
      msg << "internal vdt error: cannot call m_assign_value with a undefined type value" << dispatch;
  }
} 


vdt::vdt (const std::string& value, const std::string& name, vdt_type type): name_s(name) {
  vdt_type tmp_type = m_search_type (value);

  if (type != unassigned_vdt && tmp_type != type) {
    bx_message msg(bx_message::critic, "vdt: ");
    if (name_s.size ()) msg << "\"" << name_s << "\" ";
    msg << "requested " << char(type) << "type, while \"" << value << "\" has type " << char(tmp_type) << dispatch;
  }

  type_internal = tmp_type;
  m_assign_value (value);
}

const vdt& vdt::operator= (const std::string& value) {
  vdt_type tmp_type = m_search_type (value);

  if (type_internal != unassigned_vdt) m_check_type (tmp_type);

  type_internal = tmp_type;
  m_assign_value (value);

  return *this;
}

const vdt& vdt::operator= (long int value) {
  
  if (type_internal != unassigned_vdt) m_check_type (vdt::int_vdt);

  type_internal = int_vdt; 
  value_i = value; 
  
  return *this; 
}
  
const vdt& vdt::operator= (double value) {
  
  if (type_internal != unassigned_vdt) m_check_type (vdt::float_vdt);

  type_internal = float_vdt; 
  value_f = value; 
  
  return *this; 
}

long int vdt::get_int () const {
  m_check_type (vdt::int_vdt);

  return value_i;
}

double vdt::get_float () const {
  m_check_type (vdt::float_vdt);

  return (type_internal == float_vdt) ? value_f : value_i;	// int->float cast always allowed
}
  
const std::string& vdt::get_string () const {
  m_check_type (vdt::string_vdt);

  return value_s;
}

const vdt::vdt_vector& vdt::get_vector () const {
  m_check_type (vdt::vector_vdt);

  return value_v;
}

void vdt::m_check_type (vdt_type new_type) const {

  if (type_internal == new_type) return;
  
  if (new_type == float_vdt && type_internal == int_vdt) return;	// int->float cast always allowed
  
  bx_message msg(bx_message::critic, "vdt: ");
  if (name_s.size ()) msg << "\"" << name_s << "\" ";
  msg << "requested " << char(new_type) << " while curren type is " << char(type_internal) << " (" << *this << ") cast not allowed" << dispatch;
}


std::ostream& operator<< (std::ostream& out, const vdt& v) {
  switch (v.get_type ()) {
    case vdt::int_vdt:
      out << v.get_int ();
      break;
    case vdt::float_vdt:
      out << v.get_float ();
      break;
    case vdt::string_vdt:
      out << v.get_string ();
      break;
    case vdt::vector_vdt:
      std::copy (v.get_vector ().begin (), v.get_vector ().end (), std::ostream_iterator<vdt>(out, " "));
      break;
    default:
      out << "unknown type"; 
  }
  return out;
}
  
/*
 * $Log: vdt.cc,v $
 * Revision 1.11  2004/06/14 11:51:59  razeto
 * Fixed a stamp
 *
 * Revision 1.10  2004/06/01 15:36:14  razeto
 * Upgrade to full support vdt vectors
 *
 * Revision 1.9  2004/06/01 09:09:25  razeto
 * Added array support (with { 1, 2, 3, 4 } syntax)
 *
 * Revision 1.8  2004/05/26 09:20:47  razeto
 * Upgraded: now a vdt object typically knows its name (usefull for error reporting)
 *
 * Revision 1.7  2004/05/25 17:14:27  razeto
 * Upgraded
 *
 * Revision 1.6  2004/05/18 14:22:48  razeto
 * upgraded
 *
 * Revision 1.5  2004/04/09 07:48:19  razeto
 * Fixed a bug for negative numbers. Added int->float read casting
 *
 * Revision 1.4  2004/04/05 13:09:07  razeto
 * Added messanger for exception handling. Updated a comment
 *
 * Revision 1.3  2004/04/01 12:05:01  razeto
 * Fixed a bug in the assignement operator.
 * Added a ostreamer function.
 * Added stupid assignement operator since casting is not done.
 *
 * Revision 1.2  2004/03/29 13:23:57  razeto
 * Added the float/int int/float casting
 *
 * Revision 1.1  2004/03/29 13:14:12  razeto
 * Added interface directory (where options and dbi reside).
 * Added vdt object for storing parameters
 *
 */
