/* BOREXINO Reconstruction program
 *
 * Author: Sergey Sukhotin <sukhotin@cdf.in2p3.fr>
 *
 * $Id: bx_fadc_decoder.cc,v 1.4 2004/08/26 14:32:39 sukhotin Exp $
 *
 * Decoder for the FADC raw event
 */
 
#include "bx_fadc_decoder.hh"
#include "fadc_trigger_time.hh"
#include "fadc_channel.hh"
#include "fadc_sum.hh"
#include "fadc_calc.hh"
#include "bx_root_barn.hh"
#include "messenger.hh"

#include <vector>
#include <algorithm>
#include <functional>
#include <numeric>

bx_fadc_decoder::bx_fadc_decoder (): bx_base_module("bx_fadc_decoder", bx_base_module::main_loop) {
  require_event_stage (bx_base_module::fadc, bx_base_event::raw);
}

void bx_fadc_decoder::begin () {
  get_message(bx_message::debug) << "begin" << dispatch;
  hramp  = new TH1F("Trg_Ramp","Trigger Ramp Time",100,-100,0);
  hnwin  = new TH1F("nwin","Number of windows per Event",32,0,32);
  hwflag = new TH1F("wflag","Windows Flag",16,0,16);
  hwlength = new TH1F("wlength","Windows Length",1000,0,1000);
  hsamp = new TH1F("DSum_Amp","DSum Amplitude",2000,0,20000);
  hspeak = new TH1F("DSum_Peak","DSum Peak Position",200,0,200);
  hscharge = new TH1F("DSum_Charge","DSum Charge/10.0",2000,0,20000);
  hsshape = new TH1F("DSum_PS","DSum Pulse Shape",1000,0,1000);
  for(int kk=0; kk<102; kk++)
  {
    sprintf(hname,"Ch_%3d_A",kk);
    sprintf(title,"Max_Ampl in Ch%3d",kk);
    hwamp[kk] = new TH1F(hname,title,256,0,256);
    bx_root_barn::get ()->store (bx_root_barn::test, hwamp[kk], this);
  }
  for(int kk=0; kk<102; kk++)
  {
    sprintf(hname,"Ch_%3d_C",kk);
    sprintf(title,"Charge in Ch%3d",kk);
    hwcharge[kk] = new TH1F(hname,title,250,0,2500);
    bx_root_barn::get ()->store (bx_root_barn::test, hwcharge[kk], this);
  }
  for(int kk=0; kk<102; kk++)
  {
    sprintf(hname,"Ch_%3d_P",kk);
    sprintf(title,"Peak Position in Ch%3d",kk);
    hwpeak[kk] = new TH1F(hname,title,100,0,100);
    bx_root_barn::get ()->store (bx_root_barn::test, hwpeak[kk], this);
  }
  bx_root_barn::get ()->store (bx_root_barn::test, hramp,    this);
  bx_root_barn::get ()->store (bx_root_barn::test, hnwin,    this);
  bx_root_barn::get ()->store (bx_root_barn::test, hwflag,   this);
  bx_root_barn::get ()->store (bx_root_barn::test, hwlength, this);
  bx_root_barn::get ()->store (bx_root_barn::test, hsamp,    this);
  bx_root_barn::get ()->store (bx_root_barn::test, hspeak,   this);
  bx_root_barn::get ()->store (bx_root_barn::test, hscharge, this);
  bx_root_barn::get ()->store (bx_root_barn::test, hsshape,  this);
}
//-----------------------------------------------------------------
bx_reco_event* bx_fadc_decoder::doit (bx_reco_event *ev) {
  bx_trigger_event& et = ev->get_trigger_nonconst ();
  bx_fadc_event&    ef = ev->get_fadc_nonconst ();
  bx_fadc_decoded_event& e = dynamic_cast<bx_fadc_decoded_event&>(ef);

  unsigned char ttype = et.get_trgtype ();
  e.u1_num_windows = 0;
  e.u1_event_flag  = ef.get_flag_header ();
  e.f4_time_to_previous = ef.get_fadc_time ();

  unsigned char counter = 1;
  for (unsigned char i = 0; i < ef.get_num_windows (); i++) {
    const bx_fadc_raw_window &win  = ef.get_window (i);

    if (!i) {
      e.f4_trigger_time = fadc_trigger_time(win).get_ramp_time ();
      if (ttype == 1) hramp->Fill(e.f4_trigger_time);
    } else {
        fadc_sum dsum(win, ttype);
        unsigned char np = dsum.get_npulses();
        for (unsigned char j = 0; j < np; j++) {
          e.u1_num_windows += np;
          bx_fadc_decoded_window  dwin;
          dwin.u1_win_number = counter;
          dwin.u1_win_flag   = win.get_win_flag ();
          unsigned int first = dsum.get_first (j); 
          unsigned int last  = dsum.get_last (j);
          dwin.u2_win_begin  = win.get_win_begin () - first;
          dwin.u2_win_size   = last - first;
          dwin.f4_sum_ampl   = dsum.get_ampl (j);
          dwin.f4_sum_peak   = dsum.get_peak (j);
          dwin.f4_sum_charge = dsum.get_charge (j);
          for(unsigned int k = first; k < last; ++k) {
            if (ttype == 1) hsshape->Fill(k,dsum.get_sample (k));
            dwin.sum_samples.push_back (dsum.get_sample (k));
          }
          for (unsigned char l = 0; l < win.get_nchannels (); l++) {
            const bx_fadc_raw_channel &ch = win.get_channel (l);
            fadc_channel r_ch(ch, first, last);
            dwin.channels_ampl.push_back (r_ch.get_ampl ());
            dwin.channels_charge.push_back (r_ch.get_charge ());
            dwin.channels_peak.push_back (r_ch.get_peak ());
            if (ttype == 1) {
              hwpeak[l]->Fill(dwin.channels_peak[l]);
              hwamp[l]->Fill(dwin.channels_ampl[l]);
              hwcharge[l]->Fill(dwin.channels_charge[l]);
            }
          }
          if (ttype == 1) {
            hwflag->Fill(dwin.u1_win_flag);
            hwlength->Fill(dwin.u2_win_size);
            hsamp->Fill(dwin.f4_sum_ampl);
            hscharge->Fill(dwin.f4_sum_charge);
            hspeak->Fill(dwin.f4_sum_peak);
          }
          e.decoded_win.push_back (dwin);
          counter++;
        }
      } // end of else ...
  } // end of window cycle ...
  if (ttype == 1) hnwin->Fill(e.u1_num_windows);
  return ev;
}

void bx_fadc_decoder::end () {}

/*
 * $Log: bx_fadc_decoder.cc,v $
 * Revision 1.4  2004/08/26 14:32:39  sukhotin
 * An algorithm to search a possible second pulse in a window
 * is implemented.
 *
 * Revision 1.3  2004/08/06 10:30:28  razeto
 * cycle_1 branch merged in the main trunk.
 *
 * Revision 1.2.2.2  2004/07/26 13:01:13  sukhotin
 * do{ ...} while(0) is removed.
 *
 * Revision 1.2.2.1  2004/07/23 13:55:30  sukhotin
 * Removed the bug. "require_event_stage(...)" is added to the constructor.
 *
 * Revision 1.2  2004/07/13 16:18:28  sukhotin
 * The code indentation is improved. Some more comments are added.
 * function.h is changed for functional
 *
 */

