/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: bx_laben_decoder.cc,v 1.10 2004/09/13 10:11:31 razeto Exp $
 *
 * Implementation of bx_laben_decoder
 *
 */
#include "bx_laben_decoder.hh"
#include "messenger.hh"
#include "laben_time_hit.hh"
#include "laben_charge_hit.hh"
#include "constants.hh"
#include "bx_dbi.hh"
#include "db_channel.hh"
#include "cmap.hh"
#include "bx_root_barn.hh"

#include <map>
#include <algorithm>


// ctor
bx_laben_decoder::bx_laben_decoder (): bx_base_module("bx_laben_decoder", bx_base_module::main_loop) {
  require_event_stage (bx_base_module::laben, bx_base_event::raw);
}

// module interface
void bx_laben_decoder::begin () {
  get_message(bx_message::debug) << "begin" << dispatch;


  p_disabled_lg = new bool[constants::laben::channels + 1];
  std::fill_n (p_disabled_lg, constants::laben::channels + 1, false);
  const vdt::vdt_vector& v = get_parameter("disable_lg").get_vector ();
  for (unsigned i = 0; i < v.size (); i++) p_disabled_lg[v[i].get_int ()] = true;
  b_discard_out_of_gate_hits = bool (get_parameter ("discard_out_of_gate_hits").get_int ());
  b_warn_empy_channel = bool (get_parameter ("warn_empy_channel").get_int ());

  trigger_charge = new TH1F ("trigger_charge", "Trigger hit charge values", 255, 0, 255);
  laser_charge = new TH1F ("laser_charge", "Laser hit charge values", 255, 0, 255);
  bx_root_barn::get ()->store (bx_root_barn::test, trigger_charge, this);
  bx_root_barn::get ()->store (bx_root_barn::test, laser_charge, this);
  
  
}

struct out_of_gate_checker {
  out_of_gate_checker (double low_bound, double high_bound): f8_low_bound(low_bound), f8_high_bound(high_bound) {}
  bool operator() (const bx_laben_decoded_hit& hit) { return (hit.get_raw_time () >= f8_low_bound && hit.get_raw_time () <= f8_high_bound); }
  double f8_low_bound, f8_high_bound;
};

bx_reco_event* bx_laben_decoder::doit (bx_reco_event *ev) {
  bx_laben_event& er = ev->get_laben_nonconst();
  bx_laben_decoded_event& e = dynamic_cast<bx_laben_decoded_event&>(er);

    // Check for gray counter cross in the trigger gate.
  bool gray_cross = m_check_gray_cross (er);

    // 1) Create the matrix and an iterator
  std::map <int, bx_laben_decoded_hit::bx_laben_decoded_hit_vector> channel_hits_map;
  std::map <int, bx_laben_decoded_hit::bx_laben_decoded_hit_vector>::iterator it;

    // 2) Fill the matrix (even fill the time since the laben_time_hit is create to check if the hit is valid)
  for (int i = 0; i < er.get_raw_nhits (); i++) {
    const bx_laben_raw_hit &hit = er.get_raw_hit (i);
    int lg = hit.get_logical_channel ();

      // Ignore disabled channels
    if (p_disabled_lg[lg]) continue;

      // Create the laben time hit
    laben_time_hit t_hit(hit, gray_cross);

      // Skip invalid hits
    if (!t_hit.is_valid ()) continue;

      // Create a decoded hit
    bx_laben_decoded_hit decoded_hit(hit);

      // Associate the db_channel
    decoded_hit.p_db_channel = &bx_dbi::get ()->get_channel (lg);

      // Dump a severe error if a hit is present on an empty channel
    if (b_warn_empy_channel && !ev->get_trigger ().is_pulser () && decoded_hit.get_db_channel ()->is_empty ()) get_message (bx_message::error) << "hit on empty channel " << lg << dispatch;

      // Fill the time data (charge will be filled later)
    decoded_hit = t_hit;

      // Push the decoded hit in the column
    channel_hits_map[lg].push_back (decoded_hit);
  }

    // 3) Process several time the matrix looping on every column
    // 3a) Fill the charge data
  for (it = channel_hits_map.begin (); it != channel_hits_map.end (); it++) {
    bx_laben_decoded_hit::bx_laben_decoded_hit_vector &c_vector = it->second;

      // Sort in time the column vector (all hits have the same channel)
    std::sort (c_vector.begin (), c_vector.end ());

      // Fill the charge data: the first is special since ho previus hits 
      // can be overlapped, so use the simple laben_charge_hit ctor.
    bx_laben_decoded_hit& hit = c_vector[0];
    laben_charge_hit c_hit(hit);
    hit = c_hit;
      // If other elements are present use the laben_charge_hit with 2 raw events
    for (unsigned long i = 1; i < c_vector.size (); i++) {
      bx_laben_decoded_hit& prev_hit = c_vector[i - 1];
      bx_laben_decoded_hit& this_hit = c_vector[i];
      laben_charge_hit c_hit (this_hit, prev_hit);
      this_hit = c_hit;
    } 
  }
    // 3b) Fill internal vector used by m_get_moda_mean
  std::vector<double> trigger_times, laser_times;
  for (it = channel_hits_map.begin (); it != channel_hits_map.end (); it++) {
    bx_laben_decoded_hit::bx_laben_decoded_hit_vector &c_vector = it->second;
    const db_channel* ch_info = c_vector[0].get_db_channel ();
    for (unsigned int i = 0; i < c_vector.size (); i++) {
      if (ch_info->is_trigger ()) {
	trigger_times.push_back (c_vector[i].get_raw_time ());
	trigger_charge->Fill (c_vector[i].get_uncorrected_charge_bin ());
      } else if (ch_info->is_laser ()) {
	laser_times.push_back (c_vector[i].get_raw_time ());
	laser_charge->Fill (c_vector[i].get_uncorrected_charge_bin ());
      }
    }
  }
    // And calculate the times
  e.f8_trigger_rawt = m_get_moda_mean (trigger_times);
  e.f8_laser_rawt = m_get_moda_mean (laser_times);

    // 3c) Remove out of gate hits
  double low_bound = e.get_trigger_rawt () - 5000;
  double high_bound = e.get_trigger_rawt () + 5000;
  for (it = channel_hits_map.begin (); it != channel_hits_map.end () && b_discard_out_of_gate_hits; it++) {
    bx_laben_decoded_hit::bx_laben_decoded_hit_vector &c_vector = it->second;
    std::remove_if (c_vector.begin (), c_vector.end (), out_of_gate_checker (low_bound, high_bound));
  }

    // 3d) Calculate the vector lenghts
  int std_decoded_nhits = 0, trigger_decoded_nhits = 0, laser_decoded_nhits = 0;
  for (it = channel_hits_map.begin (); it != channel_hits_map.end (); it++) {
    bx_laben_decoded_hit::bx_laben_decoded_hit_vector &c_vector = it->second;
    if (c_vector.size ()) {
      const db_channel* ch_info = c_vector[0].get_db_channel ();
      if (ch_info->is_ordinary ()) std_decoded_nhits += c_vector.size ();
      else if (ch_info->is_trigger ()) trigger_decoded_nhits += c_vector.size ();
      else if (ch_info->is_laser ()) laser_decoded_nhits += c_vector.size ();
    }
  }
    // Ask to the decoded hit vectors to reserve the required space (avoiding too many relocations)
  e.decoded_hits.reserve (std_decoded_nhits);
  e.trigger_reference_decoded_hits.reserve (trigger_decoded_nhits);
  e.laser_reference_decoded_hits.reserve (laser_decoded_nhits);

    // 3e) Finally fill the vectors
    // Now process the matrix looping on every column
  for (it = channel_hits_map.begin (); it != channel_hits_map.end (); it++) {
    bx_laben_decoded_hit::bx_laben_decoded_hit_vector &c_vector = it->second;

      // Add the column to the event decoded_hit list.
    e.decoded_hits.insert (e.decoded_hits.end (), c_vector.begin (), c_vector.end ());
  }

    // Sort the list in time  
  std::sort (e.decoded_hits.begin (), e.decoded_hits.end ());
  
  er.mark_stage (bx_base_event::decoded);
  return ev;
}

void bx_laben_decoder::end () {
  delete [] p_disabled_lg;

  get_message(bx_message::debug) << "end" << dispatch;
}

  // Collect statistic to sse if we are in a gray cross boundary
bool bx_laben_decoder::m_check_gray_cross (const bx_laben_event& er) {
  int n_cross = 0;
  for (int i = 0; i < er.get_raw_nhits (); i++) {
    const bx_laben_raw_hit &hit = er.get_raw_hit (i);

      // Create the laben time hit
    laben_time_hit t_hit(hit);
    
      // Skip invalid hits
    if (!t_hit.is_valid ()) continue;
    
      // Check if the hit is close to a crossing reagion, the increment ncross
    if (t_hit.is_gray_crossing_window ()) n_cross ++;
  }
    // a gray cross window is present if 70% of hits are in the window
  return float (n_cross)  >= float (er.get_raw_nhits ()) * .7;
}

#if 0
  // Fill the time reference vectors; using cmap for times allow to have only one hit
  // for each channel (the first in time).
void bx_laben_decoder::m_fill_reference_time_vector (bx_laben_event& er, bool cross) {
  bx_laben_decoded_event& e = dynamic_cast<bx_laben_decoded_event&>(er);

    // Make a map for trigger/laser channels
  std::cmap <int, double> trigger_channel_time_references, laser_channel_time_references;

  for (int i = 0; i < er.get_raw_nhits (); i++) {
    const bx_laben_raw_hit &hit = er.get_raw_hit (i);

      // Create the laben time hit
    laben_time_hit t_hit(hit, cross);
    
      // Skip invalid hits
    if (!t_hit.is_valid ()) continue;
    
      // Get the db_channel info
    int lg = hit.get_logical_channel ();
    const db_channel& ch_info = bx_dbi::get ()->get_channel (lg);

      // Skip ordinary channels and user disabled channels
    if (ch_info.is_ordinary ()) continue;
    if (std::find (disabled_lg_v.begin (), disabled_lg_v.end (), lg) != disabled_lg_v.end ()) continue; 

      // Do stuff only on trigger/laser channels
    if (ch_info.is_trigger ()) {
      double rawt = t_hit.get_time (); 
      if (trigger_channel_time_references.check (lg) && trigger_channel_time_references[lg] < rawt) continue;
      trigger_channel_time_references[lg] = rawt;
    } else if (ch_info.is_laser ()) {
      double rawt = t_hit.get_time (); 
      if (laser_channel_time_references.check (lg) && laser_channel_time_references[lg] < rawt) continue;
      laser_channel_time_references[lg] = rawt;
    }
  }
    // Fill the event vectors
  std::map <int, double>::iterator it;
  for (it = trigger_channel_time_references.begin (); it != trigger_channel_time_references.end (); it ++)
    e.trigger_time_v.push_back (bx_laben_decoded_event::time_reference_element(it->first, it->second));
  for (it = laser_channel_time_references.begin (); it != laser_channel_time_references.end (); it ++)
    e.laser_time_v.push_back (bx_laben_decoded_event::time_reference_element(it->first, it->second));
}

void bx_laben_decoder::m_fill_reference_time_values (bx_laben_event& er) {
  bx_laben_decoded_event& e = dynamic_cast<bx_laben_decoded_event&>(er);

  if (e.trigger_time_v.size ()) {
    double mean = m_get_moda_mean (e.trigger_time_v);
    e.f8_trigger_rawt = mean;
  } else e.f8_trigger_rawt = 0;

  if (e.laser_time_v.size ()) {
    double mean = m_get_moda_mean (e.laser_time_v);
    e.f8_laser_rawt = mean;
  } else e.f8_laser_rawt = 0;
  
}
#endif
double bx_laben_decoder::m_get_moda_mean (const std::vector<double>& v) {
  if (!v.size ()) return 0;

  double *times = new double[v.size ()];
  for (unsigned i = 0; i < v.size (); i++) times[i] = v[i];
  std::sort (times, times + v.size ());
  
  double moda = m_search_moda (times, v.size ());
  double sum = 0;
  double count = 0;
  for (unsigned i = 0; i < v.size (); i++) {
    if (times[i] < (moda - 5) || times[i] > (moda + 5)) continue;
    sum += times[i];
    count ++;
  }
  
  delete [] times;
  return count ? sum / count : 0.;
}

double bx_laben_decoder::m_search_moda (const double *times, unsigned short size) {
  if (!size) return 0;
  if (size == 1) return times[0];
  
  double min = times[0];
  double max = times[size - 1];
  double dt = max - min;
  if (dt < 10) return (min + max) / 2; // If the are too close use simple interpolation

  unsigned char *histo = new unsigned char[int(dt / 5) + 1];
  std::fill (histo, histo + int(dt / 5), 0);
  
  for (unsigned i = 0; i < size; i++) {
    int bin = int((times[i] - min) / 5);
    histo[bin]++;
  }

  int moda_bin = std::max_element (histo, histo + int(dt / 5) + 1) - histo;
  
  delete [] histo;
  return moda_bin * 5 + min;
}
/*
 * $Log: bx_laben_decoder.cc,v $
 * Revision 1.10  2004/09/13 10:11:31  razeto
 * Removed some debug instructions and fixed some typos
 *
 * Revision 1.9  2004/09/09 11:53:18  razeto
 * Doit method mainly rewritten; now several loops on the hit matrix
 * are done now to allow a better (and hopefully cleaner) decoding.
 * New decoded event format introduced (to fully have time reference
 * hits).
 * Decoder is now almost ready to recognise time reference hits from
 * predecoding hits using the charge samples (for runid > 1800).
 *
 * Revision 1.8  2004/07/14 11:49:00  razeto
 * Fixed a bug (array overflow) and upgraded some routines
 *
 * Revision 1.7  2004/06/22 16:04:12  razeto
 * Fixed a bug
 *
 * Revision 1.6  2004/06/08 10:25:19  razeto
 * Updated to new hit name (rawt -> raw_time)
 *
 * Revision 1.5  2004/06/07 09:39:41  razeto
 * Removed the time_sort functor, using decoded hit operator<.
 * Added a global time sorting for the decoded hits.
 *
 * Revision 1.4  2004/06/01 15:44:48  razeto
 * Added event stage marking
 *
 * Revision 1.3  2004/06/01 15:38:25  razeto
 * A lot of development to support time references, out_of_gate hits and much more
 *
 * Revision 1.2  2004/05/21 11:28:23  razeto
 * Updated conforming to the new conventions of event::get_logical_channel
 *
 * Revision 1.1  2004/05/21 08:40:33  razeto
 * Added
 *
 */
