/* BOREXINO Reconstruction program
 *
 * Author: Daniela Manuzio <dmanuzio@ge.infn.it>
 *
 * $Id: bx_laben_electronics_status.cc,v 1.1 2004/07/22 09:57:50 dmanuzio Exp $
 *
 * 
 * Implemenentation of bx_laben_electronics_status
 * Module that studies the status of the inner detector
 * electronics, looking for bad, problematic or dead channels.
 * Input: the decoded hits
 * Output: some histos and a list of bad channels
 * 
*/

#include <cmath>
#include <iostream>
#include <vector>
#include <algorithm>
#include "bx_laben_electronics_status.hh"
#include "bx_reco_event.hh"
#include "bx_dbi.hh"
#include "db_profile.hh"
#include "db_channel.hh"
#include "bx_root_barn.hh"

bx_laben_electronics_status::bx_laben_electronics_status() : bx_base_module("bx_laben_electronics_status", bx_base_module::main_loop) {
  require_event_stage (bx_base_module::laben, bx_base_event::decoded);
}
      
void bx_laben_electronics_status::begin () {

  // Configuration parameters
  f_dead_channel  = get_parameter ("dead_channel_thresh").get_float ();
  f_multiple_hits = get_parameter ("multiple_hits_thresh").get_float ();
  f_zero_charge   = get_parameter ("zero_charge_thresh").get_float ();
  f_0xFF_charge   = get_parameter ("0xFF_charge_thresh").get_float ();
  f_mean_charge   = get_parameter ("mean_charge_cut").get_float ();
  
#ifdef ENABLE_TEST_ROOT_OUTPUT
  hit_map = new TH1F ("hit_map", "bx_laben_electronics_status hit map", 2240, 1, 2241);
  base_map = new TH2F ("base_map","bx_laben_electronics_status base map",2240, 1, 2241, 256, 0, 256);
  peak_map = new TH2F ("peak_map","bx_laben_electronics_status peak map",2240, 1, 2241, 256, 0, 256);
  multiplicity_map = new TH2F ("multiplicity_map","bx_laben_electronics_status multiplicity map",2240, 1, 2241, 10, 0, 10);
  multiplicity_dt  = new TH2F ("multiplicity_dt","bx_laben_electronics_status multiplicity dt",2240, 1, 2241, 7000, 0, 700);

  bx_root_barn::get ()->store (bx_root_barn::test, hit_map, this);
  bx_root_barn::get ()->store (bx_root_barn::test, base_map, this);
  bx_root_barn::get ()->store (bx_root_barn::test, peak_map, this);
  bx_root_barn::get ()->store (bx_root_barn::test, multiplicity_map, this);
  bx_root_barn::get ()->store (bx_root_barn::test, multiplicity_dt, this);
#endif

  // Internal vectors and arrays
  hits = new int[constants::laben::channels];
  time = new double[constants::laben::channels];
  out_gate = new int[constants::laben::channels];
  multiple_hits = new int[constants::laben::channels];
  
  std::fill_n (hits,constants::laben::channels,0);
  std::fill_n (time,constants::laben::channels,0.);
  std::fill_n (out_gate,constants::laben::channels,0);
  std::fill_n (multiple_hits,constants::laben::channels,0);

  precalib_events=0;
  get_message(bx_message::debug) << "begin" << dispatch;

}

bx_reco_event* bx_laben_electronics_status::doit (bx_reco_event *ev) {

  if ( ev->get_trigger().is_pulser() ) {
    
    precalib_events++;
    const bx_laben_event& er = ev->get_laben();

    for (int i = 0; i < er.get_decoded_nhits (); i++) {
      const bx_laben_decoded_hit &dhit = er.get_decoded_hit (i);
      const bx_laben_raw_hit &rhit = dhit.get_raw_hit();
      int lg = rhit.get_logical_channel()-1;
      multiple_hits[lg]++;
      hits[lg]++;

      if ( dhit.is_out_of_gate() ) out_gate[lg]++;
	
#ifdef ENABLE_TEST_ROOT_OUTPUT
      hit_map->Fill(lg+1);
      base_map->Fill(lg+1,rhit.get_base());
      peak_map->Fill(lg+1,rhit.get_peak());
      
      if ( multiple_hits[lg]>1 && multiple_hits[lg]<5 ) {
	double dt = dhit.get_raw_time() - time[lg];
	multiplicity_dt->Fill(lg+1,(float)dt);
      }
      time[lg] = dhit.get_raw_time();
#endif
    }

#ifdef ENABLE_TEST_ROOT_OUTPUT
    for (int i = 0; i < constants::laben::channels; i++) multiplicity_map->Fill( i+1, multiple_hits[i] );
#endif
  }
  
  return ev;  
}


void bx_laben_electronics_status::end () {
  
  if ( precalib_events ) {
    int dead_channels=0;

#ifdef ENABLE_TEST_ROOT_OUTPUT
    // Mean and rms for the base-charge global distribution
    TH1D *proj = base_map->ProjectionY("proj");
    proj->SetAxisRange(1,254);
    float mean_ref = proj->GetBinCenter(proj->GetMaximumBin());
    proj->SetAxisRange(mean_ref-40,mean_ref+40);
    float sigma_ref = proj->GetRMS();
    proj->Delete();
    get_message(bx_message::warn) << "Global distrib: " << mean_ref << " , " << sigma_ref << dispatch;
#endif

    // Status of the single electronic channels 
    for (int i = 0; i < constants::laben::channels; i++) {
      const db_channel& channel_info = bx_dbi::get()->get_channel(i+1);
      if ( !channel_info.is_trigger() && !channel_info.is_laser() ) {

        if ( hits[i]/(float)precalib_events < f_dead_channel ) {
  	  dead_channels++;
          get_message(bx_message::warn) << "Channel " << i+1 << " : DEAD" 
	      << " ( " << hits[i] << " in " << precalib_events << " ev )" << dispatch;
        }
       
        else if ( hits[i]/(float)precalib_events > f_multiple_hits ) {
	  get_message(bx_message::warn) << "Channel " << i+1 << " : multiple hits" 
	      << " ( " << hits[i] << " in " << precalib_events << " ev )" << dispatch;
        }
      
        else {
#ifdef ENABLE_TEST_ROOT_OUTPUT
	  TH1D *proj = base_map->ProjectionY("proj",i+1,i+1);
	  float zero = proj->Integral(1,1);
          float FF = proj->Integral(256,256);
          float tot = proj->Integral(0,256);
	
	  if ( (zero/tot) > f_zero_charge ) 
	    get_message(bx_message::warn) << "Channel " << i+1 << " : zero charge hits" << " ( " << (zero/tot) << " )" << dispatch;
	
	  else if ( (FF/tot) > f_0xFF_charge ) 
            get_message(bx_message::warn) << "Channel " << i+1 << " : 0xFF charge hits" << " ( " << (FF/tot) << " )" << dispatch;

	  else {
	    proj->SetAxisRange(1,254);
	    float mean = proj->GetBinCenter(proj->GetMaximumBin());
	  
	    if ( fabs(mean-mean_ref) > f_mean_charge )
              get_message(bx_message::warn) << "Channel " << i+1 << " : shifted charge" << " ( " << mean << " )" << dispatch;

            if ( proj->GetRMS() > 2*sigma_ref ) 
              get_message(bx_message::warn) << "Channel " << i+1 << " : broad charge" << " ( " << proj->GetRMS() << " )" << dispatch;
	  }
	  proj->Delete();
#endif
	}
      }
    }
  }
     
  // Delete vectors
  delete [] hits;
  delete [] time;
  delete [] multiple_hits;
  
  get_message(bx_message::debug) << "end" << dispatch;

}

/*
 * $Log: bx_laben_electronics_status.cc,v $
 * Revision 1.1  2004/07/22 09:57:50  dmanuzio
 * New module to test the electronics (laben and front end problems)
 *
 *
 */

