/* BOREXINO Reconstruction program
 *
 * Author: Daniela Manuzio <dmanuzio@ge.infn.it>
 *
 * $Id: bx_laben_findcluster.cc,v 1.5 2004/07/15 13:13:47 dmanuzio Exp $
 *
 * Implemenentation of bx_laben_findcluster
 * Module that finds the candidate fragments in the laben hit 
 * collection. No real splitting is done in this module.
 * Input:  the list of time ordered laben hits
 * Output: a list of bx_cluster objects
 *
 * Algorithm:
 * Candidate: 1. number of hits in 1 bin > (dark_thresh) 
 *            2. number of hits in 3 consecutive bins > (low_thresh)
 * First:     1. number of hits in 3 consecutive bins > (trg_thresh)
 *            2. right delay (trg_delay) with trigger reference time            
 * Following: 1. time distance from previous cluster > (time_separation)
 * 
*/

#include <cmath>
#include <iostream>
#include <vector>
#include <algorithm>
#include "bx_laben_findcluster.hh"
#include "bx_reco_event.hh"
#include "bx_dbi.hh"
#include "db_profile.hh"

bx_laben_findcluster::bx_laben_findcluster() : bx_base_module("bx_laben_findcluster", bx_base_module::main_loop) {
  require_event_stage (bx_base_module::laben, bx_base_event::decoded);
  
  // initialization
  binned_times.clear();
  start_cluster.clear();
  end_cluster.clear();
  double_event.clear();
  
}
      
void bx_laben_findcluster::begin () {
}

bx_reco_event* bx_laben_findcluster::doit (bx_reco_event *ev) {

  // Check on trigger type. It runs only on neutrino triggers.
  if ( ev->get_trigger().is_neutrino() ) {

    bx_laben_event& er = ev->get_laben_nonconst();
    bx_laben_clustered_event &ew = dynamic_cast<bx_laben_clustered_event&>(er);
  
    // initialization
    int number_clusters=0;
    binned_times.clear();
    start_cluster.clear();
    end_cluster.clear();
    double_event.clear();

    // number of decoded laben hits
    int nhits = er.get_decoded_nhits();

    // trigger threshold 
    int trg_thresh = (int)((ev->get_trigger().get_btb_threshold()&0xFF)*0.8);

    // mean number of dark noise hits in 16 ns  
    float dark_hits_bin = 0.1;   //FIXME: to be taken from data base
    int dark_thresh = (int)(2*(dark_hits_bin + 3*sqrt(dark_hits_bin)));
    if (dark_thresh<1) dark_thresh=1;
  
    // minimum threshold (50 keV or trigger threshold)
    int low_thresh = 20;	
    if ( low_thresh > trg_thresh ) low_thresh = trg_thresh;
  
    // delay time with laben trigger reference
    float trg_jitter = 200.;		
    float trg_delay = 2000.;	
    float trg_delay_shifted = 2500.;	

    // Lowest time separation between two clusters
    float time_separation = 150.;

    // initialization and filling of the binned time array (16ns bins)
    int number_bins = 438;
    for( int i=0; i<number_bins; i++ ) binned_times.push_back(0);
 
    for (int i=0; i <nhits; i++) {
      if ( !er.get_decoded_hit(i).is_out_of_gate() ) {
        float time = er.get_decoded_hit(i).get_raw_time() - er.get_trigger_rawt() + 6600.;
        int bin = (int)(time/16.);
        if ( bin>-1 && bin<number_bins ) binned_times[bin]++;
      }
    }

    // Search for the cluster candidates 
    for (int bin=0; bin<number_bins-2; bin++) {
      if ( binned_times[bin] > dark_thresh ) {
        int number_hits = binned_times[bin]+binned_times[bin+1]+binned_times[bin+2];
        
        if ( number_hits > low_thresh ) {
          float start_time = bin*16.;

	  if ( number_clusters == 0 ) {
	    float dt = fabs(start_time - trg_delay);
	    if ( number_hits > trg_thresh && dt < trg_jitter ) {
	      start_cluster.push_back(start_time);
	      end_cluster.push_back(start_time);
	      double_event.push_back(0);
	      number_clusters++;
	      bin+=2;
	    } 
	  } else if ( number_clusters < 3 ){
	    float dt = start_time - start_cluster[number_clusters-1];
	    if ( dt > time_separation ) {
	      start_cluster.push_back(start_time);
	      end_cluster.push_back(start_time);
	      double_event.push_back(0);
	      number_clusters++;
	      bin+=2;
	    } else {
	      double_event[number_clusters-1]=1;
	    }
	  } else {
	    get_message(bx_message::info) << "Found more than 3 candidate clusters for event " << 
	    ev->get_trigger().get_evid() << dispatch;
	    break;
	  }
        }
      } 
    }
  
    if ( !number_clusters )  get_message(bx_message::info) << "No candidate cluster found in event " << 
      ev->get_trigger().get_evid() << dispatch;
    else get_message(bx_message::debug) << "Found " << number_clusters << " candidate clusters in event " << 
      ev->get_trigger().get_evid() << dispatch;
  
    // Search for the end time of the clusters
    int cons_bins = 5;
    int end_thresh = (int)(cons_bins*dark_thresh);
    
    for ( int cluster=0; cluster<number_clusters; cluster++ ) {
      if ( cluster>0 ) cons_bins = 4;
      int start_bin = (int)(start_cluster[cluster]/16.);
      int fired = start_bin-1;
      int cons = 0;
      int sum = 0;
    
      for ( int bin=start_bin; bin<number_bins-cons_bins; bin++ ) {
	if ( binned_times[bin] <= dark_thresh ) {
	  if ( fired == bin-1 ) {
	    cons++;
	    sum+=binned_times[bin];
	    if ( cons == cons_bins-1 ) {
	      float this_time = (bin-cons_bins)*16.;
	      float diff_time = fabs(this_time-start_cluster[cluster]);
	      if ( sum <= end_thresh && diff_time > time_separation ) {
		end_cluster[cluster] = (bin-cons_bins)*16.;
		break;
	      }
	      else {
	        cons=0;
		bin-=cons;
	      }
	    }
	  }
	  else cons=0;
          fired = bin;
	  sum=binned_times[bin];
	}
      }
      
      if ( end_cluster[cluster] == start_cluster[cluster] ) {
        get_message(bx_message::warn) << "End of cluster " << cluster+1 << " not found for event " << 
        ev->get_trigger().get_evid() << dispatch;
	get_message(bx_message::info) << "Threshold: " << end_thresh << dispatch;
        end_cluster[cluster] = start_cluster[cluster] + 400.;
      }
    }

    // Fill the bx_laben_clustered_hit array with the hits' raw times
    for ( int cluster=0; cluster<number_clusters; cluster++ ) {
      start_cluster[cluster]-=16.;

      ew.clusters.push_back(bx_laben_cluster());
      bx_laben_cluster &c = ew.clusters[ew.clusters.size()-1];
    
      for( int hit=0; hit<nhits;hit++ ) {
        const bx_laben_decoded_hit &decoded_hit =er.get_decoded_hit(hit);
        float time = decoded_hit.get_raw_time() - er.get_trigger_rawt() + 6600.;
	
        if ( time > start_cluster[cluster] && time < end_cluster[cluster] ) {
	  c.clustered_hits.push_back( bx_laben_clustered_hit(decoded_hit) );
	  bx_laben_clustered_hit &clustered_hit = c.clustered_hits[c.clustered_hits.size()-1];
	  clustered_hit.f8_time = decoded_hit.get_raw_time(); 
        }
      }
    }
    er.mark_stage(bx_base_event::rough_clustered);
  } 

  return ev;  
}

void bx_laben_findcluster::end () {
}
/*
 * $Log: bx_laben_findcluster.cc,v $
 * Revision 1.5  2004/07/15 13:13:47  dmanuzio
 * Removed the bug. Test done!
 *
 * Revision 1.4  2004/07/15 13:12:51  dmanuzio
 * Linking error put on purpose to test EchidnaTest script
 *
 * Revision 1.3  2004/07/15 13:04:13  dmanuzio
 * Test done
 *
 * Revision 1.2  2004/07/15 13:03:29  dmanuzio
 * Test of a new utility
 *
 * Revision 1.1  2004/07/09 13:59:08  dmanuzio
 * Added 2 modules for Laben clustering
 *
 *
 */

