/* BOREXINO Reconstruction program
 *
 * Author: Daniela Manuzio <dmanuzio@ge.infn.it>
 *
 * $Id: bx_laben_findcluster_time.cc,v 1.2 2004/07/12 12:37:20 dmanuzio Exp $
 *
 * Implementation of bx_laben_findcluster_time
 * Module that computes the best start and end time of a cluster
 * Input: the list of bx_cluster objects
 * Output: update the bx_cluster objects variables
 * 
*/

#include <cmath>
#include <iostream>

#include "bx_laben_findcluster_time.hh"
#include "bx_reco_event.hh"
#include "bx_dbi.hh"
#include "db_profile.hh"
#include "db_channel.hh"
#include "bx_root_barn.hh"
#include "constants.hh"

bx_laben_findcluster_time::bx_laben_findcluster_time() : bx_base_module("bx_laben_findcluster_time", bx_base_module::main_loop) {
  require_event_stage (bx_base_module::laben, bx_base_event::rough_clustered);
}
      
void bx_laben_findcluster_time::begin () {
#ifdef ENABLE_TEST_ROOT_OUTPUT
  clusters_raw_times = new TH2F ("clusters_raw_times", "bx_laben_findcluster_time clusters raw times", 
      3, 1, 4, 7000, 0, 7000);
  clusters_mean_times = new TH2F ("clusters_mean_times", "bx_laben_findcluster_time clusters mean times", 
      3, 1, 4, 200, 0, 200);
  clusters_times = new TH2F ("clusters_times", "bx_laben_findcluster_time clusters times", 
      3, 1, 4, 700, -100., 600);
  clusters_hits = new TH2F ("clusters_hits", "bx_laben_findcluster_time clusters hits", 
      3, 1, 4, 1000, 0, 1000);
  clusters_pmts = new TH2F ("clusters_pmts", "bx_laben_findcluster_time clusters pmts", 
      3, 1, 4, 1000, 0, 1000);
  clusters_npe = new TH2F ("clusters_npe", "bx_laben_findcluster_time clusters npe", 
      3, 1, 4, 1000, 0, 1000);
  clusters_number = new TH1F ("clusters_number", "bx_laben_findcluster_time clusters number",
      4, 0, 4);

  bx_root_barn::get ()->store (bx_root_barn::test, clusters_raw_times, this);
  bx_root_barn::get ()->store (bx_root_barn::test, clusters_mean_times, this);
  bx_root_barn::get ()->store (bx_root_barn::test, clusters_times, this);
  bx_root_barn::get ()->store (bx_root_barn::test, clusters_hits, this);
  bx_root_barn::get ()->store (bx_root_barn::test, clusters_pmts, this);
  bx_root_barn::get ()->store (bx_root_barn::test, clusters_npe, this);
  bx_root_barn::get ()->store (bx_root_barn::test, clusters_number, this);
  
  histo_raw= new TH2F ("histo_raw", "bx_laben_findcluster_time histo raw", 3000, 0, 3000, 7000, 0, 7000);
  bx_root_barn::get ()->store (bx_root_barn::test, histo_raw, this);
  histo_cluster= new TH2F ("histo_cluster", "bx_laben_findcluster_time histo cluster", 3000, 0, 3000, 7000, 0, 7000);
  bx_root_barn::get ()->store (bx_root_barn::test, histo_cluster, this);
  
#endif
  get_message(bx_message::debug) << "begin" << dispatch;
      
  fired_channel = new int[constants::laben::channels];
}

bx_reco_event* bx_laben_findcluster_time::doit (bx_reco_event *ev) {

  if ( ev->get_trigger().is_neutrino() ) {
    
    bx_laben_event& er = ev->get_laben_nonconst();
    bx_laben_clustered_event &ew = dynamic_cast<bx_laben_clustered_event&>(ev->get_laben_nonconst());

    #ifdef ENABLE_TEST_ROOT_OUTPUT
    clusters_number->Fill(er.get_nclusters());
    
    for ( int i=0; i<er.get_decoded_nhits(); i++ ) 
      histo_raw->Fill(ev->get_trigger().get_evid(), er.get_decoded_hit(i).get_raw_time()-er.get_trigger_rawt()+6600.);
    #endif
  
    float time_thresh = 7.e3/(10*er.get_decoded_nhits());
    float second_time_thresh = 2.;
    int hits_thresh = 5;
    int second_hits_thresh = 3;
    int sum,fired;
    double start,second_start;
 
    for ( int cluster=0; cluster<er.get_nclusters(); cluster++ ) {
    
      sum = 0;
      fired = -2;
      start = -1.e10;
    
      // First search for a rough value
      for ( int hit=0; hit<er.get_cluster(cluster).get_clustered_nhits()-1; hit++ ) {
        double dt0 = er.get_cluster(cluster).get_clustered_hit(hit).get_time()-
	            er.get_cluster(cluster).get_clustered_hit(0).get_time();
        double dt  = er.get_cluster(cluster).get_clustered_hit(hit+1).get_time()-
	            er.get_cluster(cluster).get_clustered_hit(hit).get_time();

        if ( dt < time_thresh && dt0 < 64. ) {
	  if ( fired == hit-1 ) {
	    sum++;
	    if ( sum+1 >= hits_thresh ) { 
	      start = er.get_cluster(cluster).get_clustered_hit(hit-hits_thresh+1).get_time();
	      break;
	    }
	  } else 
	    sum=0;
	  fired = hit;
        }
        else 
	  sum=0;
      }
      if ( start == -1.e10 ) {
	get_message(bx_message::warn) << "Rough start time of cluster " << cluster+1
	<< " not found for event " << ev->get_trigger().get_evid() << " ( " 
	<< er.get_cluster(cluster).get_clustered_hit(0).get_time()-er.get_trigger_rawt() << " ) " << dispatch;
	start = er.get_cluster(cluster).get_clustered_hit(0).get_time();
      }
    
      sum = 0;
      fired = -2;
      second_start = -1.e10; 
    
      // Second search with more severe cuts (if found it gives a more precise value)
      for ( int hit=0; hit<er.get_cluster(cluster).get_clustered_nhits()-1; hit++ ) {
        double t   = er.get_cluster(cluster).get_clustered_hit(hit).get_time();
        double dt0 = er.get_cluster(cluster).get_clustered_hit(hit).get_time()-
	            er.get_cluster(cluster).get_clustered_hit(0).get_time();
        double dt  = er.get_cluster(cluster).get_clustered_hit(hit+1).get_time()-
	            er.get_cluster(cluster).get_clustered_hit(hit).get_time();
        if ( dt < second_time_thresh && dt0 < 64. && t >= start ) {
	  if ( fired == hit-1 ) {
	    sum++;
	    if ( sum+1 >= second_hits_thresh ) {
	      second_start = er.get_cluster(cluster).get_clustered_hit(hit-second_hits_thresh+1).get_time();
	      break;
	    }
	  } else 
	    sum=0;
	  fired = hit;
        }
        else 
	  sum=0;
      }

      // Setting of the cluster start time 
      ew.clusters[cluster].f4_start_time = (float)start;
      if ( second_start != -1.e10 && second_start != start ) ew.clusters[cluster].f4_start_time = (float)second_start;
      
      // Some photons might be discarded from the hit list.
      std::vector<bx_laben_clustered_hit>::iterator it = ew.clusters[cluster].clustered_hits.begin();
      std::vector<bx_laben_clustered_hit>::iterator start_it;
      
      for (; it != ew.clusters[cluster].clustered_hits.end(); it++) {
	double dt = (*it).get_time()-er.get_cluster(cluster).get_start_time();
        if ( dt < 0. && er.get_cluster(cluster).get_clustered_nhits() > 0 ) {
	  start_it = ew.clusters[cluster].clustered_hits.begin();
	  ew.clusters[cluster].clustered_hits.erase(start_it);
	}
      }
      
      // Setting of the photons' times (relative to the time of the first photon) and fill the histos
      for (it=ew.clusters[cluster].clustered_hits.begin(); it != ew.clusters[cluster].clustered_hits.end(); it++) {
	double rt = (*it).get_time() - er.get_trigger_rawt() + 6600.;
        double dt = (*it).get_time() - er.get_cluster(cluster).get_start_time();
	#ifdef ENABLE_TEST_ROOT_OUTPUT
	clusters_raw_times->Fill( cluster+1, rt );
	clusters_times->Fill( cluster+1, dt );
	if ( cluster == 0 ) histo_cluster->Fill(ev->get_trigger().get_evid(),rt);
        #endif
	(*it).f8_time = dt;
      }

      // Warning message
      if ( er.get_cluster(cluster).get_clustered_nhits() == 0 ) {
	get_message(bx_message::warn) << "Cluster " << cluster+1 << " of event " <<
	ev->get_trigger().get_evid() << " is empty!" << dispatch;
	return ev;
      }

      // Setting of the other variables of the cluster
      // FIXME: npe is still MISSING
      std::fill_n (fired_channel, constants::laben::channels, 0);
      ew.clusters[cluster].f4_mean_time = 0.;

      for ( it=ew.clusters[cluster].clustered_hits.begin(); it != ew.clusters[cluster].clustered_hits.end(); it++ ) {
	ew.clusters[cluster].f4_mean_time += (*it).f8_time;
        const bx_laben_decoded_hit& hit = (*it).get_decoded_hit();
        const db_channel* db = hit.get_db_channel();
        int lg = db->get_lg();
        if ( !fired_channel[lg] ) {
	  fired_channel[lg]++;
	  ew.clusters[cluster].i4_npmt++;
          bool cone = db->pmt_has_cone();
	  if ( cone ) ew.clusters[cluster].i4_npmt_conc++;
	}
      }
      
      ew.clusters[cluster].f4_mean_time /= er.get_cluster(cluster).get_clustered_nhits();

      #ifdef ENABLE_TEST_ROOT_OUTPUT
      clusters_mean_times->Fill( cluster+1, er.get_cluster(cluster).get_mean_time() );
      clusters_hits->Fill( cluster+1, er.get_cluster(cluster).get_clustered_nhits() ); 
      clusters_pmts->Fill( cluster+1, er.get_cluster(cluster).get_npmt() );
      clusters_npe->Fill( cluster+1, er.get_cluster(cluster).get_npe() );
      #endif

      // Some debugging informatios
      int last_hit_cluster = er.get_cluster(cluster).get_clustered_nhits()-1;
      double time_duration = er.get_cluster(cluster).get_clustered_hit(last_hit_cluster).get_time();
      get_message(bx_message::debug) << "Cluster " << cluster+1 << " :\n" 
      << "            hits:       " << er.get_cluster(cluster).get_clustered_nhits() << "\n" 
      << "            start time: " << er.get_cluster(cluster).get_start_time() << "\n"	
      << "            duration:   " << time_duration << "\n"
      << "            mean time:  " << er.get_cluster(cluster).get_mean_time() << "\n"
      << "            pmts:       " << er.get_cluster(cluster).get_npmt() << "\n"
      << "            pmts conc:  " << er.get_cluster(cluster).get_npmt_conc() << "\n" << dispatch;
      
    }
    er.mark_stage(bx_base_event::clustered);
  }
 
  return ev;  
}

void bx_laben_findcluster_time::end () {
  delete [] fired_channel;
}

/*
 * $Log: bx_laben_findcluster_time.cc,v $
 * Revision 1.2  2004/07/12 12:37:20  dmanuzio
 * Removed a bug
 *
 * Revision 1.1  2004/07/09 13:59:08  dmanuzio
 * Added 2 modules for Laben clustering
 *
 *
 */

