/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 *
 * $Id: bx_muon_decoder.cc,v 1.10 2004/06/25 14:41:30 ddangelo Exp $
 *
 * Implementation of bx_muon_decoder.hh
 * 
*/

#include "bx_reco_event.hh"
#include "bx_muon_decoder.hh"
#include "bx_dbi.hh"
#include "db_run.hh"
#include "db_profile.hh"
#include "bx_trigger_event.hh"
#include "bx_reco_event.hh"

// ctor
bx_muon_decoder::bx_muon_decoder() : bx_base_module("bx_muon_decoder", bx_base_module::main_loop) {
  require_event_stage (bx_base_module::muon, bx_base_event::raw);
}


// public interface methods 
     
void bx_muon_decoder::begin () {
}

bx_reco_event* bx_muon_decoder::doit (bx_reco_event *ev) {
  if(ev->get_trigger().is_neutrino())
    m_process_physics_event(ev);
  else if(ev->get_trigger().is_laser394())
    m_process_laser_event(ev);
  else
    get_message(bx_message::debug) << "don't know what to do with this trgtype" << ev->get_trigger().get_trgtype() << dispatch;

  return ev;  
}

void bx_muon_decoder::end () {
}


// private helper methods

void bx_muon_decoder::m_process_physics_event (bx_reco_event *ev) {
  bx_muon_event& er = ev->get_muon_nonconst();
  // cast to a muon event  
  bx_muon_decoded_event &ew = dynamic_cast<bx_muon_decoded_event&>(ev->get_muon_nonconst());

  const db_run& run_info = bx_dbi::get ()->get_run ();
  const db_profile& profile_info = bx_dbi::get ()->get_profile ();

  double time, charge;
  for (unsigned long i=0 ; i<er.get_raw_nhits(); i++) {    
    const bx_muon_raw_hit& h = er.get_raw_hit(i);

    if (profile_info.logical_channel_description(h.get_logical_channel()) == db_profile::ordinary) {
      // time in ns from gate start (8192 ticks before trigger==tdc_common_stop) 
      time = (constants::muon::tdc::time_buffer - h.get_lead_time() /*- run_info.get_time_offset(h.get_muon_channel()) */) * constants::muon::tdc::ns_per_clock; 
      if (time < 0. || time > constants::muon::tdc::time_buffer*constants::muon::tdc::ns_per_clock) continue; // skip shit
      charge = (h.get_time_diff() - run_info.get_muon_precalib_pedestal(h.get_muon_channel())) * constants::muon::pe_per_clock;
      if (charge < 0) {
	charge = 0.;
	get_message(bx_message::debug) << "ev " << ev->get_event_number() << " mch " << h.get_muon_channel() << "; negative charge." << dispatch;
      }
      ew.decoded_hits.push_back(bx_muon_decoded_hit(time, charge, &h));
      ew.nhits_per_channel[h.get_muon_channel()]++;
    }  
  }
  er.mark_stage (bx_base_event::decoded);
}

// trivial predicate for the find_if algorithm (see below)
bool is_reference_channel (const bx_muon_raw_hit& h) {
  const db_profile& profile_info = bx_dbi::get ()->get_profile ();
  return profile_info.logical_channel_description(h.get_logical_channel()) == db_profile::laser;
}

void bx_muon_decoder::m_process_laser_event (bx_reco_event *ev) {
  bx_muon_event& er = ev->get_muon_nonconst();
  // cast to a muon event  
  bx_muon_decoded_event &ew = dynamic_cast<bx_muon_decoded_event&>(ev->get_muon_nonconst());

  const db_run& run_info = bx_dbi::get ()->get_run ();
  const db_profile& profile_info = bx_dbi::get ()->get_profile ();


  // find reference time
  std::vector<bx_muon_raw_hit>::const_iterator ref = find_if(er.get_raw_hits().begin(), er.get_raw_hits().end(), is_reference_channel);
  if(ref == er.get_raw_hits().end()) {
    get_message(bx_message::error) << "Laser event without reference pulse" << dispatch;
    return;
  }

  double time, charge;
  for (unsigned long i=0 ; i<er.get_raw_nhits(); i++) {    
    const bx_muon_raw_hit& h = er.get_raw_hit(i);

    if (profile_info.logical_channel_description(h.get_logical_channel()) == db_profile::ordinary) {
      // time in ns from led reference pulse 
      time = (ref->get_lead_time() - h.get_lead_time()) * constants::muon::tdc::ns_per_clock; 
      if (time < 0. || time > constants::muon::tdc::time_buffer*constants::muon::tdc::ns_per_clock) continue; // skip shit
      charge = (h.get_time_diff() - run_info.get_muon_precalib_pedestal(h.get_muon_channel())) * constants::muon::pe_per_clock;
      if (charge < 0) {
	charge = 0.;
	get_message(bx_message::debug) << "ev " << ev->get_event_number() << " mch " << h.get_muon_channel() << "; negative charge." << dispatch;
      }
      ew.decoded_hits.push_back(bx_muon_decoded_hit(time, charge, &h));
      ew.nhits_per_channel[h.get_muon_channel()]++;
    }     
  }  
  er.mark_stage (bx_base_event::decoded);
}

/*
 * $Log: bx_muon_decoder.cc,v $
 * Revision 1.10  2004/06/25 14:41:30  ddangelo
 * added require and mark event stage
 *
 * Revision 1.9  2004/06/07 12:48:57  ddangelo
 * special channels removed from decoded hits vector
 *
 * Revision 1.8  2004/06/02 17:15:00  ddangelo
 * fixed some stuff
 *
 * Revision 1.7  2004/06/01 11:36:27  ddangelo
 * updated to new constants syntax
 *
 * Revision 1.6  2004/05/20 11:02:30  ddangelo
 * trigger type selection introduced.
 * Job splitted in 2 private methods (physics/led)
 * 2nd one finds led ref pulse with a pre-loop.
 * chann descr asked to db_profile (tmp)
 *
 * Revision 1.5  2004/05/18 18:21:03  ddangelo
 * added an include
 *
 * Revision 1.4  2004/04/28 11:04:49  ddangelo
 * Modules adapted to call the right const/nonconst event portion getter
 *
 * Revision 1.3  2004/04/27 09:46:50  ddangelo
 * modifications to match new event structure.
 * Other debugging
 *
 * Revision 1.2  2004/04/06 12:42:17  razeto
 * Added module role to base_module, and the code to handle everywhere. Some other minor fixes
 *
 * Revision 1.1  2004/04/02 09:56:03  ddangelo
 * created bx_muon_decoder module.
 * precalibration and calibration/db info are still commented out
 *
 *
 */

