/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: bx_precalib_laben_adc.cc,v 1.11 2004/07/12 10:47:10 razeto Exp $
 *
 * Implementation of bx_precalibrator
 *
 */
#include "bx_precalib_laben_adc.hh"
#include "messenger.hh"
#include "laben_time_hit.hh"
#include "constants.hh"
#include "bx_dbi.hh"
#include "db_run.hh"
#include "bx_root_barn.hh"

#include <algorithm>

// ctor
bx_precalib_laben_adc::bx_precalib_laben_adc (): bx_base_module("bx_precalib_laben_adc", bx_base_module::precalib_cycle1) {
  require_event_stage (bx_base_module::laben, bx_base_event::raw);
}

// module interface
void bx_precalib_laben_adc::begin () {
    // allocate the matrix
  adc_sample_map = new int*[constants::laben::channels]; 
  for (int i = 0; i < constants::laben::channels; i++) {
    adc_sample_map[i] = new int[256];
    std::fill_n (adc_sample_map[i], 256, 0);
  }

  u1_maxima_bin_add = get_parameter ("maxima_bin_add").get_int ();
  
#ifdef ENABLE_TEST_ROOT_OUTPUT
  adc_samples = new TH1F ("adc_samples", "bx_precalib_laben_adc TDC RAMP samples", 256, 0, 256);
  adc_limits = new TH1F ("adc_limits", "bx_precalib_laben_adc TDC RAMP limits", 256, 0, 256);
  bx_root_barn::get ()->store (bx_root_barn::test, adc_samples, this);
  bx_root_barn::get ()->store (bx_root_barn::test, adc_limits, this);
#endif

  get_message(bx_message::debug) << "begin" << dispatch;
}

bx_reco_event* bx_precalib_laben_adc::doit (bx_reco_event *ev) {
  const bx_laben_event& er = ev->get_laben();

    // Update has data field
  if (er.get_raw_nhits ()) b_has_data = true;

    // Use only first hit
  int fired_channels[constants::laben::channels];
  std::fill_n (fired_channels, constants::laben::channels, 0);
      
  for (int i = 0; i < er.get_raw_nhits (); i++) {
    const bx_laben_raw_hit &hit = er.get_raw_hit (i);
    if (fired_channels[hit.get_logical_channel () - 1]++) continue;
    adc_sample_map[hit.get_logical_channel () - 1][hit.get_time_1()] ++;
    adc_sample_map[hit.get_logical_channel () - 1][hit.get_time_2()] ++;
#ifdef ENABLE_TEST_ROOT_OUTPUT
    adc_samples->Fill (hit.get_time_1());
    adc_samples->Fill (hit.get_time_2());
#endif
  }
  return ev;
}

void bx_precalib_laben_adc::end () {
  if (b_has_data) {

    db_run& run_info = bx_dbi::get ()->get_run ();

    for (int i = 0; i < constants::laben::channels; i++) {
        // calculate local maxima in the first 20 and in the last 20 bins
      int low_bin = std::max_element (adc_sample_map[i], adc_sample_map[i] + 20) - adc_sample_map[i];
      int high_bin = std::max_element (adc_sample_map[i] + 236, adc_sample_map[i] + 256) - adc_sample_map[i];
    
        // add/remove a costant offset
      if (low_bin >= 3) low_bin -= u1_maxima_bin_add;
      if (high_bin <= 252) high_bin += u1_maxima_bin_add;
    
        // if adc_sample_map[i][high_bin] == 0 no hit present
      if (! adc_sample_map[i][high_bin]) high_bin = 255; // low_bin is already at 0 considering max_element engine

        // set the values
      run_info.set_laben_precalib_low_bin (i + 1, low_bin, this);
      run_info.set_laben_precalib_high_bin (i + 1, high_bin, this);

#ifdef ENABLE_TEST_ROOT_OUTPUT
      adc_limits->Fill (low_bin);
      adc_limits->Fill (high_bin);
#endif
    
      get_message(bx_message::debug) << "adc limits for channel " << i << " are " <<  low_bin << ":" << high_bin << dispatch;
    } 
  } 
  
    // deallocate the matrix
  for (int i = 0; i < constants::laben::channels; i++) delete [] adc_sample_map[i];
  delete [] adc_sample_map;

  get_message(bx_message::debug) << "end" << dispatch;
}
/*
 * $Log: bx_precalib_laben_adc.cc,v $
 * Revision 1.11  2004/07/12 10:47:10  razeto
 * Updated to use only the first hit on every channel
 *
 * Revision 1.10  2004/06/01 11:50:36  razeto
 * Updated to follow new constants design
 *
 * Revision 1.9  2004/05/31 15:00:55  razeto
 * Updated to use the base module require statements.
 * Upgraded to avoid doing stuff at the end if there is no data (has_data is false)
 *
 * Revision 1.8  2004/05/21 11:28:23  razeto
 * Updated conforming to the new conventions of event::get_logical_channel
 *
 * Revision 1.7  2004/05/21 08:39:13  razeto
 * Updated to use bx_root_barn
 *
 * Revision 1.6  2004/05/18 15:01:54  razeto
 * A lot of development done
 *
 * Revision 1.5  2004/04/28 11:04:49  ddangelo
 * Modules adapted to call the right const/nonconst event portion getter
 *
 * Revision 1.4  2004/04/27 14:25:18  ddangelo
 * Modifications to match new event structure.
 * Useless dynamic_cast in event reading replaced by getter.
 * safer const reference is used instead of non-const pointer.
 *
 * Revision 1.3  2004/04/26 13:53:00  razeto
 * Updated to follow new name/syntax of db_run and db_profile
 *
 * Revision 1.2  2004/04/13 12:09:17  razeto
 * Fixed a bug: the values were not set
 *
 * Revision 1.1  2004/04/12 15:59:49  razeto
 * Added a firsto core of laben precalibration
 *
 */
