/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: bx_precalib_laben_d80.cc,v 1.9 2004/07/12 10:47:10 razeto Exp $
 *
 * Implementation of bx_precalibrator
 * D80 is istogrammed in a bin large 600 bin with the following method:
 * each bin has 0.1 resolution and is centered on the 80 value; so the
 * range is 50-109.
 *
 */
#include "bx_precalib_laben_d80.hh"
#include "messenger.hh"
#include "laben_time_hit.hh"
#include "constants.hh"
#include "bx_dbi.hh"
#include "db_run.hh"
#include "laben_time_hit.hh"
#include "bx_root_barn.hh"

#include <algorithm>

// ctor
bx_precalib_laben_d80::bx_precalib_laben_d80 (): bx_base_module("bx_precalib_laben_d80", bx_base_module::precalib_cycle3) {
  require_event_stage (bx_base_module::laben, bx_base_event::raw);
}

// module interface
void bx_precalib_laben_d80::begin () {
    // allocate the matrix
  d80_delay_map = new int*[constants::laben::channels]; 
  for (int i = 0; i < constants::laben::channels; i++) {
    d80_delay_map[i] = new int[600];
    std::fill_n (d80_delay_map[i], 600, 0);
  }

  f_d80_low_limit = get_parameter ("d80_low_limit").get_float ();
  f_d80_high_limit = get_parameter ("d80_high_limit").get_float ();
  
#ifdef ENABLE_TEST_ROOT_OUTPUT
  d80 = new TH1F ("d80", "bx_precalib_laben_d80 TDC D80 values", 600, 50, 110);
  d80_channel_distrubution = new TH2F ("d80_channel_distrubution", "bx_precalib_laben_d80 TDC D80 channel distribution", 2400, 1, 2401, 600, 50, 110);
  bx_root_barn::get ()->store (bx_root_barn::test, d80, this);
  bx_root_barn::get ()->store (bx_root_barn::test, d80_channel_distrubution, this);
#endif
      
  get_message(bx_message::debug) << "begin" << dispatch;
}

bx_reco_event* bx_precalib_laben_d80::doit (bx_reco_event *ev) {
  const bx_laben_event& er = ev->get_laben ();

    // Update has data field
  if (er.get_raw_nhits ()) b_has_data = true;

    // Use only first hit
  int fired_channels[constants::laben::channels];
  std::fill_n (fired_channels, constants::laben::channels, 0);
  
  for (int i = 0; i < er.get_raw_nhits (); i++) {
    const bx_laben_raw_hit &hit = er.get_raw_hit (i);
    if (fired_channels[hit.get_logical_channel () - 1]++) continue;
    laben_time_hit t_hit(hit);
    float d80 = t_hit.get_d80 ();
    if (!t_hit.is_good (0.25)) continue;
    if (d80 < 50.) d80 = 50.;
    else if (d80 > 109.) d80 = 109.;
    d80_delay_map[hit.get_logical_channel () - 1][int ((d80 - 50.) * 10.)] ++;
#ifdef ENABLE_TEST_ROOT_OUTPUT
    d80_channel_distrubution->Fill (hit.get_logical_channel (), d80);
#endif
  }
  return ev;
}

void bx_precalib_laben_d80::end () {
  if (b_has_data) {

    db_run& run_info = bx_dbi::get ()->get_run ();
  
    for (int i = 0; i < constants::laben::channels; i++) {
        // calculate the histogram mean
      double sum = 0.;
      long int counts = 0;
      for (int j = 0; j < 600; j++) {
         counts += d80_delay_map[i][j];
         sum += (float(j) / 10 + 50) * d80_delay_map[i][j];
      }
      double mean = sum / counts;

      if (counts < 20 || mean < f_d80_low_limit && mean > f_d80_high_limit) {
        if (counts > 20) get_message (bx_message::info) << "d80 not found for channel " << i << " between limits (" << f_d80_low_limit << ":" << f_d80_high_limit << ")" << dispatch;
        mean = 55.;
      }
    
        // set the value
      run_info.set_laben_precalib_delta80 (i + 1, mean, this);

#ifdef ENABLE_TEST_ROOT_OUTPUT
      d80->Fill (mean);
#endif
    
      get_message (bx_message::debug) << "d80 for channel " << i << " is " << mean << dispatch;
    }
  }

    // deallocate the matrix
  for (int i = 0; i < constants::laben::channels; i++) delete [] d80_delay_map[i];
  delete [] d80_delay_map;

  get_message(bx_message::debug) << "end" << dispatch;
}
/*
 * $Log: bx_precalib_laben_d80.cc,v $
 * Revision 1.9  2004/07/12 10:47:10  razeto
 * Updated to use only the first hit on every channel
 *
 * Revision 1.8  2004/06/01 11:50:36  razeto
 * Updated to follow new constants design
 *
 * Revision 1.7  2004/05/31 15:00:55  razeto
 * Updated to use the base module require statements.
 * Upgraded to avoid doing stuff at the end if there is no data (has_data is false)
 *
 * Revision 1.6  2004/05/21 11:28:23  razeto
 * Updated conforming to the new conventions of event::get_logical_channel
 *
 * Revision 1.5  2004/05/21 08:39:13  razeto
 * Updated to use bx_root_barn
 *
 * Revision 1.4  2004/05/18 15:01:54  razeto
 * A lot of development done
 *
 * Revision 1.3  2004/04/28 11:04:49  ddangelo
 * Modules adapted to call the right const/nonconst event portion getter
 *
 * Revision 1.2  2004/04/27 14:25:18  ddangelo
 * Modifications to match new event structure.
 * Useless dynamic_cast in event reading replaced by getter.
 * safer const reference is used instead of non-const pointer.
 *
 * Revision 1.1  2004/04/26 13:50:51  razeto
 * Added
 *
 */
