/* BOREXINO Reconstruction program
 *
 * Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
 *
 * $Id: bx_precalib_laben_gray.cc,v 1.11 2004/08/10 11:05:53 razeto Exp $
 *
 * Implementation of bx_precalibrator
 *
 */
#include "bx_precalib_laben_gray.hh"
#include "messenger.hh"
#include "laben_time_hit.hh"
#include "constants.hh"
#include "bx_dbi.hh"
#include "db_run.hh"
#include "db_profile.hh"
#include "bx_root_barn.hh"

#include <algorithm>
#include <numeric>


// ctor
bx_precalib_laben_gray::bx_precalib_laben_gray (): bx_base_module("bx_precalib_laben_gray", bx_base_module::precalib_cycle1) {
  require_event_stage (bx_base_module::laben, bx_base_event::raw);
}

// module interface
void bx_precalib_laben_gray::begin () {

  i_channel_skip = get_parameter ("channel_reference_skip").get_int ();
  f_mean_bound = get_parameter ("mean_bound").get_float ();
  f_rms_bound = get_parameter ("rms_bound").get_float ();
  i_histo_lenght = 256;

    // allocate the matrix
  gray_map = new int[constants::laben::channels / i_channel_skip];
  gray_shift_map = new int**[constants::laben::channels / i_channel_skip];
  for (int j = 0; j < constants::laben::channels / i_channel_skip; j++) {
    gray_shift_map[j] = new int *[constants::laben::channels];
    for (int i = 0; i < constants::laben::channels; i++) {
      gray_shift_map[j][i] = new int[i_histo_lenght](0);
      std::fill_n (gray_shift_map[j][i], i_histo_lenght, 0);
    }
  }

#ifdef ENABLE_TEST_ROOT_OUTPUT
  gray_diffs_global = new TH2F ("gray_diffs_global", "bx_precalib_laben_gray TDC gray counter diffs respect to all references", 2400, 1, 2401, 256, -128, 128);
  gray_diffs_reference = new TH2F ("gray_diffs_reference", "bx_precalib_laben_gray TDC gray counter diffs for used reference", 2400, 1, 2401, 256, -128, 128);
  gray_shifts = new TH1S ("gray_shifts", "bx_precalib_laben_gray TDC gray counter shift", 2400, 1, 2401);
  bx_root_barn::get ()->store (bx_root_barn::test, gray_diffs_global, this);
  bx_root_barn::get ()->store (bx_root_barn::test, gray_diffs_reference, this);
  bx_root_barn::get ()->store (bx_root_barn::test, gray_shifts, this);
#endif
      
  get_message(bx_message::debug) << "begin" << dispatch;
}

bx_reco_event* bx_precalib_laben_gray::doit (bx_reco_event *ev) {
  const bx_laben_event& er = ev->get_laben();

    // Update has data field
  if (er.get_raw_nhits ()) b_has_data = true;

    // First set the refernce channels
  for (int i = 0; i < er.get_raw_nhits (); i++) {
    const bx_laben_raw_hit &hit = er.get_raw_hit (i);
    for (int j = 0; j < constants::laben::channels / i_channel_skip; j++) 
      if (hit.get_logical_channel () == i_channel_skip * (j + 1) && hit.get_time_1 () != 0xff && hit.get_time_2 () != 0xff)
        gray_map[j] = hit.get_gray_counter ();
  }
  
    // Then fill the shift histogram using only first hit
  int fired_channels[constants::laben::channels];
  std::fill_n (fired_channels, constants::laben::channels, 0);
  for (int i = 0; i < er.get_raw_nhits (); i++) {
    const bx_laben_raw_hit &hit = er.get_raw_hit (i);
    if (hit.get_time_1 () == 0xff && hit.get_time_2 () == 0xff) continue;
    int lg = hit.get_logical_channel ();
    if (fired_channels[lg - 1]++) continue;
    int gray = hit.get_gray_counter ();
    for (int j = 0; j < constants::laben::channels / i_channel_skip; j++) {
      int bin = gray - gray_map[j] + i_histo_lenght / 2;
      if (bin < 0) bin = 0;
      else if (bin >= i_histo_lenght) bin = i_histo_lenght - 1;
      gray_shift_map[j][lg - 1][bin]++;
#ifdef ENABLE_TEST_ROOT_OUTPUT
      gray_diffs_global->Fill (lg, gray - gray_map[j]);
#endif
    }
  }
  return ev;
}

bool bx_precalib_laben_gray::m_check_bound (int **v, float& mean, float& rms) {
  unsigned long sum = 0, hits = 0;
  
  for (int j = 0; j < constants::laben::channels; j++) {
    for (int i = 0; i < i_histo_lenght; i++) {
      sum += i * v[j][i];
      hits += v[j][i];
    }
  }
  mean = (double)sum / hits;

  double l2 = 0;
  for (int j = 0; j < constants::laben::channels; j++) {
    for (int i = 0; i < i_histo_lenght; i++) {
      l2 += (mean - i) * (mean - i) * v[j][i];
    }
  }
  rms = ::sqrt (l2) / hits;
  
  mean = ::fabs (mean - i_histo_lenght / 2);
  return mean < f_mean_bound && rms < f_rms_bound;
}
  
void bx_precalib_laben_gray::end () {
  if(b_has_data) {

    db_run& run_info = bx_dbi::get ()->get_run ();
    const db_profile& profile_info = bx_dbi::get()->get_profile ();

    bool found = false;

    for (int j = 0; j < constants::laben::channels / i_channel_skip; j++) {
      float mean, rms;
    
      if (m_check_bound (gray_shift_map[j], mean, rms)) {
        get_message (bx_message::info) << "found channel reference " << i_channel_skip * (j + 1) << " with shift " << mean << "+-" << rms << dispatch;
        found = true;
      
        for (int i = 0; i < constants::laben::channels; i++) {
	  int max_bin = std::max_element (gray_shift_map[j][i], gray_shift_map[j][i] + i_histo_lenght) - gray_shift_map[j][i];
	  int gray_shift = max_bin - i_histo_lenght / 2;
	  if (!std::accumulate (gray_shift_map[j][i], gray_shift_map[j][i] + i_histo_lenght, int(0))) gray_shift = 0;
	  if ((gray_shift < -20 || gray_shift > 20) && profile_info.logical_channel_description (i + 1) == db_profile::ordinary)
	    get_message (bx_message::info) << "ch " << i << " shift " << gray_shift << dispatch;
	
	  if (profile_info.logical_channel_description (i + 1) == db_profile::ordinary)
	    run_info.set_laben_precalib_gray_shift (i + 1, gray_shift, this);
	  else
	    run_info.set_laben_precalib_gray_shift (i + 1, 0, this);
  

	  get_message (bx_message::debug) << "gray shift for channel " << i << " is " << gray_shift << dispatch;
#ifdef ENABLE_TEST_ROOT_OUTPUT
	  gray_shifts->SetBinContent (i + 1, gray_shift); // Bins start at 1
          for (int k = 0; k < i_histo_lenght; k++) 
	    gray_diffs_reference->SetBinContent (i + 1, k + 1, gray_shift_map[j][i][k]);
#endif
        }

        break;
      }
    }
  
    if (!found) get_message (bx_message::critic) << "no gray shift reference channel found" << dispatch;    

  }

    // delete the matrix
  delete [] gray_map;
  for (int j = 0; j < constants::laben::channels / i_channel_skip; j++) {
    for (int i = 0; i < constants::laben::channels; i++)
      delete [] gray_shift_map[j][i];
    delete [] gray_shift_map[j];
  }

  get_message(bx_message::debug) << "end" << dispatch;
}
/*
 * $Log: bx_precalib_laben_gray.cc,v $
 * Revision 1.11  2004/08/10 11:05:53  razeto
 * Fixed a bug in finding valid reference channel
 *
 * Revision 1.10  2004/07/12 10:47:10  razeto
 * Updated to use only the first hit on every channel
 *
 * Revision 1.9  2004/06/01 11:50:36  razeto
 * Updated to follow new constants design
 *
 * Revision 1.8  2004/05/31 15:00:55  razeto
 * Updated to use the base module require statements.
 * Upgraded to avoid doing stuff at the end if there is no data (has_data is false)
 *
 * Revision 1.7  2004/05/21 11:28:23  razeto
 * Updated conforming to the new conventions of event::get_logical_channel
 *
 * Revision 1.6  2004/05/21 08:39:13  razeto
 * Updated to use bx_root_barn
 *
 * Revision 1.5  2004/05/18 15:01:54  razeto
 * A lot of development done
 *
 * Revision 1.4  2004/04/28 11:04:49  ddangelo
 * Modules adapted to call the right const/nonconst event portion getter
 *
 * Revision 1.3  2004/04/27 14:25:18  ddangelo
 * Modifications to match new event structure.
 * Useless dynamic_cast in event reading replaced by getter.
 * safer const reference is used instead of non-const pointer.
 *
 * Revision 1.2  2004/04/26 13:53:00  razeto
 * Updated to follow new name/syntax of db_run and db_profile
 *
 * Revision 1.1  2004/04/12 15:59:49  razeto
 * Added a firsto core of laben precalibration
 *
 */
