/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 *
 * $Id: bx_precalib_muon_findpulse.cc,v 1.9 2004/06/01 11:36:27 ddangelo Exp $
 *
 * Implementation of bx_precalib_muon_findpulse.hh
 *
 */

#include <numeric>

#include "bx_precalib_muon_findpulse.hh"
#include "messenger.hh"
#include "constants.hh"
#include "bx_dbi.hh"
#include "db_run.hh"
#include "db_profile.hh"
#include "algorithms.hh"

bx_precalib_muon_findpulse::bx_precalib_muon_findpulse(): bx_base_module("bx_precalib_muon_findpulse", bx_base_module::precalib_cycle1) {
}

void bx_precalib_muon_findpulse::begin () {
  pulse_times = new std::vector<unsigned long>(constants::muon::tdc::time_buffer);
  b_has_data = false;
  get_message(bx_message::debug) << "begin" << dispatch;
}

bx_reco_event* bx_precalib_muon_findpulse::doit (bx_reco_event *ev) {

  const bx_muon_event& er = ev->get_muon();

  b_has_data |= bool(er.get_raw_nhits());

  const db_profile& profile_info = bx_dbi::get ()->get_profile ();

  // start looping over hits
  for (unsigned long i = 0; i <er.get_raw_nhits(); i++) { 
    const bx_muon_raw_hit& h = er.get_raw_hit(i);

    // skip special channels
    if (profile_info.logical_channel_description(h.get_logical_channel()) != db_profile::ordinary)
      continue;

    (*pulse_times)[h.get_lead_time()]++;
      
  } // end of loop over hits

  return ev; 
}

void bx_precalib_muon_findpulse::end () {
  if (!b_has_data) return; 

  db_run& run_info = bx_dbi::get ()->get_run ();

  unsigned long entries = accumulate(pulse_times->begin(), pulse_times->end(), 0);
  unsigned long max = max_element(pulse_times->begin(), pulse_times->end()) - pulse_times->begin();
  double mean = compute_mean(pulse_times->begin(), pulse_times->end(), 0.);
  double rms = compute_rms(pulse_times->begin(), pulse_times->end(), 0., mean);

  run_info.set_muon_precalib_pulse_time( (unsigned long)(mean), this);	
  
  get_message(bx_message::info) << "entries " << entries << " max " << max << " mean " << run_info.get_muon_precalib_pulse_time()  << " rms " << rms << dispatch;

  delete pulse_times;

  get_message(bx_message::debug) << "end" << dispatch;
}

/*
 * $Log: bx_precalib_muon_findpulse.cc,v $
 * Revision 1.9  2004/06/01 11:36:27  ddangelo
 * updated to new constants syntax
 *
 * Revision 1.8  2004/05/25 12:59:33  ddangelo
 * std::vector<>::at() replaced by std::vector<>::operator[]()
 * to compile with gcc 2.95 as well
 *
 * Revision 1.7  2004/05/20 15:41:53  ddangelo
 * check on has_data introduced
 * check on chann descr implemented with db data (db_profile for the moment)
 * instead of static event members.
 * computetion done with algotithms instead of c like math.
 * Data stored in a std::vector* (histo use).
 *
 * Revision 1.5  2004/04/28 11:04:49  ddangelo
 * Modules adapted to call the right const/nonconst event portion getter
 *
 * Revision 1.4  2004/04/27 15:05:53  ddangelo
 * fixed the bug that was causing the crash with no muon data.
 *
 * Revision 1.3  2004/04/27 09:46:50  ddangelo
 * modifications to match new event structure.
 * Other debugging
 *
 * Revision 1.1  2004/04/20 11:36:25  ddangelo
 * added 2 modules for outer muon precalibration
 *
 *
 */
