/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 *
 * $Id: bx_precalib_muon_pedestals.cc,v 1.8 2004/06/01 11:36:27 ddangelo Exp $
 *
 * Implementation of bx_precalib_muon_pedestals.hh
 *
 */

#include <stdlib.h>
#include <math.h>
#include <numeric>

#include "bx_precalib_muon_pedestals.hh"
#include "messenger.hh"
#include "constants.hh"
#include "bx_dbi.hh"
#include "db_run.hh"
#include "db_profile.hh"
#include "algorithms.hh"

// ctor
bx_precalib_muon_pedestals::bx_precalib_muon_pedestals (): bx_base_module("bx_precalib_muon_pedestals", bx_base_module::precalib_cycle2) {
}

void bx_precalib_muon_pedestals::begin () {
  // retrieve parameters
  min_pedestal_width = (unsigned long)(get_parameter("min_pedestal_width").get_int());
  max_pedestal_width = (unsigned long)(get_parameter("max_pedestal_width").get_int());
  precalib_pulse_tolerance = (unsigned long)(get_parameter("precalib_pulse_tolerance").get_int());

  pedestals.resize(constants::muon::channels);
  for (int i = 0; i < constants::muon::channels; i++) pedestals[i] = new std::vector<unsigned long>(max_pedestal_width-min_pedestal_width);
  b_has_data = false;

  get_message(bx_message::debug) << "begin" << dispatch;
}

bx_reco_event* bx_precalib_muon_pedestals::doit (bx_reco_event *ev) {
  const bx_muon_event& er = ev->get_muon();
  if (!er.get_raw_nhits()) return ev;

  unsigned long pulse_time = bx_dbi::get ()->get_run ().get_muon_precalib_pulse_time();
  if (!pulse_time) {
    get_message(bx_message::error) << "no precalib pulse time. Was bx_precalib_muon_findpulse enabled?" << dispatch;
    return ev;
  }

  b_has_data = true;

  const db_profile& profile_info = bx_dbi::get ()->get_profile();

  for (unsigned long i = 0; i <er.get_raw_nhits(); i++) { 
    const bx_muon_raw_hit& h = er.get_raw_hit(i);

    // skip special channels
    if (profile_info.logical_channel_description(h.get_logical_channel()) != db_profile::ordinary)
      continue;

    // skip hits unrelated to calibration pulse (time filter)
    if ( ::abs( h.get_lead_time() - pulse_time ) > precalib_pulse_tolerance )
      continue;

    // check if width is ok
    if ( h.get_time_diff() > min_pedestal_width && h.get_time_diff() <= max_pedestal_width )
      (*(pedestals[h.get_muon_channel()]))[h.get_time_diff()-min_pedestal_width]++;
    else
      get_message(bx_message::warn) << "Unusual pedestal width (skipping); ev " << ev->get_event_number() << " mch " << h.get_muon_channel() << " width " << h.get_time_diff() << dispatch; 
  } // end ov loop over hits
  return ev;
}

void bx_precalib_muon_pedestals::end () {
  if(!b_has_data) return;

  db_run& run_info = bx_dbi::get ()->get_run ();
  const db_profile& profile_info = bx_dbi::get ()->get_profile();

  for(int i=0; i<constants::muon::channels; i++) { 
    int lg = constants::muon::channel_offset + i + 1;
    if (profile_info.logical_channel_description(lg) == db_profile::ordinary) {
      unsigned long entries = accumulate(pedestals[i]->begin(), pedestals[i]->end(), 0);
      if (entries) {
	unsigned long max = min_pedestal_width + max_element(pedestals[i]->begin(), pedestals[i]->end()) - pedestals[i]->begin();
	double mean = compute_mean(pedestals[i]->begin(), pedestals[i]->end(), 0.);
	double rms = compute_rms(pedestals[i]->begin(), pedestals[i]->end(), 0., mean);
	mean += min_pedestal_width;

	run_info.set_muon_precalib_pedestal(i, mean, this);	
	get_message(bx_message::info) << "mch " << i << " entries " << entries << " max " << max << " mean " << mean << " rms " << rms << dispatch;
      }
      else {
	run_info.set_muon_precalib_pedestal(i, 0, this);	
	get_message(bx_message::warn) << "mch " << i << " pedestal not available" << dispatch;
      }	
    }
    else 
      get_message(bx_message::info) << "mch " << i << " is not ordinary channel" << dispatch;

    delete pedestals[i];
  } /* end of loop on channels */

  get_message(bx_message::debug) << "end" << dispatch;
}  

/*
 * $Log: bx_precalib_muon_pedestals.cc,v $
 * Revision 1.8  2004/06/01 11:36:27  ddangelo
 * updated to new constants syntax
 *
 * Revision 1.7  2004/05/20 15:45:56  ddangelo
 *  check on has_data introduced
 *  check on chann descr implemented with db data (db_profile for the moment)
 *  instead of static event members.
 *  computetion done with algotithms instead of c like math.
 *  Data stored in a std::vector* (histo use).
 *
 * Revision 1.5  2004/04/28 11:04:49  ddangelo
 * Modules adapted to call the right const/nonconst event portion getter
 *
 * Revision 1.4  2004/04/28 10:48:21  ddangelo
 * Pedestal calculation reworked.
 *
 * Revision 1.3  2004/04/27 09:46:50  ddangelo
 * modifications to match new event structure.
 * Other debugging
 *
 * Revision 1.1  2004/04/20 11:36:25  ddangelo
 * added 2 modules for outer muon precalibration
 *
 *
 */
