/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 * based on previous work by Razeto&Pallavicini
 *
 * $Id: bx_writer.cc,v 1.17 2004/07/13 14:52:11 ddangelo Exp $
 *
 * Implementation of bx_event_writer
 *
 */
#include "constants.hh"
#include "bx_writer.hh"
#include "BxEvent.hh"
#include "bx_dbi.hh"
//class bx_reco_event;
#include "bx_reco_event.hh"
#include "bx_root_barn.hh"

#include <TFile.h>
#include <TTree.h>
#include <iostream>
#include <sstream>
#include <iomanip>

// ctor
bx_event_writer::bx_event_writer (): bx_base_module("bx_event_writer", bx_base_module::writer) {
}

// module interface
void bx_event_writer::begin () {
  // get parameters: 1.file name
  std::string file_name = "auto";
  if (check_parameter ("file_name")) 
    file_name = get_parameter ("file_name").get_string ();
  if(file_name == "auto") {
    std::ostringstream fn_stream;
    fn_stream << "Run" << std::setw(6) << std::setfill('0') << bx_dbi::get()->get_current_run_number() << ".root";
    file_name = fn_stream.str();
  }

  // get parameters: 2.overwrite
  bool overwrite = false;
  if (check_parameter ("file_overwrite")) 
    overwrite = get_parameter ("file_overwrite").get_int ();

  // open root file
  if (overwrite) 
    p_root_file = new TFile (file_name.c_str (), "RECREATE");
  else {
    p_root_file = new TFile (file_name.c_str (), "CREATE");
    if (p_root_file->IsZombie ()) 
      get_message (bx_message::critic) << "root file " << file_name << " already exists" << dispatch;
  }
  get_message (bx_message::info) << "root file " << file_name << " successfully opened" << dispatch;

  // get parameters: 3.buffer_size
  int buffer_size = 32000;
  if (check_parameter ("buffer_size"))
    buffer_size = get_parameter ("buffer_size").get_int();

  // get parameters: 4.splitting_level
  int splitting_level = 99;
  if (check_parameter ("splitting_level"))
    splitting_level = get_parameter ("splitting_level").get_int();

  bx_write_opts opts;
  m_parse_options(opts);

  // create BxEvent
  p_root_event = new BxEvent(opts);
  // create and branch TTree
  p_root_tree = new TTree ("bxtree", "Borexino Events Tree (generated by echidna)");
  p_root_tree->Branch ("events", "BxEvent", &p_root_event, buffer_size, splitting_level);
  get_message (bx_message::info) << "root tree successfully created and branched" << dispatch;
 
}

bx_reco_event* bx_event_writer::doit (bx_reco_event *ev) {
  // We cannot use new/delete for BxEvent obj (see comment in BxEvent.hh)
  *p_root_event = *ev;
  p_root_tree->Fill ();  

  if(!(ev->get_event_number()%1000)) 
    get_message(bx_message::info) << "filling ev: " << ev->get_event_number() << dispatch;  
  
  return ev;
}

void bx_event_writer::end () {
  if (p_root_file) {
    p_root_tree->Write ();
    if (p_root_event) delete p_root_event;
    if (p_root_tree) delete p_root_tree;
    bx_root_barn::get()->write(p_root_file);
    p_root_file->Close ();
    delete p_root_file;
    get_message (bx_message::info) << "Root file closed." << dispatch;
  }
}

// helper method to retrieve user parameters and fill opt 
void bx_event_writer::m_parse_options(bx_write_opts &opt) {
  opt.laben.raw       = check_parameter ("write_laben_raw")       ? bool( get_parameter ("write_laben_raw").get_int () )       : false;
  opt.laben.decoded   = check_parameter ("write_laben_decoded")   ? bool( get_parameter ("write_laben_decoded").get_int () )   : false;
  opt.laben.clustered = check_parameter ("write_laben_clustered") ? bool( get_parameter ("write_laben_clustered").get_int () ) : false;
  opt.global.split    = check_parameter ("write_global_split")    ? bool( get_parameter ("write_global_split").get_int () )    : false;
  opt.global.rec      = check_parameter ("write_global_rec")      ? bool( get_parameter ("write_global_rec").get_int () )      : false;
  opt.muon.raw        = check_parameter ("write_muon_raw")        ? bool( get_parameter ("write_muon_raw").get_int () )        : false;
  opt.muon.decoded    = check_parameter ("write_muon_decoded")    ? bool( get_parameter ("write_muon_decoded").get_int () )    : false;
  opt.fadc.raw        = check_parameter ("write_fadc_raw")        ? bool( get_parameter ("write_fadc_raw").get_int () )        : false;
  opt.fadc.decoded    = check_parameter ("write_fadc_decoded")    ? bool( get_parameter ("write_fadc_decoded").get_int () )    : false;
  opt.mctruth.hits    = check_parameter ("write_mctruth_hits")    ? bool( get_parameter ("write_mctruth_hits").get_int () )    : false;
}

/*
 * $Log: bx_writer.cc,v $
 * Revision 1.17  2004/07/13 14:52:11  ddangelo
 * added a parameter for mctruth
 *
 * Revision 1.16  2004/06/07 15:01:00  ddangelo
 * debugging
 *
 * Revision 1.15  2004/06/07 12:57:23  ddangelo
 * fixed some dependencies
 *
 * Revision 1.14  2004/06/03 14:59:36  ddangelo
 * new/delete on BxEvent replaced with operator=().
 * This should solve the problem of TTree::Fill() caching the addresses of TClonesArray.
 * BxEvent still in tmp state.
 *
 * Revision 1.13  2004/05/31 22:04:07  ddangelo
 * added support for automatic file names
 *
 * Revision 1.12  2004/05/31 11:20:42  ddangelo
 * bug tmp commented out.
 * call to bx_root_barn::write() reintroduced
 *
 * Revision 1.11  2004/05/30 11:58:14  ddangelo
 * Redisign to use new Root file classes.
 * Filler visitor removed.
 * User control over TTree::Branch() parameters introduced.
 * User parameter control over hit lists writing introduced.
 *
 * Revision 1.9  2004/04/18 09:15:57  razeto
 * Fixed indentation
 *
 * Revision 1.8  2004/04/16 15:02:22  pallas
 * Start developing of root event structure
 * Not defined yet, but preprared infrastructure
 *
 * Revision 1.7  2004/04/06 12:42:17  razeto
 * Added module role to base_module, and the code to handle everywhere. Some other minor fixes
 *
 * Revision 1.6  2004/04/03 09:20:35  razeto
 * Added messenger
 *
 * Revision 1.5  2004/03/21 18:59:41  razeto
 * Some cosmetic changes
 *
 * Revision 1.4  2004/03/21 18:53:01  razeto
 * Some cosmetic changes
 *
 * Revision 1.3  2004/03/20 18:59:26  pallas
 * Change counters
 *
 * Revision 1.2  2004/03/20 18:55:02  pallas
 * Debugging framework
 *
 * Revision 1.1  2004/03/20 17:39:45  pallas
 * First release
 *
 * Revision 1.1  2004/03/20 14:16:47  razeto
 * Added sources
 *
 */
