/* BOREXINO Reconstruction program
 *
 * Author: Davide D'Angelo <davide.dangelo@lngs.infn.it>
 * based on original design by Marco Pallavicini
 *
 * $Id: BxEvent.hh,v 1.13 2004/08/31 13:28:41 ddangelo Exp $
 *
 * This is the event stored into root TTree
 * Names match ROOT style and not echidna style.
 * 
 */

#ifndef _BXEVENT_HH
#define _BXEVENT_HH

#include <TObject.h>
#include <TClass.h>
#include <TClonesArray.h>
#include <vector>

// struct of flags for write options
struct bx_write_opts {
  struct {
    bool raw;
    bool decoded;
    bool clustered;
  } laben;
  struct {
    bool raw;
    bool decoded;
  } muon;
  struct {
    bool raw;
    bool decoded;
  } fadc;
  struct {
    bool split;
    bool rec;
  } global;
  struct {
    bool hits;
  } mctruth;
};

#if !defined(_ECHIDNA_ROOTLIB_) && !defined(__CINT__)
class bx_reco_event;
class bx_trigger_event;
class bx_laben_event;
class bx_laben_raw_hit;
class bx_laben_decoded_hit;
class bx_laben_clustered_hit;
class bx_laben_cluster;
class bx_muon_event;
class bx_muon_raw_hit;
class bx_muon_decoded_hit;
class bx_fadc_event;
class bx_global_event;
class bx_global_split_fragment;
class bx_global_split_hit;
class bx_global_rec_fragment;
class bx_global_rec_hit;
class bx_global_psd_fragment;
class bx_mctruth_hit;
class bx_mctruth_frame;
class bx_mctruth_event;
#endif



/*********** BxTrigger *************/
class BxTrigger : public TObject {
  public:
    BxTrigger() { Class()->IgnoreTObjectStreamer(); }
#if !defined(_ECHIDNA_ROOTLIB_) && !defined(__CINT__)
    virtual void operator=(const bx_trigger_event&);
#endif
    virtual ~BxTrigger ();

    // getters
    UChar_t GetTrgType () const { return trgtype; }
    ULong_t GetTrgTime() const { return trgtime; }

    virtual Bool_t IsFolder() { return kTRUE; }

  private:
    UChar_t trgtype;          // trigger type
    ULong_t trgtime;          // trigger time

    ClassDef (BxTrigger, 1)
};


/*********** BxLaben *************/
class BxLabenRawHit: public TObject {
  public:
    BxLabenRawHit () { Class()->IgnoreTObjectStreamer(); }
#if !defined(_ECHIDNA_ROOTLIB_) && !defined(__CINT__)
    BxLabenRawHit (const bx_laben_raw_hit&);
#endif
    virtual ~BxLabenRawHit ();
 
    UShort_t GetLg () const { return lg; }
    UChar_t GetTime1 () const { return time1; }
    UChar_t GetTime2 () const { return time2; }
    UShort_t GetGray () const { return gray; }
    UChar_t GetBase () const { return base; }
    UChar_t GetPeak () const { return peak; }
  
    virtual Bool_t IsFolder() { return kTRUE; }

  private:
    UShort_t lg;
    UChar_t  time1, time2;
    UShort_t gray;
    UChar_t  base, peak;

    ClassDef (BxLabenRawHit, 1)
};

class BxLabenDecodedHit: public TObject {
  public:
    BxLabenDecodedHit () { Class()->IgnoreTObjectStreamer(); }
#if !defined(_ECHIDNA_ROOTLIB_) && !defined(__CINT__)
    BxLabenDecodedHit (const bx_laben_decoded_hit&);
#endif
    virtual ~BxLabenDecodedHit ();
 
    UShort_t GetLg () const { return lg; }
    Double_t GetRawTime   () const { return raw_time; }
    Float_t  GetTimeError () const { return time_error; }
    Float_t  GetD80       () const { return d80; }
    Bool_t   IsOut        () const { return is_out; }
    Float_t  GetRawCharge () const { return raw_charge; }
    Float_t  GetCharge    () const { return charge; }

    virtual Bool_t IsFolder() { return kTRUE; }

  private:
    UShort_t lg;
    Double_t raw_time;
    Float_t  time_error;
    Float_t  d80;
    Bool_t   is_out;
    Float_t raw_charge; // bins (without pileup corr.)
    Float_t charge;  // photoelectrons

    ClassDef (BxLabenDecodedHit, 1)
};

class BxLabenClusteredHit: public TObject {
  public:
    BxLabenClusteredHit () { Class()->IgnoreTObjectStreamer(); }
#if !defined(_ECHIDNA_ROOTLIB_) && !defined(__CINT__)
    BxLabenClusteredHit (const bx_laben_clustered_hit&);
#endif
    virtual ~BxLabenClusteredHit ();
 
    Double_t GetTime () const { return time; }

    virtual Bool_t IsFolder() { return kTRUE; }

  private:
    Double_t time;

    ClassDef (BxLabenClusteredHit, 1)
};

class BxLabenCluster: public TObject {
  public:
    BxLabenCluster () { Class()->IgnoreTObjectStreamer(); 
    hits = new TClonesArray("BxLabenClusteredHit", 0); }
#if !defined(_ECHIDNA_ROOTLIB_) && !defined(__CINT__)
    BxLabenCluster (const bx_laben_cluster&);
#endif
    virtual ~BxLabenCluster ();

    // getters
  //    const TClonesArray* GetHits     () const { return hits; }
    Int_t GetNHits       () const { return nhits; }
    Int_t GetNpe         () const { return npe; }
    Int_t GetNpmt        () const { return npmt; }
    Int_t GetNpmtConc    () const { return npmt_conc; }
    Float_t GetStartTime () const { return start_time; }
    Float_t GetMeanTime  () const { return mean_time; }

    virtual Bool_t IsFolder() { return kTRUE; }

  private:
    Int_t nhits;
    Int_t npe;
    Int_t npmt;
    Int_t npmt_conc;
    Float_t start_time;
    Float_t mean_time;
    TClonesArray *hits; //->

    ClassDef (BxLabenCluster, 1)
};


class BxLaben: public TObject {
public:
  BxLaben() { Class()->IgnoreTObjectStreamer();
              raw_hits = new TClonesArray("BxLabenRawHit", 0); 
              decoded_hits = new TClonesArray("BxLabenDecodedHit", 0);
              clusters = new TClonesArray("BxLabenCluster", 0); }
#if !defined(_ECHIDNA_ROOTLIB_) && !defined(__CINT__)
  BxLaben(bx_write_opts&);
    virtual void  operator=(const bx_laben_event&);
#endif
    virtual ~BxLaben ();

    // getters
    const TClonesArray* GetRawHits     () const { return raw_hits; }
    const TClonesArray* GetDecodedHits () const { return decoded_hits; }
    const TClonesArray* GetClusters    () const { return clusters; }

    Bool_t HasRaw       () const { return has_raw; }
    Bool_t HasDecoded   () const { return has_decoded; }
    Bool_t HasClustered () const { return has_clustered; }

    Double_t GetTriggerTime () const { return trigger_time; }
    Double_t GetLaserTime   () const { return laser_time; }

    virtual Bool_t IsFolder() { return kTRUE; }

  private:
    Double_t trigger_time;
    Double_t laser_time;
    Bool_t has_raw;
    Bool_t has_decoded;
    Bool_t has_clustered;
    TClonesArray *raw_hits;  //->
    TClonesArray *decoded_hits;  //->
    TClonesArray *clusters;  //->

    ClassDef (BxLaben, 1)
};


/*********** BxMuon *************/
class BxMuonRawHit: public TObject {
  public:
    BxMuonRawHit () { Class()->IgnoreTObjectStreamer(); }
#if !defined(_ECHIDNA_ROOTLIB_) && !defined(__CINT__)
    BxMuonRawHit (const bx_muon_raw_hit&);
#endif
    virtual ~BxMuonRawHit ();
 
    Float_t  GetLeadTime  () const { return lead_time; }
    Float_t  GetTrailTime () const { return trail_time; }
    UShort_t GetMch       () const { return mch; }

    virtual Bool_t IsFolder () { return kTRUE; }

  private:
    UShort_t mch;       // muon channel (0 based)
    ULong_t  lead_time;  // lead time (TDC ticks)
    ULong_t  trail_time; // trail time (TDC ticks)

    ClassDef (BxMuonRawHit, 1)
};

class BxMuonDecodedHit: public TObject {
  public:
    BxMuonDecodedHit () { Class()->IgnoreTObjectStreamer(); }
#if !defined(_ECHIDNA_ROOTLIB_) && !defined(__CINT__)
    BxMuonDecodedHit (const bx_muon_decoded_hit&);
#endif
    virtual ~BxMuonDecodedHit ();
 
    Float_t  GetTime   () const { return time; }
    Float_t  GetCharge () const { return charge; }
    UShort_t GetMch    () const { return mch; }

    virtual Bool_t IsFolder () { return kTRUE; }

  private:
    UShort_t mch;
    Float_t  time;   // time after gate start (neutrino) or after led ref (laser); ns
    Float_t  charge; // Charge (ns)

    ClassDef (BxMuonDecodedHit, 1)
};

class BxMuon: public TObject {
  public:
    BxMuon() { Class()->IgnoreTObjectStreamer(); 
    //               raw_hits = new TClonesArray("BxMuonRawHit", 0);  
               decoded_hits = new TClonesArray("BxMuonDecodedHit", 0); }
#if !defined(_ECHIDNA_ROOTLIB_) && !defined(__CINT__)
    BxMuon(bx_write_opts&);
    virtual void operator=(const bx_muon_event&);
#endif
    virtual ~BxMuon ();

    // getters
  //    const TClonesArray* GetRawHits     () const { return raw_hits; }
    const TClonesArray* GetDecodedHits () const { return decoded_hits; }

    Bool_t HasRaw     () const { return has_raw; }
    Bool_t HasDecoded () const { return has_decoded; }

    virtual Bool_t IsFolder() { return kTRUE; }

  private:
    Bool_t has_raw;
    Bool_t has_decoded;
    std::vector<BxMuonRawHit> raw_hits;  //->
    TClonesArray* decoded_hits;  //->

    ClassDef (BxMuon, 1)
};


/*********** BxFadc *************/
class BxFadc: public TObject {
  public:
    BxFadc() {  Class()->IgnoreTObjectStreamer(); 
                /* raw_wins = new TClonesArray("BxFadcRawWin", 0); */ }
#if !defined(_ECHIDNA_ROOTLIB_) && !defined(__CINT__)
    BxFadc(bx_write_opts&);
    virtual void operator=(const bx_fadc_event&);
#endif
    virtual ~BxFadc ();

    // getters
    //const TClonesArray* GetRawWins () const { return raw_wins; }
    Float_t GetDummy  () const { return dummy; }

    virtual Bool_t IsFolder() { return kTRUE; }

  private:
    //TClonesArray *raw_wins;
    Float_t dummy;

    ClassDef (BxFadc, 1)
};


/*********** BxGlobal *************/
class BxGlobalSplitHit: public TObject {
  public:
    BxGlobalSplitHit () { Class()->IgnoreTObjectStreamer(); }
#if !defined(_ECHIDNA_ROOTLIB_) && !defined(__CINT__)
    BxGlobalSplitHit (const bx_global_split_hit&);
#endif
    virtual ~BxGlobalSplitHit ();

    // getters
    Float_t GetDummy  () const { return dummy; }

    virtual Bool_t IsFolder() { return kTRUE; }

  private:
    Float_t dummy;

    ClassDef (BxGlobalSplitHit, 1)
};

class BxGlobalSplitFragment: public TObject {
  public:
    BxGlobalSplitFragment () { Class()->IgnoreTObjectStreamer(); 
                               /*hits = new TClonesArray("BxGlobalSplitHit", 0); */}
#if !defined(_ECHIDNA_ROOTLIB_) && !defined(__CINT__)
    BxGlobalSplitFragment (const bx_global_split_fragment&);
#endif
    virtual ~BxGlobalSplitFragment ();

    // getters
  //    const TClonesArray* GetHits     () const { return hits; }
    Float_t GetDummy  () const { return dummy; }

    virtual Bool_t IsFolder() { return kTRUE; }

  private:
    Float_t dummy;
  //    TClonesArray *hits; //->

    ClassDef (BxGlobalSplitFragment, 1)
};

class BxGlobalRecHit: public TObject {
  public:
    BxGlobalRecHit () { Class()->IgnoreTObjectStreamer(); }
#if !defined(_ECHIDNA_ROOTLIB_) && !defined(__CINT__)
    BxGlobalRecHit (const bx_global_rec_hit&);
#endif
    virtual ~BxGlobalRecHit ();

    // getters
    Float_t GetDummy  () const { return dummy; }

    virtual Bool_t IsFolder() { return kTRUE; }

  private:
    Float_t dummy;

    ClassDef (BxGlobalRecHit, 1)
};

class BxGlobalRecFragment: public TObject {
  public:
    BxGlobalRecFragment () {  Class()->IgnoreTObjectStreamer(); 
                              /*hits = new TClonesArray("BxGlobalRecHit", 0); */}
#if !defined(_ECHIDNA_ROOTLIB_) && !defined(__CINT__)
    BxGlobalRecFragment (const bx_global_rec_fragment&, const bx_global_psd_fragment&, bool has_hits);
#endif
    virtual ~BxGlobalRecFragment ();

    // getters
  //    const TClonesArray* GetHits     () const { return hits; }
    Float_t GetDummy  () const { return dummy; }

    virtual Bool_t IsFolder() { return kTRUE; }

  private:
    // This should include rec & psd info
    Float_t dummy;
  //    TClonesArray *hits; //->

    ClassDef (BxGlobalRecFragment, 1)
};

class BxGlobal: public TObject {
  public:
    BxGlobal() { Class()->IgnoreTObjectStreamer(); 
                 split_fragments = new TClonesArray("BxGlobalSplitFragment", 0); 
                 rec_fragments   = new TClonesArray("BxGlobalRecFragment", 0); }
#if !defined(_ECHIDNA_ROOTLIB_) && !defined(__CINT__)
    BxGlobal(bx_write_opts&);
    virtual void  operator=(const bx_global_event&);
#endif
    virtual ~BxGlobal ();

    // getters
    const TClonesArray* GetSplitFragments () const { return split_fragments; }
    const TClonesArray* GetRecFragments   () const { return rec_fragments; }

    Bool_t HasSplit () const { return has_split; }
    Bool_t HasRecHits   () const { return has_rec_hits; }

    virtual Bool_t IsFolder() { return kTRUE; }

  private:
    Bool_t has_split;
    Bool_t has_rec_hits;
    TClonesArray *split_fragments;  //->
    TClonesArray *rec_fragments;  //->

    ClassDef (BxGlobal, 1)
};


/*********** BxMcTruth *************/
class BxMcTruthHit: public TObject {
  public:
    BxMcTruthHit () { Class()->IgnoreTObjectStreamer(); }
#if !defined(_ECHIDNA_ROOTLIB_) && !defined(__CINT__)
    BxMcTruthHit (const bx_mctruth_hit&);
#endif
    virtual ~BxMcTruthHit ();
 
    // getters
    UShort_t GetLg   () const { return lg; }
    Float_t  GetTime () const { return time; }

    virtual Bool_t IsFolder () { return kTRUE; }

  private:
    UShort_t lg;       // logical channel
    Float_t  time;     // tracking time (ns)

    ClassDef (BxMcTruthHit, 1)
};

class BxMcTruthFrame: public TObject {
  public:
    BxMcTruthFrame () {  Class()->IgnoreTObjectStreamer(); 
                         /*hits = new TClonesArray("BxMcTruthHit", 0);*/ }
#if !defined(_ECHIDNA_ROOTLIB_) && !defined(__CINT__)
    BxMcTruthFrame (const bx_mctruth_frame&, bool has_hits);
#endif
    virtual ~BxMcTruthFrame ();
 
    // getters
    Long_t   GetGenebNevento     () const { return geneb_nevento; }
    Long_t   GetGenebJdecad      () const { return geneb_jdecad; }
    Long_t   GetGenebIsotopo     () const { return geneb_isotopo; }
    Long_t   GetGenebNumdo       () const { return geneb_numdo; }
    Double_t GetGenebDtempoass   () const { return geneb_dtempoass; }
    Float_t  GetGenebEee         () const { return geneb_eee; }
    Float_t  GetGenebXi          () const { return geneb_xi; }
    Float_t  GetGenebYi          () const { return geneb_yi; }
    Float_t  GetGenebZi          () const { return geneb_zi; }
    Short_t  GetGenebItipo       () const { return geneb_itipo; }
    Short_t  GetGenebNmax        () const { return geneb_nmax; }
    Short_t  GetGenebNumcer      () const { return geneb_numcer; }
    UShort_t GetTrackingNph      () const { return tracking_nph; }
    Float_t  GetGenebPlight (int i) const { return geneb_plight[i]; } 

  //    const TClonesArray* GetHits  () const { return hits; }

    virtual Bool_t IsFolder () { return kTRUE; }

  private:
    Long_t   geneb_nevento;
    Long_t   geneb_jdecad;
    Long_t   geneb_isotopo;
    Long_t   geneb_numdo;
    Double_t geneb_dtempoass;
    Float_t  geneb_eee;
    Float_t  geneb_xi;
    Float_t  geneb_yi;
    Float_t  geneb_zi;
    Short_t  geneb_itipo;
    Short_t  geneb_nmax;
    Short_t  geneb_numcer;
    UShort_t tracking_nph;
    Float_t  geneb_plight[3];

  //    TClonesArray *hits; //->
    ClassDef (BxMcTruthFrame, 1)
};

class BxMcTruth: public TObject {
  public:
    BxMcTruth() { Class()->IgnoreTObjectStreamer(); 
                  frames = new TClonesArray("BxMcTruthFrame", 0); }
#if !defined(_ECHIDNA_ROOTLIB_) && !defined(__CINT__)
    BxMcTruth (bx_write_opts&);
    virtual void operator=(const bx_mctruth_event& e);
#endif
    virtual ~BxMcTruth ();

    // getters
    Bool_t HasHits     () const { return has_hits; }
    const TClonesArray* GetFrames () const { return frames; }

    Bool_t IsFolder() { return kTRUE; }

  private:
    Bool_t has_hits;
    TClonesArray *frames; //->

    ClassDef (BxMcTruth, 1)
};

/*********** BxEvent *************/
class BxEvent : public TObject {
  public:
    BxEvent() { Class()->IgnoreTObjectStreamer(); }
#if !defined(_ECHIDNA_ROOTLIB_) && !defined(__CINT__)
    BxEvent(bx_write_opts&);
    virtual void operator=(const bx_reco_event& ev);
#endif
    virtual ~BxEvent ();

    // event header getters
    Int_t GetRun   () const { return run; }
    Int_t GetEvNum () const { return evnum; }

    // subclass getters
    const BxTrigger& GetTrigger () const { return trigger; }
    const BxLaben&   GetLaben   () const { return laben; }
    const BxFadc&    GetFadc    () const { return fadc; }
    const BxMuon&    GetMuon    () const { return muon; }
    const BxGlobal&  GetGlobal  () const { return global; }
    const BxMcTruth& GetMcTruth () const { return mctruth; }

    virtual Bool_t IsFolder() { return kTRUE; }

  private:
    Int_t run;                 // run number
    Int_t evnum;               // event number (trigger ID)

    BxTrigger trigger;
    BxLaben   laben;
    BxMuon    muon;
    BxFadc    fadc;
    BxGlobal  global;
    BxMcTruth mctruth;

    ClassDef (BxEvent, 1)    
};


#endif

/*
 * $Log: BxEvent.hh,v $
 * Revision 1.13  2004/08/31 13:28:41  ddangelo
 * added charge to decoded hit
 *
 * Revision 1.12  2004/07/26 17:36:33  ddangelo
 * fBits and fUniqueID suppressed in ROOT file.
 *
 * Revision 1.11  2004/07/26 16:08:35  ddangelo
 * ClassDef macros moved at the end of class definitions.
 *
 * Revision 1.10  2004/07/25 16:41:43  ddangelo
 * added logical channel to laben decoded hits.
 *
 * Revision 1.9  2004/07/25 16:29:31  ddangelo
 * some development, not much active at the moment
 *
 * Revision 1.8  2004/07/13 14:50:48  ddangelo
 * added BxClusteredHit. Currently used due to the TClonesArray problem.
 *
 * Revision 1.7  2004/07/13 13:37:10  ddangelo
 * added McTruth and McTruthFrame to root event.
 * McTruthHit commented out for the moment, due to ROOT problems.
 * To be debugged.
 *
 * Revision 1.6  2004/07/07 15:45:26  ddangelo
 * added BxLabenCluster.
 * Some minor debugging.
 *
 * Revision 1.5  2004/06/25 14:46:39  ddangelo
 * fixed a bug.
 * removed a comment.
 *
 * Revision 1.4  2004/06/22 13:02:52  ddangelo
 * added laben laser and trigger time in BxLaben obj
 *
 * Revision 1.3  2004/06/07 17:14:03  ddangelo
 * conditional compile macros introduced.
 * Laben raw and decoded hit introduced.
 * Muon raw and decoded hit implemented with 2 different classes.
 *
 * Revision 1.2  2004/06/03 15:00:46  ddangelo
 * (Copy) constructor replaced by operator=() for every class.
 * Still in a tmp state.
 *
 * Revision 1.1  2004/05/30 11:54:48  ddangelo
 * A first working version of root file classes.
 * Not many physical variables yet;
 * Global Event still commented.
 * names match ROOT standards not echidna ones.
 * Makefile updated (file names).
 *
 *
 */
