#!/usr/bin/perl -w
#
# Author: Alessandro Razeto <Alessandro.Razeto@ge.infn.it>
#
# $Id: bx_repository.pl,v 1.7 2004/09/11 19:02:53 razeto Exp $
#
# Repository management utility: copy local run to a repository (via ftp) or
# copy run files from a remote repository to a local repository tree (which 
# can be used in echidna specifying the bx_event_reader.repository_url to
# file://local_dir)
#
#
use POSIX qw(strftime _exit);
use Pg;
use Time::ParseDate;
use Time::Local;
use Net::FTP;
use strict;

# Connection handlers
my $db_connection = 0;
my $ftp_connection = 0;
my $http_connection = 0;

# Defaults
my $do_nothing = 0;
my $rewrite_files = 0;

my $db_host = "bxdb";
my $db_port = 5432;
my $db_name = "daq_config";
my $db_user = "borex_guest";
my $db_password = "";

my $remote_repository_host = "bxmaster.lngs.infn.it";
$remote_repository_host = "bxmaster-data" if (`hostname` =~ /^bxbuild/);
my $remote_repository_ftp_path = "/bxstorage";
my $remote_repository_ftp_user = "storage";
my $remote_repository_ftp_password = "Data";
my $remote_repository_http_path = "/bxstorage";
my $remote_repository_http_user = "borex";
my $remote_repository_http_password = "b0rexin0";


while (defined $ARGV[0] and $ARGV[0] =~ /^-/) {
  $_ = shift;
  last if /^--/;
  if (/^-do_nothing$/) {
  $do_nothing = 1;
  } elsif (/^-rewrite_files$/) {
  $rewrite_files = 1;
  } elsif (/^-db_host$/) {
    $db_host = shift;
    test_opt ("-db_host", $db_host);
  } elsif (/^-db_name$/) {
    $db_name = shift;
    test_opt ("-db_name", $db_name);
  } elsif (/^-db_user$/) {
    $db_user = shift;
    test_opt ("-db_user", $db_user);
  } elsif (/^-db_password$/) {
    $db_password = shift;
    test_opt ("-db_password", $db_password);
  } elsif (/^-remote_repository_host$/) {
    $remote_repository_host = shift;
    test_opt ("-remote_repository_host", $remote_repository_host);
  } elsif (/^-remote_repository_ftp_path$/) {
    $remote_repository_ftp_path = shift;
    test_opt ("-remote_repository_ftp_path", $remote_repository_ftp_path);
  } elsif (/^-remote_repository_ftp_user$/) {
    $remote_repository_ftp_user = shift;
    test_opt ("-remote_repository_ftp_user", $remote_repository_ftp_user);
  } elsif (/^-remote_repository_ftp_password$/) {
    $remote_repository_ftp_password = shift;
    test_opt ("-remote_repository_ftp_password", $remote_repository_ftp_password);
  } elsif (/^-remote_repository_http_path$/) {
    $remote_repository_http_path = shift;
    test_opt ("-remote_repository_http_path", $remote_repository_http_path);
  } elsif (/^-remote_repository_http_user$/) {
    $remote_repository_http_user = shift;
    test_opt ("-remote_repository_http_user", $remote_repository_http_user);
  } elsif (/^-remote_repository_http_password$/) {
    $remote_repository_http_password = shift;
    test_opt ("-remote_repository_http_password", $remote_repository_http_password);
  } else {
    usage ($ARGV[0]);
  } 
}

my $mode = shift;
usage () if (!defined $mode || $mode =~ /^-/);

if ($mode =~ /^upload$/) {
  upload (@ARGV);
} elsif ($mode =~ /^download$/) {
  download (@ARGV);
} elsif ($mode =~ /^reorganize$/) {
  reorganize (@ARGV);
} elsif ($mode =~ /^check$/) {
  check (@ARGV);
} else { usage ($ARGV[0]); }



sub upload {
  my $file = shift;
  die "upload syntax: upload file/directory\n" unless (defined $file); 

  die "$file not found\n" unless (-e $file);

  my @file_list;
  if (-d $file) {
    @file_list = glob ("$file/Run*.out*");
    die "directory $file does not contain raw data files\n" unless (scalar @file_list);
  } elsif (-f $file) {
    push @file_list, $file;
  } else {
    die "unknown file type for $file\n";
  }

  connect_db ();
  connect_ftp ();

  foreach $file (@file_list) {
    if ($file =~ /(Run(\d{6})_\d{2}.out(.gz)?)/) {
      my $run_id = int $2;
      my $file_name = $1;
      unless (defined $3) {
        warn "file $file_name is not compressed\n";
        `gzip $file` unless ($do_nothing);
        push @file_list, "$file.gz";
        next;
      }
      my $start_time = get_run_start_time ($run_id);
      next unless (defined $start_time);
      my ($year_index, $week_index) = get_run_start_times ($start_time);
      copy_file_over_ftp ("$remote_repository_ftp_path/rawdata/$year_index/$week_index/", $file, $file_name);
    }
  }
}

sub download {
  my $local_repository = shift;
  my $run_list_pattern = join ("", @_);
  die "download syntax: download local_repository run_list\n" unless (defined $local_repository && defined $run_list_pattern); 

  unless (-d $local_repository) {
    mkdir $local_repository or die "Cannot create local repository $!\n";
  }
  unless (-d "$local_repository/rawdata") {
    mkdir "$local_repository/rawdata" or die "Cannot create $local_repository/rawdata $!\n";
  }

  chdir "$local_repository/rawdata" or die "Cannot chdir $local_repository/rawdata $!\n";
  connect_db ();

  my @run_list = extract_unique_run_list ($run_list_pattern);
  
  foreach my $run (@run_list) {
    my $slices = get_run_slices ($run);
    my $start_time = get_run_start_time ($run);
    next unless (defined $start_time);
    my ($year_index, $week_index) = get_run_start_times ($start_time);
    for (my $i = 0; $i < $slices; $i++) {
      my $file_name = sprintf ("Run%06d_%02d.out.gz", $run, $i + 1);
      my $file_url = "http://$remote_repository_host$remote_repository_http_path/rawdata/$year_index/$week_index/$file_name";
      unless (-d "$year_index") {
        mkdir "$year_index" or die "Cannot create $year_index $!\n"; 
      }
      unless (-d "$year_index/$week_index") {
        mkdir "$year_index/$week_index" or die "Cannot create $year_index/$week_index $!\n"; 
      }
      `wget -nv -O $year_index/$week_index/$file_name $file_url` if (!-f "$year_index/$week_index/$file_name" or (-f "$year_index/$week_index/$file_name" and $rewrite_files));
    }
  } 
}

sub reorganize {
  my $file = shift;
  my $local_repository = shift;
  die "reorganize syntax: reorganize file/directory local_reporitory\n" unless (defined $file && defined $local_repository); 

  die "$file not found\n" unless (-e $file);
  unless (-d $local_repository) {
    mkdir $local_repository or die "Cannot create local repository $!\n";
  }
  unless (-d "$local_repository/rawdata") {
    mkdir "$local_repository/rawdata" or die "Cannot create $local_repository/rawdata $!\n";
  }

  chdir "$local_repository/rawdata" or die "Cannot chdir $local_repository/rawdata $!\n";

  my @file_list;
  if (-d $file) {
    @file_list = glob ("$file/Run*.out*");
    die "directory $file does not contain raw data files\n" unless (scalar @file_list);
  } elsif (-f $file) {
    push @file_list, $file;
  } else {
    die "unknown file type for $file\n";
  }

  connect_db ();

  foreach $file (@file_list) {
    if ($file =~ /(Run(\d{6})_\d{2}.out(.gz)?)/) {
      my $run_id = int $2;
      my $file_name = $1;
      unless (defined $3) {
        warn "file $file_name is not compressed\n";
        `gzip $file` unless ($do_nothing);
        push @file_list, "$file.gz";
        next;
      }
      my $start_time = get_run_start_time ($run_id);
      next unless (defined $start_time);
      my ($year_index, $week_index) = get_run_start_times ($start_time);
      unless ($do_nothing) {
        mkdir "$year_index" or die "Cannot create $year_index $!\n" unless (-d "$year_index");
        mkdir "$year_index/$week_index" or die "Cannot create $year_index/$week_index $!\n" unless (-d "$year_index/$week_index"); 
        my $dest_file = "$year_index/$week_index/$file_name";
        `mv $file $dest_file` if (!-f $dest_file  or (-f $dest_file and $rewrite_files));
      } 
    }
  }
}

sub check {
  my $repository = shift;
  if ($repository =~ /^[0-9\-,]+$/) {
    unshift @_, $repository;
    $repository = "http://$remote_repository_host$remote_repository_http_path";
  }
  my $run_list_pattern = join ("", @_);
  die "check syntax: check [repository] run_list\n" unless (defined $repository && defined $run_list_pattern); 

  connect_db ();

  my @run_list = extract_unique_run_list ($run_list_pattern);
  
  foreach my $run (@run_list) {
    my $slices = get_run_slices ($run);
    my $start_time = get_run_start_time ($run);
    next unless (defined $start_time);
    my ($year_index, $week_index) = get_run_start_times ($start_time);
    for (my $i = 0; $i < $slices; $i++) {
      my $file_name = sprintf ("Run%06d_%02d.out.gz", $run, $i + 1);
      my $file_url = "$repository/rawdata/$year_index/$week_index/$file_name";
      my $found = 0;
      if ($file_url =~ /^http/) {
        my $ret = `wget -nv --spider $file_url 2>&1`;
        $found = 1 if ($ret =~ /200.OK/);
      } else {
        $found = 1 unless (-f $file_url);
      }
      print "Run $run slice $i not found on $repository\n" unless ($found);
    }
  } 
}

sub test_opt {
  my $opt_name = shift;
  my $opt_value = shift;
  die "$opt_name requires an argument ($opt_value)\n" if (!defined $opt_value || $opt_value =~ /^-/);
}

sub connect_db {
  if ($db_connection) {
    warn "Already connected to db\n";
    return;
  }
  $db_connection = Pg::connectdb("host=$db_host port=$db_port dbname=$db_name user=$db_user");
  die "Unable to connect to the database\n" if ($db_connection->status != PGRES_CONNECTION_OK);
}

sub get_last_run {
  my $query = "SELECT \"RunNumber\" FROM \"Run\" ORDER by \"RunNumber\" DESC;";
  my $result = $db_connection->exec($query);
  die "Error in query: " . $db_connection->errorMessage . "\n" unless ($result->resultStatus == PGRES_TUPLES_OK);
  return undef unless ($result->ntuples);
  my $run_number_pos = $result->fnumber ("\"RunNumber\"");
  my $run_number = $result->getvalue (0, $run_number_pos);

  return $run_number;
}

sub get_run_slices {
  my $run_id = shift;
  my $query = "SELECT \"NumberOfFiles\" FROM \"Run\" WHERE \"RunNumber\"=$run_id;";
  my $result = $db_connection->exec($query);
  die "Error in query: " . $db_connection->errorMessage . "\n" unless ($result->resultStatus == PGRES_TUPLES_OK);
  return undef unless ($result->ntuples);
  my $number_of_files_pos = $result->fnumber ("\"NumberOfFiles\"");
  my $number_of_files = $result->getvalue (0, $number_of_files_pos);
  return $number_of_files;
}

sub get_run_start_time {
  my $run_id = shift;
  my $query = "SELECT * FROM \"Run\" WHERE \"RunNumber\"=$run_id;";
  my $result = $db_connection->exec($query);
  die "Error in query: " . $db_connection->errorMessage . "\n" unless ($result->resultStatus == PGRES_TUPLES_OK);
  return undef unless ($result->ntuples);
  my $start_date_pos = $result->fnumber ("\"StartTime\"");
  my $start_date = $result->getvalue (0, $start_date_pos);

  $query = "SELECT EXTRACT(EPOCH FROM TIMESTAMP '$start_date')";
  $result = $db_connection->exec($query);
  die "Error in query: " . $db_connection->errorMessage . "\n" unless ($result->resultStatus == PGRES_TUPLES_OK);
  return undef unless ($result->ntuples == 1);
  my $start_time_pos = $result->fnumber ("date_part");
  my $start_time = $result->getvalue (0, $start_time_pos);
  
  return $start_time;
}

sub get_run_start_times {
  my $start_time = shift;
  my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday) = localtime $start_time;
  my $week_time = $start_time - ((($wday - 1) * 24 + $hour) * 60 + $min) * 60;
  my @tmp = localtime $week_time;
  my $week_index = strftime ("%b_%d", @tmp); 
  my $year_index = $tmp[5] + 1900;
  return ($year_index, $week_index);
}

sub connect_ftp {
  unless ($do_nothing) {
    $ftp_connection = Net::FTP->new ("$remote_repository_host", Debug => 0) or die "Cannot connect to $remote_repository_host: $@\n";
    $ftp_connection->login ($remote_repository_ftp_user, $remote_repository_ftp_password) or die "Cannot login: " . $ftp_connection->message . "\n";
  }
}

sub copy_file_over_ftp {
  my $dir = shift;
  my $file = shift;
  my $remote_file = shift;
  
  $ftp_connection->cwd ("/") or die "Cannot chdir / " . $ftp_connection->message . "\n";
  unless ($do_nothing) {
    unless ($ftp_connection->cwd ($dir)) {
      $ftp_connection->mkdir ($dir, 1) or $ftp_connection->mkdir ($dir, 1) or die "Cannot mkdir $dir: " . $ftp_connection->message . "\n";
      $ftp_connection->cwd ($dir) or die "Cannot chdir $dir " . $ftp_connection->message . "\n";
    }
    my $size = $ftp_connection->size ($remote_file);
    if (!$size or ($size and $rewrite_files)) {
      $ftp_connection->put ($file) or die "Cannot put $file " . $ftp_connection->message . "\n";
      print "copied $file in ftp://$remote_repository_host$dir\n";
    } else {
      #my $local_size = -s $file;
      #warn "file $file has different size than ftp://$remote_repository_host$dir/$remote_file ($local_size != $size)\n" unless ($size == $local_size);
      print "ignored $file already present at ftp://$remote_repository_host$dir$remote_file\n";
    }
  }
}

sub extract_unique_run_list {
  my $run_list_pattern = shift;
  my @run_list;
  my @fields = split (/,/, $run_list_pattern);
  foreach my $field (@fields) {
    if ($field =~ /^(\d+)?-(\d+)?$/) {
      my $low_limit = $1;
      my $high_limit = $2;
      $low_limit = 1200 unless (defined $low_limit);
      $high_limit = get_last_run () unless (defined $high_limit);
      warn "Ignoring range $field ($high_limit < $low_limit)\n" if ($high_limit < $low_limit);
      for (my $i = $low_limit; $i <= $high_limit; $i++) { push @run_list, $i; }
    } elsif ($field =~ /^(\d+)$/) {
      push @run_list, $1;
    } else {
      die "wrong range format for field \"$field\"\n";
    }
  }

  my @tmp_list = sort @run_list;
  @run_list = ();
  my $old_field = -1;
  foreach my $field (@tmp_list) {
    next if ($field == $old_field);
    push @run_list, $field;
    $old_field = $field;
  }
  
  return @run_list;
}

sub usage {
  die "Sorry not usage help yet\n";
}
